@@ -1,22 +1,27 @@
 #!/bin/bash
 # Copyright 1999-2009 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: ebuild.sh 14543 2009-10-10 02:16:23Z zmedico $
+# $Id: ebuild.sh 15576 2010-03-02 20:16:29Z zmedico $
 
 PORTAGE_BIN_PATH="${PORTAGE_BIN_PATH:-/usr/lib/portage/bin}"
 PORTAGE_PYM_PATH="${PORTAGE_PYM_PATH:-/usr/lib/portage/pym}"
 
+# Ensure that /dev/std* streams have appropriate sandbox permission for
+# bug #288863. This can be removed after sandbox is fixed and portage
+# depends on the fixed version.
+export SANDBOX_WRITE="${SANDBOX_WRITE:+${SANDBOX_WRITE}:}/dev/stdout:/dev/stderr"
+export SANDBOX_READ="${SANDBOX_READ:+${SANDBOX_READ}:}/dev/stdin"
+
 # Don't use sandbox's BASH_ENV for new shells because it does
 # 'source /etc/profile' which can interfere with the build
 # environment by modifying our PATH.
 unset BASH_ENV
 
-# sandbox's bashrc sources /etc/profile which unsets ROOTPATH,
-# so we have to back it up and restore it.
-if [ -n "${PORTAGE_ROOTPATH}" ] ; then
-	export ROOTPATH=${PORTAGE_ROOTPATH}
-	unset PORTAGE_ROOTPATH
-fi
+# Avoid sandbox violations in temporary directories.
+for x in TEMP TMP TMPDIR ; do
+	[[ -n ${!x} ]] && export SANDBOX_WRITE="${SANDBOX_WRITE:+${SANDBOX_WRITE}:}${!x}"
+done
+unset x
 
 ROOTPATH=${ROOTPATH##:}
 ROOTPATH=${ROOTPATH%%:}
@@ -73,10 +78,6 @@ unset GZIP BZIP BZIP2 CDPATH GREP_OPTIONS GREP_COLOR GLOBIGNORE
 
 source "${PORTAGE_BIN_PATH}/isolated-functions.sh"  &>/dev/null
 
-# Set IMAGE for minimal backward compatibility with
-# overlays or user's bashrc, but don't export it.
-[ "${EBUILD_PHASE}" == "preinst" ] && IMAGE=${D}
-
 [[ $PORTAGE_QUIET != "" ]] && export PORTAGE_QUIET
 
 # the sandbox is disabled by default except when overridden in the relevant stages
@@ -142,7 +143,7 @@ useq() {
 		# Skip this for older EAPIs since lots of ebuilds/eclasses
 		# have stuff in global scope that really belongs somewhere
 		# like pkg_setup or src_configure.
-		if [[ -n $EAPI ]] && ! hasq "$EAPI" 0 1 2 ; then
+		if [[ -n $EAPI ]] && ! hasq "$EAPI" 0 1 2 3 3_pre2 ; then
 			die "use() called during invalid phase: $EBUILD_PHASE"
 		fi
 
@@ -160,13 +161,16 @@ useq() {
 	fi
 }
 
+# Return true if given package is installed. Otherwise return false.
+# Takes single depend-type atoms.
 has_version() {
 	if [ "${EBUILD_PHASE}" == "depend" ]; then
 		die "portageq calls (has_version calls portageq) are not allowed in the global scope"
 	fi
-	# return shell-true/shell-false if exists.
-	# Takes single depend-type atoms.
-	PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
+
+	# Set EPYTHON variable as empty so that portageq doesn't try
+	# to use potentially unsupported version of Python.
+	EPYTHON= PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
 	"${PORTAGE_BIN_PATH}"/portageq has_version "${ROOT}" "$1"
 	local retval=$?
 	case "${retval}" in
@@ -186,7 +190,10 @@ portageq() {
 	if [ "${EBUILD_PHASE}" == "depend" ]; then
 		die "portageq calls are not allowed in the global scope"
 	fi
-	PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
+
+	# Set EPYTHON variable as empty so that portageq doesn't try
+	# to use potentially unsupported version of Python.
+	EPYTHON= PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
 	"${PORTAGE_BIN_PATH}/portageq" "$@"
 }
 
@@ -196,13 +203,16 @@ portageq() {
 # ----------------------------------------------------------------------------
 
 
+# Returns the best/most-current match.
+# Takes single depend-type atoms.
 best_version() {
 	if [ "${EBUILD_PHASE}" == "depend" ]; then
 		die "portageq calls (best_version calls portageq) are not allowed in the global scope"
 	fi
-	# returns the best/most-current match.
-	# Takes single depend-type atoms.
-	PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
+
+	# Set EPYTHON variable as empty so that portageq doesn't try
+	# to use potentially unsupported version of Python.
+	EPYTHON= PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
 	"${PORTAGE_BIN_PATH}/portageq" 'best_version' "${ROOT}" "$1"
 	local retval=$?
 	case "${retval}" in
@@ -225,15 +235,8 @@ use_with() {
 		return 1
 	fi
 
-	local UW_SUFFIX=""
-	if [ ! -z "${3}" ]; then
-		UW_SUFFIX="=${3}"
-	fi
-
-	local UWORD="$2"
-	if [ -z "${UWORD}" ]; then
-		UWORD="$1"
-	fi
+	local UW_SUFFIX=${3:+=$3}
+	local UWORD=${2:-$1}
 
 	if useq $1; then
 		echo "--with-${UWORD}${UW_SUFFIX}"
@@ -250,15 +253,8 @@ use_enable() {
 		return 1
 	fi
 
-	local UE_SUFFIX=""
-	if [ ! -z "${3}" ]; then
-		UE_SUFFIX="=${3}"
-	fi
-
-	local UWORD="$2"
-	if [ -z "${UWORD}" ]; then
-		UWORD="$1"
-	fi
+	local UE_SUFFIX=${3:+=$3}
+	local UWORD=${2:-$1}
 
 	if useq $1; then
 		echo "--enable-${UWORD}${UE_SUFFIX}"
@@ -269,7 +265,19 @@ use_enable() {
 }
 
 register_die_hook() {
-	export EBUILD_DEATH_HOOKS="${EBUILD_DEATH_HOOKS} $*"
+	local x
+	for x in $* ; do
+		hasq $x $EBUILD_DEATH_HOOKS || \
+			export EBUILD_DEATH_HOOKS="$EBUILD_DEATH_HOOKS $x"
+	done
+}
+
+register_success_hook() {
+	local x
+	for x in $* ; do
+		hasq $x $EBUILD_SUCCESS_HOOKS || \
+			export EBUILD_SUCCESS_HOOKS="$EBUILD_SUCCESS_HOOKS $x"
+	done
 }
 
 # Ensure that $PWD is sane whenever possible, to protect against
@@ -492,7 +500,7 @@ econf() {
 		fi
 
 		# EAPI=3 adds --disable-dependency-tracking to econf
-		if ! hasq "$EAPI" 0 1 2 ; then
+		if ! hasq "$EAPI" 0 1 2 3 3_pre2 ; then
 			set -- --disable-dependency-tracking "$@"
 		fi
 
@@ -550,7 +558,7 @@ einstall() {
 		CONF_LIBDIR="${!LIBDIR_VAR}"
 	fi
 	unset LIBDIR_VAR
-	if [ -n "${CONF_LIBDIR}" ] && [ "${CONF_PREFIX:-unset}" != "unset" ]; then
+	if [ -n "${CONF_LIBDIR}" ] && [ "${CONF_PREFIX:+set}" = set ]; then
 		EI_DESTLIBDIR="${D}/${CONF_PREFIX}/${CONF_LIBDIR}"
 		EI_DESTLIBDIR="$(strip_duplicate_slashes ${EI_DESTLIBDIR})"
 		LOCAL_EXTRA_EINSTALL="libdir=${EI_DESTLIBDIR} ${LOCAL_EXTRA_EINSTALL}"
@@ -639,7 +647,7 @@ _eapi2_src_compile() {
 	fi
 }
 
-_eapi3_src_install() {
+_eapi4_src_install() {
 	if [[ -f Makefile || -f GNUmakefile || -f makefile ]] ; then
 		emake DESTDIR="${D}" install
 	fi
@@ -682,7 +690,7 @@ dyn_unpack() {
 	if [ -e "${WORKDIR}" ]; then
 		local x
 		local checkme
-		for x in ${AA}; do
+		for x in $A ; do
 			vecho ">>> Checking ${x}'s mtime..."
 			if [ "${PORTAGE_ACTUAL_DISTDIR:-${DISTDIR}}/${x}" -nt "${WORKDIR}" ]; then
 				vecho ">>> ${x} has been updated; recreating WORKDIR..."
@@ -746,7 +754,8 @@ dyn_clean() {
 
 	if [[ $EMERGE_FROM = binary ]] || ! hasq keepwork $FEATURES; then
 		rm -f "$PORTAGE_BUILDDIR"/.{ebuild_changed,exit_status,logid,unpacked,prepared} \
-			"$PORTAGE_BUILDDIR"/.{configured,compiled,tested,packaged}
+			"$PORTAGE_BUILDDIR"/.{configured,compiled,tested,packaged} \
+			"$PORTAGE_BUILDDIR"/.die_hooks
 
 		rm -rf "${PORTAGE_BUILDDIR}/build-info"
 		rm -rf "${WORKDIR}"
@@ -882,6 +891,16 @@ abort_install() {
 	exit 1
 }
 
+has_phase_defined_up_to() {
+	local phase
+	for phase in unpack prepare configure compile install; do
+		has ${phase} ${DEFINED_PHASES} && return 0
+		[[ ${phase} == $1 ]] && return 1
+	done
+	# We shouldn't actually get here
+	return 1
+}
+
 dyn_prepare() {
 
 	if [[ -e $PORTAGE_BUILDDIR/.prepared ]] ; then
@@ -890,19 +909,21 @@ dyn_prepare() {
 		return 0
 	fi
 
-	local srcdir
 	if [[ -d $S ]] ; then
-		srcdir=$S
+		cd "${S}"
+	elif hasq $EAPI 0 1 2 3 3_pre2 ; then
+		cd "${WORKDIR}"
+	elif [[ -z ${A} ]] && ! has_phase_defined_up_to prepare; then
+		cd "${WORKDIR}"
 	else
-		srcdir=$WORKDIR
+		die "The source directory '${S}' doesn't exist"
 	fi
-	cd "$srcdir"
 
 	trap abort_prepare SIGINT SIGQUIT
 
 	[ -n "$EBUILD_PHASE" ] && rm -f "$T/logging/$EBUILD_PHASE"
 	ebuild_phase pre_src_prepare
-	vecho ">>> Preparing source in $srcdir ..."
+	vecho ">>> Preparing source in $PWD ..."
 	ebuild_phase src_prepare
 	touch "$PORTAGE_BUILDDIR"/.prepared
 	vecho ">>> Source prepared."
@@ -919,12 +940,22 @@ dyn_configure() {
 		return 0
 	fi
 
+	if [[ -d $S ]] ; then
+		cd "${S}"
+	elif hasq $EAPI 0 1 2 3 3_pre2 ; then
+		cd "${WORKDIR}"
+	elif [[ -z ${A} ]] && ! has_phase_defined_up_to configure; then
+		cd "${WORKDIR}"
+	else
+		die "The source directory '${S}' doesn't exist"
+	fi
+
 	trap abort_configure SIGINT SIGQUIT
 
 	[ -n "$EBUILD_PHASE" ] && rm -f "$T/logging/$EBUILD_PHASE"
 	ebuild_phase pre_src_configure
 
-	vecho ">>> Configuring source in $srcdir ..."
+	vecho ">>> Configuring source in $PWD ..."
 	ebuild_phase src_configure
 	touch "$PORTAGE_BUILDDIR"/.configured
 	vecho ">>> Source configured."
@@ -942,12 +973,22 @@ dyn_compile() {
 		return 0
 	fi
 
+	if [[ -d $S ]] ; then
+		cd "${S}"
+	elif hasq $EAPI 0 1 2 3 3_pre2 ; then
+		cd "${WORKDIR}"
+	elif [[ -z ${A} ]] && ! has_phase_defined_up_to compile; then
+		cd "${WORKDIR}"
+	else
+		die "The source directory '${S}' doesn't exist"
+	fi
+
 	trap abort_compile SIGINT SIGQUIT
 
 	[ -n "$EBUILD_PHASE" ] && rm -f "$T/logging/$EBUILD_PHASE"
 	ebuild_phase pre_src_compile
 
-	vecho ">>> Compiling source in ${srcdir} ..."
+	vecho ">>> Compiling source in $PWD ..."
 	ebuild_phase src_compile
 	touch "$PORTAGE_BUILDDIR"/.compiled
 	vecho ">>> Source compiled."
@@ -974,10 +1015,11 @@ dyn_test() {
 	else
 		cd "${WORKDIR}"
 	fi
+
 	if ! hasq test $FEATURES && [ "${EBUILD_FORCE_TEST}" != "1" ]; then
 		vecho ">>> Test phase [not enabled]: ${CATEGORY}/${PF}"
 	elif hasq test $RESTRICT; then
-		ewarn "Skipping make test/check due to ebuild restriction."
+		einfo "Skipping make test/check due to ebuild restriction."
 		vecho ">>> Test phase [explicitly disabled]: ${CATEGORY}/${PF}"
 	else
 		local save_sp=${SANDBOX_PREDICT}
@@ -1008,11 +1050,16 @@ dyn_install() {
 	ebuild_phase pre_src_install
 	rm -rf "${PORTAGE_BUILDDIR}/image"
 	mkdir "${PORTAGE_BUILDDIR}/image"
-	if [ -d "${S}" ]; then
+	if [[ -d $S ]] ; then
 		cd "${S}"
-	else
+	elif hasq $EAPI 0 1 2 3 3_pre2 ; then
 		cd "${WORKDIR}"
+	elif [[ -z ${A} ]] && ! has_phase_defined_up_to install; then
+		cd "${WORKDIR}"
+	else
+		die "The source directory '${S}' doesn't exist"
 	fi
+
 	vecho
 	vecho ">>> Install ${PF} into ${D} category ${CATEGORY}"
 	#our custom version of libtool uses $S and $D to fix
@@ -1040,14 +1087,19 @@ dyn_install() {
 	set -f
 	local f x
 	IFS=$' \t\n\r'
-	for f in ASFLAGS CATEGORY CBUILD CC CFLAGS CHOST CTARGET CXX \
-		CXXFLAGS DEFINED_PHASES DEPEND EXTRA_ECONF EXTRA_EINSTALL EXTRA_MAKE \
-		FEATURES INHERITED IUSE LDFLAGS LIBCFLAGS LIBCXXFLAGS \
-		LICENSE PDEPEND PF PKGUSE PROPERTIES PROVIDE RDEPEND RESTRICT SLOT \
-		KEYWORDS HOMEPAGE SRC_URI DESCRIPTION; do
+	for f in CATEGORY DEFINED_PHASES FEATURES INHERITED IUSE  \
+		PF PKGUSE SLOT KEYWORDS HOMEPAGE DESCRIPTION ; do
 		x=$(echo -n ${!f})
 		[[ -n $x ]] && echo "$x" > $f
 	done
+	if [[ $CATEGORY != virtual ]] ; then
+		for f in ASFLAGS CBUILD CC CFLAGS CHOST CTARGET CXX \
+			CXXFLAGS EXTRA_ECONF EXTRA_EINSTALL EXTRA_MAKE \
+			LDFLAGS LIBCFLAGS LIBCXXFLAGS ; do
+			x=$(echo -n ${!f})
+			[[ -n $x ]] && echo "$x" > $f
+		done
+	fi
 	echo "${USE}"       > USE
 	echo "${EAPI:-0}"   > EAPI
 	set +f
@@ -1056,7 +1108,7 @@ dyn_install() {
 	unset f
 
 	save_ebuild_env --exclude-init-phases | filter_readonly_variables \
-		--filter-sandbox --allow-extra-vars > environment
+		--filter-path --filter-sandbox --allow-extra-vars > environment
 
 	bzip2 -f9 environment
 
@@ -1257,10 +1309,10 @@ inherit() {
 
 		# Retain the old data and restore it later.
 		unset B_IUSE B_DEPEND B_RDEPEND B_PDEPEND
-		[ "${IUSE-unset}"    != "unset" ] && B_IUSE="${IUSE}"
-		[ "${DEPEND-unset}"  != "unset" ] && B_DEPEND="${DEPEND}"
-		[ "${RDEPEND-unset}" != "unset" ] && B_RDEPEND="${RDEPEND}"
-		[ "${PDEPEND-unset}" != "unset" ] && B_PDEPEND="${PDEPEND}"
+		[ "${IUSE+set}"       = set ] && B_IUSE="${IUSE}"
+		[ "${DEPEND+set}"     = set ] && B_DEPEND="${DEPEND}"
+		[ "${RDEPEND+set}"    = set ] && B_RDEPEND="${RDEPEND}"
+		[ "${PDEPEND+set}"    = set ] && B_PDEPEND="${PDEPEND}"
 		unset IUSE DEPEND RDEPEND PDEPEND
 		#turn on glob expansion
 		set +f
@@ -1272,22 +1324,22 @@ inherit() {
 
 		# If each var has a value, append it to the global variable E_* to
 		# be applied after everything is finished. New incremental behavior.
-		[ "${IUSE-unset}"    != "unset" ] && export E_IUSE="${E_IUSE} ${IUSE}"
-		[ "${DEPEND-unset}"  != "unset" ] && export E_DEPEND="${E_DEPEND} ${DEPEND}"
-		[ "${RDEPEND-unset}" != "unset" ] && export E_RDEPEND="${E_RDEPEND} ${RDEPEND}"
-		[ "${PDEPEND-unset}" != "unset" ] && export E_PDEPEND="${E_PDEPEND} ${PDEPEND}"
+		[ "${IUSE+set}"       = set ] && export E_IUSE="${E_IUSE} ${IUSE}"
+		[ "${DEPEND+set}"     = set ] && export E_DEPEND="${E_DEPEND} ${DEPEND}"
+		[ "${RDEPEND+set}"    = set ] && export E_RDEPEND="${E_RDEPEND} ${RDEPEND}"
+		[ "${PDEPEND+set}"    = set ] && export E_PDEPEND="${E_PDEPEND} ${PDEPEND}"
 
-		[ "${B_IUSE-unset}"    != "unset" ] && IUSE="${B_IUSE}"
-		[ "${B_IUSE-unset}"    != "unset" ] || unset IUSE
+		[ "${B_IUSE+set}"     = set ] && IUSE="${B_IUSE}"
+		[ "${B_IUSE+set}"     = set ] || unset IUSE
 
-		[ "${B_DEPEND-unset}"  != "unset" ] && DEPEND="${B_DEPEND}"
-		[ "${B_DEPEND-unset}"  != "unset" ] || unset DEPEND
+		[ "${B_DEPEND+set}"   = set ] && DEPEND="${B_DEPEND}"
+		[ "${B_DEPEND+set}"   = set ] || unset DEPEND
 
-		[ "${B_RDEPEND-unset}" != "unset" ] && RDEPEND="${B_RDEPEND}"
-		[ "${B_RDEPEND-unset}" != "unset" ] || unset RDEPEND
+		[ "${B_RDEPEND+set}"  = set ] && RDEPEND="${B_RDEPEND}"
+		[ "${B_RDEPEND+set}"  = set ] || unset RDEPEND
 
-		[ "${B_PDEPEND-unset}" != "unset" ] && PDEPEND="${B_PDEPEND}"
-		[ "${B_PDEPEND-unset}" != "unset" ] || unset PDEPEND
+		[ "${B_PDEPEND+set}"  = set ] && PDEPEND="${B_PDEPEND}"
+		[ "${B_PDEPEND+set}"  = set ] || unset PDEPEND
 
 		#turn on glob expansion
 		set +f
@@ -1359,7 +1411,7 @@ _ebuild_arg_to_phase() {
 
 	case "$arg" in
 		pretend)
-			! hasq $eapi 0 1 2 && \
+			! hasq $eapi 0 1 2 3 3_pre2 && \
 				phase_func=pkg_pretend
 			;;
 		setup)
@@ -1455,8 +1507,8 @@ _ebuild_phase_funcs() {
 			declare -F src_compile >/dev/null || \
 				src_compile() { _eapi2_src_compile "$@" ; }
 
-			[[ $eapi == 2 ]] || declare -F src_install >/dev/null || \
-				src_install() { _eapi3_src_install "$@" ; }
+			has $eapi 2 3 3_pre2 || declare -F src_install >/dev/null || \
+				src_install() { _eapi4_src_install "$@" ; }
 
 			if hasq $phase_func $default_phases ; then
 
@@ -1549,8 +1601,8 @@ READONLY_EBUILD_METADATA="DEFINED_PHASES DEPEND DESCRIPTION
 	PDEPEND PROVIDE RDEPEND RESTRICT SLOT SRC_URI"
 
 READONLY_PORTAGE_VARS="D EBUILD EBUILD_PHASE \
-	EBUILD_SH_ARGS EMERGE_FROM FILESDIR PORTAGE_BINPKG_FILE \
-	PORTAGE_BIN_PATH PORTAGE_IUSE \
+	EBUILD_SH_ARGS EMERGE_FROM FILESDIR \
+	PORTAGE_BINPKG_FILE PORTAGE_BIN_PATH PORTAGE_IUSE \
 	PORTAGE_PYM_PATH PORTAGE_MUTABLE_FILTERED_VARS \
 	PORTAGE_SAVED_READONLY_VARS PORTAGE_TMPDIR T WORKDIR"
 
@@ -1584,6 +1636,17 @@ PORTAGE_MUTABLE_FILTERED_VARS="AA HOSTNAME"
 # phase. However, if FEATURES exist inside environment.bz2 then they
 # should be overridden by current settings.
 #
+# --filter-locale causes locale related variables such as LANG and LC_*
+# variables to be filtered. These variables should persist between phases,
+# in case they are modified by the ebuild. However, the current user
+# settings should be used when loading the environment from a binary or
+# installed package.
+#
+# --filter-path causes the PATH variable to be filtered. This variable
+# should persist between phases, in case it is modified by the ebuild.
+# However, old settings should be overridden when loading the
+# environment from a binary or installed package.
+#
 # ---allow-extra-vars causes some extra vars to be allowd through, such
 # as ${PORTAGE_SAVED_READONLY_VARS} and ${PORTAGE_MUTABLE_FILTERED_VARS}.
 #
@@ -1593,13 +1656,28 @@ PORTAGE_MUTABLE_FILTERED_VARS="AA HOSTNAME"
 # variables out and discards them. See bug #190128.
 filter_readonly_variables() {
 	local x filtered_vars
-	local readonly_bash_vars="DIRSTACK EUID FUNCNAME GROUPS
-		PIPESTATUS PPID SHELLOPTS UID"
+	local readonly_bash_vars="BASHOPTS BASHPID DIRSTACK EUID
+		FUNCNAME GROUPS PIPESTATUS PPID SHELLOPTS UID"
+	local bash_misc_vars="BASH BASH_.* COMP_WORDBREAKS HISTCMD
+		HISTFILE HOSTNAME HOSTTYPE IFS LINENO MACHTYPE OLDPWD
+		OPTERR OPTIND OSTYPE POSIXLY_CORRECT PS4 PWD RANDOM
+		SECONDS SHELL SHLVL"
 	local filtered_sandbox_vars="SANDBOX_ACTIVE SANDBOX_BASHRC
 		SANDBOX_DEBUG_LOG SANDBOX_DISABLED SANDBOX_LIB
 		SANDBOX_LOG SANDBOX_ON"
-	filtered_vars="${readonly_bash_vars} ${READONLY_PORTAGE_VARS}
-		BASH_.* HISTFILE PATH POSIXLY_CORRECT"
+	filtered_vars="$readonly_bash_vars $bash_misc_vars
+		$READONLY_PORTAGE_VARS"
+
+	# Don't filter/interfere with prefix variables unless they are
+	# supported by the current EAPI.
+	case "${EAPI:-0}" in
+		0|1|2)
+			;;
+		*)
+			filtered_vars+=" ED EPREFIX EROOT"
+			;;
+	esac
+
 	if hasq --filter-sandbox $* ; then
 		filtered_vars="${filtered_vars} SANDBOX_.*"
 	else
@@ -1608,6 +1686,14 @@ filter_readonly_variables() {
 	if hasq --filter-features $* ; then
 		filtered_vars="${filtered_vars} FEATURES"
 	fi
+	if hasq --filter-path $* ; then
+		filtered_vars+=" PATH"
+	fi
+	if hasq --filter-locale $* ; then
+		filtered_vars+=" LANG LC_ALL LC_COLLATE
+			LC_CTYPE LC_MESSAGES LC_MONETARY
+			LC_NUMERIC LC_PAPER LC_TIME"
+	fi
 	if ! hasq --allow-extra-vars $* ; then
 		filtered_vars="
 			${filtered_vars}
@@ -1616,7 +1702,7 @@ filter_readonly_variables() {
 		"
 	fi
 
-	"${PORTAGE_BIN_PATH}"/filter-bash-environment.py "${filtered_vars}"
+	EPYTHON= "${PORTAGE_BIN_PATH}"/filter-bash-environment.py "${filtered_vars}"
 }
 
 # @FUNCTION: preprocess_ebuild_env
@@ -1633,7 +1719,7 @@ preprocess_ebuild_env() {
 		# environment may contain stale SANDBOX_{DENY,PREDICT,READ,WRITE}
 		# and FEATURES variables that should be filtered out. Between
 		# phases, these variables are normally preserved.
-		filter_opts="--filter-sandbox --filter-features ${filter_opts}"
+		filter_opts+=" --filter-features --filter-locale --filter-path --filter-sandbox"
 	fi
 	filter_readonly_variables ${filter_opts} < "${T}"/environment \
 		> "${T}"/environment.filtered || return $?
@@ -1691,7 +1777,7 @@ unset E_IUSE E_DEPEND E_RDEPEND E_PDEPEND
 shopt -u extglob
 
 if [[ ${EBUILD_PHASE} == depend ]] ; then
-	QA_INTERCEPTORS="awk bash cc egrep fgrep g++
+	QA_INTERCEPTORS="awk bash cc egrep equery fgrep g++
 		gawk gcc grep javac java-config nawk perl
 		pkg-config python python-config sed"
 elif [[ ${EBUILD_PHASE} == clean* ]] ; then
@@ -1777,78 +1863,101 @@ if ! hasq "$EBUILD_PHASE" clean cleanrm depend && \
 	[[ -n $EAPI ]] || EAPI=0
 fi
 
-_source_ebuild() {
-	# The bashrcs get an opportunity here to set aliases that will be expanded
-	# during sourcing of ebuilds and eclasses.
-	source_all_bashrcs
-
-	# *DEPEND and IUSE will be set during the sourcing of the ebuild.
-	# In order to ensure correct interaction between ebuilds and
-	# eclasses, they need to be unset before this process of
-	# interaction begins.
-	unset DEPEND RDEPEND PDEPEND IUSE
-	source "${EBUILD}" || die "error sourcing ebuild"
-
-	if [ "${EBUILD_PHASE}" != "depend" ] ; then
-		RESTRICT=${PORTAGE_RESTRICT}
-		[[ -e $PORTAGE_BUILDDIR/.ebuild_changed ]] && \
-			rm "$PORTAGE_BUILDDIR/.ebuild_changed"
-	fi
-
-	[[ -n $EAPI ]] || EAPI=0
-
-	if has "$EAPI" 0 1 2 ; then
-		export RDEPEND=${RDEPEND-${DEPEND}}
-		debug-print "RDEPEND: not set... Setting to: ${DEPEND}"
-	fi
-
-	# add in dependency info from eclasses
-	IUSE="${IUSE} ${E_IUSE}"
-	DEPEND="${DEPEND} ${E_DEPEND}"
-	RDEPEND="${RDEPEND} ${E_RDEPEND}"
-	PDEPEND="${PDEPEND} ${E_PDEPEND}"
-
-	unset ECLASS E_IUSE E_DEPEND E_RDEPEND E_PDEPEND
-
-	# alphabetically ordered by $EBUILD_PHASE value
-	local f valid_phases
-	case "$EAPI" in
-		0|1)
-			valid_phases="src_compile pkg_config pkg_info src_install
-				pkg_nofetch pkg_postinst pkg_postrm pkg_preinst pkg_prerm
-				pkg_setup src_test src_unpack"
-			;;
-		2)
-			valid_phases="src_compile pkg_config src_configure pkg_info
-				src_install pkg_nofetch pkg_postinst pkg_postrm pkg_preinst
-				src_prepare pkg_prerm pkg_setup src_test src_unpack"
-			;;
-		*)
-			valid_phases="src_compile pkg_config src_configure pkg_info
-				src_install pkg_nofetch pkg_postinst pkg_postrm pkg_preinst
-				src_prepare pkg_prerm pkg_pretend pkg_setup src_test src_unpack"
-			;;
-	esac
-
-	DEFINED_PHASES=
-	for f in $valid_phases ; do
-		if declare -F $f >/dev/null ; then
-			f=${f#pkg_}
-			DEFINED_PHASES+=" ${f#src_}"
-		fi
-	done
-	[[ -n $DEFINED_PHASES ]] || DEFINED_PHASES=-
-
-	# This needs to be exported since prepstrip is a separate shell script.
-	[[ -n $QA_PRESTRIPPED ]] && export QA_PRESTRIPPED
-	eval "[[ -n \$QA_PRESTRIPPED_${ARCH/-/_} ]] && export QA_PRESTRIPPED_${ARCH/-/_}"
-}
-
 if ! hasq "$EBUILD_PHASE" clean cleanrm ; then
 	if [[ $EBUILD_PHASE = depend || ! -f $T/environment || \
 		-f $PORTAGE_BUILDDIR/.ebuild_changed ]] || \
 		hasq noauto $FEATURES ; then
-		_source_ebuild
+		# The bashrcs get an opportunity here to set aliases that will be expanded
+		# during sourcing of ebuilds and eclasses.
+		source_all_bashrcs
+
+		# *DEPEND and IUSE will be set during the sourcing of the ebuild.
+		# In order to ensure correct interaction between ebuilds and
+		# eclasses, they need to be unset before this process of
+		# interaction begins.
+		unset DEPEND RDEPEND PDEPEND IUSE
+
+		if [[ $PORTAGE_DEBUG != 1 || ${-/x/} != $- ]] ; then
+			source "$EBUILD" || die "error sourcing ebuild"
+		else
+			set -x
+			source "$EBUILD" || die "error sourcing ebuild"
+			set +x
+		fi
+
+		if [[ "${EBUILD_PHASE}" != "depend" ]] ; then
+			RESTRICT=${PORTAGE_RESTRICT}
+			[[ -e $PORTAGE_BUILDDIR/.ebuild_changed ]] && \
+			rm "$PORTAGE_BUILDDIR/.ebuild_changed"
+		fi
+
+		[[ -n $EAPI ]] || EAPI=0
+
+		if has "$EAPI" 0 1 2 3 3_pre2 ; then
+			export RDEPEND=${RDEPEND-${DEPEND}}
+			debug-print "RDEPEND: not set... Setting to: ${DEPEND}"
+		fi
+
+		# add in dependency info from eclasses
+		IUSE="${IUSE} ${E_IUSE}"
+		DEPEND="${DEPEND} ${E_DEPEND}"
+		RDEPEND="${RDEPEND} ${E_RDEPEND}"
+		PDEPEND="${PDEPEND} ${E_PDEPEND}"
+
+		unset ECLASS E_IUSE E_DEPEND E_RDEPEND E_PDEPEND
+
+		# alphabetically ordered by $EBUILD_PHASE value
+		case "$EAPI" in
+			0|1)
+				_valid_phases="src_compile pkg_config pkg_info src_install
+					pkg_nofetch pkg_postinst pkg_postrm pkg_preinst pkg_prerm
+					pkg_setup src_test src_unpack"
+				;;
+			2|3|3_pre2)
+				_valid_phases="src_compile pkg_config src_configure pkg_info
+					src_install pkg_nofetch pkg_postinst pkg_postrm pkg_preinst
+					src_prepare pkg_prerm pkg_setup src_test src_unpack"
+				;;
+			*)
+				_valid_phases="src_compile pkg_config src_configure pkg_info
+					src_install pkg_nofetch pkg_postinst pkg_postrm pkg_preinst
+					src_prepare pkg_prerm pkg_pretend pkg_setup src_test src_unpack"
+				;;
+		esac
+
+		DEFINED_PHASES=
+		for _f in $_valid_phases ; do
+			if declare -F $_f >/dev/null ; then
+				_f=${_f#pkg_}
+				DEFINED_PHASES+=" ${_f#src_}"
+			fi
+		done
+		[[ -n $DEFINED_PHASES ]] || DEFINED_PHASES=-
+
+		unset _f _valid_phases
+
+		if [[ $EBUILD_PHASE != depend ]] ; then
+
+			if hasq distcc $FEATURES ; then
+				PATH="/usr/lib/distcc/bin:$PATH"
+				[[ -n $DISTCC_LOG ]] && addwrite "${DISTCC_LOG%/*}"
+			fi
+
+			if hasq ccache $FEATURES ; then
+				PATH="/usr/lib/ccache/bin:$PATH"
+
+				if [[ -n $CCACHE_DIR ]] ; then
+					addread "$CCACHE_DIR"
+					addwrite "$CCACHE_DIR"
+				fi
+
+				[[ -n $CCACHE_SIZE ]] && ccache -M $CCACHE_SIZE &> /dev/null
+			fi
+		fi
+
+		# This needs to be exported since prepstrip is a separate shell script.
+		[[ -n $QA_PRESTRIPPED ]] && export QA_PRESTRIPPED
+		eval "[[ -n \$QA_PRESTRIPPED_${ARCH/-/_} ]] && export QA_PRESTRIPPED_${ARCH/-/_}"
 	fi
 fi
 
@@ -1876,33 +1985,23 @@ export TMPDIR="${T}"
 # declare them only after it has already run.
 if [ "${EBUILD_PHASE}" != "depend" ] ; then
 	declare -r ${READONLY_EBUILD_METADATA} ${READONLY_PORTAGE_VARS}
+	case "$EAPI" in
+		0|1|2)
+			;;
+		*)
+			declare -r ED EPREFIX EROOT
+			;;
+	esac
 fi
 
 ebuild_main() {
 	local f x
 
-	if ! hasq $EBUILD_SH_ARGS clean depend help info nofetch ; then
-
-		if hasq distcc $FEATURES ; then
-			export PATH="/usr/lib/distcc/bin:$PATH"
-			[[ -n $DISTCC_LOG ]] && addwrite "${DISTCC_LOG%/*}"
-		fi
-
-		if hasq ccache $FEATURES ; then
-			export PATH="/usr/lib/ccache/bin:$PATH"
-
-			addread "$CCACHE_DIR"
-			addwrite "$CCACHE_DIR"
-
-			[[ -n $CCACHE_SIZE ]] && ccache -M $CCACHE_SIZE &> /dev/null
-		else
-			# Force configure scripts that automatically detect ccache to
-			# respect FEATURES="-ccache".
-			export CCACHE_DISABLE=1
-		fi
-	fi
-
 	if [[ $EBUILD_PHASE != depend ]] ; then
+		# Force configure scripts that automatically detect ccache to
+		# respect FEATURES="-ccache".
+		hasq ccache $FEATURES || export CCACHE_DISABLE=1
+
 		local phase_func=$(_ebuild_arg_to_phase "$EAPI" "$EBUILD_PHASE")
 		[[ -n $phase_func ]] && _ebuild_phase_funcs "$EAPI" "$phase_func"
 		unset phase_func
@@ -1932,7 +2031,8 @@ ebuild_main() {
 			# Update environment.bz2 in case installation phases
 			# need to pass some variables to uninstallation phases.
 			save_ebuild_env --exclude-init-phases | \
-				filter_readonly_variables --filter-sandbox --allow-extra-vars \
+				filter_readonly_variables --filter-path \
+				--filter-sandbox --allow-extra-vars \
 				| bzip2 -c -f9 > "$PORTAGE_UPDATE_ENV"
 		fi
 		;;
@@ -1948,7 +2048,7 @@ ebuild_main() {
 
 			for x in ASFLAGS CCACHE_DIR CCACHE_SIZE \
 				CFLAGS CXXFLAGS LDFLAGS LIBCFLAGS LIBCXXFLAGS ; do
-				[[ ${!x-unset} != unset ]] && export $x
+				[[ ${!x+set} = set ]] && export $x
 			done
 
 			hasq distcc $FEATURES && [[ -n $DISTCC_DIR ]] && \
@@ -1985,13 +2085,6 @@ ebuild_main() {
 				cp "$EBUILD" "build-info/$PF.ebuild"
 			fi
 
-			local srcdir
-			if [[ -d $S ]] ; then
-				srcdir=$S
-			else
-				srcdir=$WORKDIR
-			fi
-			cd "$srcdir"
 			#our custom version of libtool uses $S and $D to fix
 			#invalid paths in .la files
 			export S D
@@ -2012,6 +2105,11 @@ ebuild_main() {
 		export SANDBOX_ON="0"
 		;;
 	help|pretend|setup|preinst)
+		if [[ $EBUILD_SH_ARGS = setup ]] ; then
+			einfo "CPV:  $CATEGORY/$PF"
+			einfo "REPO: $PORTAGE_REPO_NAME"
+			einfo "USE:  $USE"
+		fi
 		#pkg_setup needs to be out of the sandbox for tmp file creation;
 		#for example, awking and piping a file in /tmp requires a temp file to be created
 		#in /etc.  If pkg_setup is in the sandbox, both our lilo and apache ebuilds break.
