@@ -1,8 +1,9 @@
-#!/usr/bin/env python
+#!/usr/bin/python
 # Copyright 1999-2007 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: filter-bash-environment.py 9465 2008-03-14 18:20:49Z zmedico $
+# $Id: filter-bash-environment.py 15539 2010-03-02 19:51:14Z zmedico $
 
+import codecs
 import os, re, sys
 
 here_doc_re = re.compile(r'.*\s<<[-]?(\w+)$')
@@ -12,6 +13,8 @@ func_end_re = re.compile(r'^\}$')
 var_assign_re = re.compile(r'(^|^declare\s+-\S+\s+|^declare\s+|^export\s+)([^=\s]+)=("|\')?.*$')
 close_quote_re = re.compile(r'(\\"|"|\')\s*$')
 readonly_re = re.compile(r'^declare\s+-(\S*)r(\S*)\s+')
+# declare without assignment
+var_declare_re = re.compile(r'^declare(\s+-\S+)?\s+([^=\s]+)\s*$')
 
 def have_end_quote(quote, line):
 	"""
@@ -24,7 +27,27 @@ def have_end_quote(quote, line):
 	return close_quote_match is not None and \
 		close_quote_match.group(1) == quote
 
+def filter_declare_readonly_opt(line):
+	readonly_match = readonly_re.match(line)
+	if readonly_match is not None:
+		declare_opts = ''
+		for i in (1, 2):
+			group = readonly_match.group(i)
+			if group is not None:
+				declare_opts += group
+		if declare_opts:
+			line = 'declare -%s %s' % \
+				(declare_opts, line[readonly_match.end():])
+		else:
+			line = 'declare ' + line[readonly_match.end():]
+	return line
+
 def filter_bash_environment(pattern, file_in, file_out):
+	# Filter out any instances of the \1 character from variable values
+	# since this character multiplies each time that the environment
+	# is saved (strange bash behavior). This can eventually result in
+	# mysterious 'Argument list too long' errors from programs that have
+	# huge strings of \1 characters in their environment. See bug #222091.
 	here_doc_delim = None
 	in_func = None
 	multi_line_quote = None
@@ -32,7 +55,7 @@ def filter_bash_environment(pattern, file_in, file_out):
 	for line in file_in:
 		if multi_line_quote is not None:
 			if not multi_line_quote_filter:
-				file_out.write(line)
+				file_out.write(line.replace("\1", ""))
 			if have_end_quote(multi_line_quote, line):
 				multi_line_quote = None
 				multi_line_quote_filter = None
@@ -43,24 +66,29 @@ def filter_bash_environment(pattern, file_in, file_out):
 				quote = var_assign_match.group(3)
 				filter_this = pattern.match(var_assign_match.group(2)) \
 					is not None
-				if quote is not None and not have_end_quote(quote, line):
+				# Exclude the start quote when searching for the end quote,
+				# to ensure that the start quote is not misidentified as the
+				# end quote (happens if there is a newline immediately after
+				# the start quote).
+				if quote is not None and not \
+					have_end_quote(quote, line[var_assign_match.end(2)+2:]):
 					multi_line_quote = quote
 					multi_line_quote_filter = filter_this
 				if not filter_this:
-					readonly_match = readonly_re.match(line)
-					if readonly_match is not None:
-						declare_opts = ""
-						for i in (1, 2):
-							group = readonly_match.group(i)
-							if group is not None:
-								declare_opts += group
-						if declare_opts:
-							line = "declare -%s %s" % \
-								(declare_opts, line[readonly_match.end():])
-						else:
-							line = "declare " + line[readonly_match.end():]
-					file_out.write(line)
+					line = filter_declare_readonly_opt(line)
+					file_out.write(line.replace("\1", ""))
 				continue
+			else:
+				declare_match = var_declare_re.match(line)
+				if declare_match is not None:
+					# declare without assignment
+					filter_this = pattern.match(declare_match.group(2)) \
+						is not None
+					if not filter_this:
+						line = filter_declare_readonly_opt(line)
+						file_out.write(line)
+					continue
+
 		if here_doc_delim is not None:
 			if here_doc_delim.match(line):
 				here_doc_delim = None
@@ -102,6 +130,13 @@ if __name__ == "__main__":
 		parser.error("Missing required PATTERN argument.")
 	file_in = sys.stdin
 	file_out = sys.stdout
+	if sys.hexversion >= 0x3000000:
+		file_in = codecs.iterdecode(sys.stdin.buffer.raw,
+			'utf_8', errors='replace')
+		import io
+		file_out = io.TextIOWrapper(sys.stdout.buffer,
+			'utf_8', errors='backslashreplace')
+
 	var_pattern = args[0].split()
 
 	# Filter invalid variable names that are not supported by bash.
