@@ -1,14 +1,20 @@
 # Copyright 1999-2009 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: isolated-functions.sh 14527 2009-10-09 19:50:09Z zmedico $
+# $Id: isolated-functions.sh 15146 2009-12-24 05:11:32Z zmedico $
 
 # We need this next line for "die" and "assert". It expands
 # It _must_ preceed all the calls to die and assert.
 shopt -s expand_aliases
-alias assert='_pipestatus="${PIPESTATUS[*]}"; [[ "${_pipestatus// /}" -eq 0 ]] || die'
 alias save_IFS='[ "${IFS:-unset}" != "unset" ] && old_IFS="${IFS}"'
 alias restore_IFS='if [ "${old_IFS:-unset}" != "unset" ]; then IFS="${old_IFS}"; unset old_IFS; else unset IFS; fi'
 
+assert() {
+	local x pipestatus=${PIPESTATUS[*]}
+	for x in $pipestatus ; do
+		[[ $x -eq 0 ]] || die "$@"
+	done
+}
+
 shopt -s extdebug
 
 # dump_trace([number of funcs on stack to skip],
@@ -54,7 +60,7 @@ dump_trace() {
 }
 
 nonfatal() {
-	if has "${EAPI:-0}" 0 1 2; then
+	if has "${EAPI:-0}" 0 1 2 3 3_pre2 ; then
 		die "$FUNCNAME() not supported in this EAPI"
 	fi
 	if [[ $# -lt 1 ]]; then
@@ -156,6 +162,7 @@ die() {
 		for x in $EBUILD_DEATH_HOOKS; do
 			${x} "$@" >&2 1>&2
 		done
+		> "$PORTAGE_BUILDDIR/.die_hooks"
 	fi
 
 	[[ -n ${PORTAGE_LOG_FILE} ]] \
@@ -489,18 +496,12 @@ save_ebuild_env() {
 
 		if hasq --exclude-init-phases $* ; then
 			unset S _E_DOCDESTTREE_ _E_EXEDESTTREE_
-			unset -f pkg_nofetch src_unpack src_prepare src_configure \
-			src_compile src_test src_install
 			if [[ -n $PYTHONPATH ]] ; then
 				export PYTHONPATH=${PYTHONPATH/${PORTAGE_PYM_PATH}:}
 				[[ -z $PYTHONPATH ]] && unset PYTHONPATH
 			fi
 		fi
 
-		# misc variables set by bash
-		unset BASH HOSTTYPE IFS MACHTYPE OLDPWD \
-			OPTERR OPTIND OSTYPE PS4 PWD SHELL SHLVL
-
 		# misc variables inherited from the calling environment
 		unset COLORTERM DISPLAY EDITOR LESS LESSOPEN LOGNAME LS_COLORS PAGER \
 			TERM TERMCAP USER
@@ -518,13 +519,15 @@ save_ebuild_env() {
 		for x in pkg_setup pkg_nofetch src_unpack src_prepare src_configure \
 			src_compile src_test src_install pkg_preinst pkg_postinst \
 			pkg_prerm pkg_postrm ; do
-			unset -f default_$x _eapi{0,1,2}_$x
+			unset -f default_$x _eapi{0,1,2,3,4}_$x
 		done
 		unset x
 
-		unset -f dump_trace die diefunc quiet_mode vecho elog_base eqawarn elog \
+		unset -f assert dump_trace die diefunc \
+			quiet_mode vecho elog_base eqawarn elog \
 			esyslog einfo einfon ewarn eerror ebegin _eend eend KV_major \
 			KV_minor KV_micro KV_to_int get_KV unset_colors set_colors has \
+			has_phase_defined_up_to \
 			hasg hasgq hasv hasq qa_source qa_call \
 			addread addwrite adddeny addpredict _sb_append_var \
 			lchown lchgrp esyslog use usev useq has_version portageq \
@@ -542,7 +545,7 @@ save_ebuild_env() {
 			source_all_bashrcs ebuild_main \
 			ebuild_phase ebuild_phase_with_hooks \
 			_ebuild_arg_to_phase _ebuild_phase_funcs default \
-			_source_ebuild \
+			_pipestatus \
 			${QA_INTERCEPTORS}
 
 		# portage config variables and variables set directly by portage
@@ -550,7 +553,7 @@ save_ebuild_env() {
 			DISTCC_DIR DISTDIR DOC_SYMLINKS_DIR \
 			EBUILD_EXIT_STATUS_FILE EBUILD_FORCE_TEST EBUILD_MASTER_PID \
 			ECLASSDIR ECLASS_DEPTH ENDCOL FAKEROOTKEY \
-			GOOD HILITE HOME IMAGE \
+			GOOD HILITE HOME \
 			LAST_E_CMD LAST_E_LEN LD_PRELOAD MISC_FUNCTIONS_ARGS MOPREFIX \
 			NOCOLOR NORMAL PKGDIR PKGUSE PKG_LOGDIR PKG_TMPDIR \
 			PORTAGE_ACTUAL_DISTDIR PORTAGE_ARCHLIST \
@@ -566,14 +569,17 @@ save_ebuild_env() {
 			PROFILE_PATHS PWORKDIR QA_INTERCEPTORS \
 			RC_DEFAULT_INDENT RC_DOT_PATTERN RC_ENDCOL \
 			RC_INDENTATION READONLY_EBUILD_METADATA READONLY_PORTAGE_VARS \
-			ROOT ROOTPATH RPMDIR STARTDIR TMP TMPDIR USE_EXPAND \
+			ROOT ROOTPATH RPMDIR TEMP TMP TMPDIR USE_EXPAND \
 			WARN XARGS _RC_GET_KV_CACHE
 
 		# user config variables
 		unset DOC_SYMLINKS_DIR INSTALL_MASK PKG_INSTALL_MASK
 
-		set
-		export
+		declare -p
+		declare -fp
+		if [[ ${BASH_VERSINFO[0]} == 3 ]]; then
+			export
+		fi
 	)
 }
 
