@@ -1,7 +1,7 @@
 #!/bin/bash
 # Copyright 1999-2009 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: misc-functions.sh 14531 2009-10-09 22:13:04Z zmedico $
+# $Id: misc-functions.sh 14950 2009-12-06 11:03:54Z zmedico $
 #
 # Miscellaneous shell functions that make use of the ebuild env but don't need
 # to be included directly in ebuild.sh.
@@ -441,7 +441,7 @@ install_qa_check() {
 		done
 		[[ $reset_debug = 1 ]] && set -x
 		f=$(cat "${PORTAGE_LOG_FILE}" | \
-			"$PORTAGE_BIN_PATH"/check-implicit-pointer-usage.py)
+			EPYTHON= "$PORTAGE_BIN_PATH"/check-implicit-pointer-usage.py)
 		if [[ -n ${f} ]] ; then
 
 			# In the future this will be a forced "die". In preparation,
@@ -710,8 +710,9 @@ dyn_package() {
 	tar $tar_options -cf - $PORTAGE_BINPKG_TAR_OPTS -C "${D}" . | \
 		bzip2 -cf > "$PORTAGE_BINPKG_TMPFILE"
 	assert "failed to pack binary package: '$PORTAGE_BINPKG_TMPFILE'"
-	PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
-	python -c "from portage import xpak; t=xpak.tbz2('${PORTAGE_BINPKG_TMPFILE}'); t.recompose('${PORTAGE_BUILDDIR}/build-info')"
+	EPYTHON= PYTHONPATH=${PORTAGE_PYM_PATH}${PYTHONPATH:+:}${PYTHONPATH} \
+		"$PORTAGE_BIN_PATH"/xpak-helper.py recompose \
+		"$PORTAGE_BINPKG_TMPFILE" "$PORTAGE_BUILDDIR/build-info"
 	if [ $? -ne 0 ]; then
 		rm -f "${PORTAGE_BINPKG_TMPFILE}"
 		die "Failed to append metadata to the tbz2 file"
@@ -780,6 +781,22 @@ dyn_rpm() {
 		die "Failed to move rpm"
 }
 
+die_hooks() {
+	[[ -f $PORTAGE_BUILDDIR/.die_hooks ]] && return
+	local x
+	for x in $EBUILD_DEATH_HOOKS ; do
+		$x >&2
+	done
+	> "$PORTAGE_BUILDDIR/.die_hooks"
+}
+
+success_hooks() {
+	local x
+	for x in $EBUILD_SUCCESS_HOOKS ; do
+		$x
+	done
+}
+
 if [ -n "${MISC_FUNCTIONS_ARGS}" ]; then
 	source_all_bashrcs
 	[ "$PORTAGE_DEBUG" == "1" ] && set -x
