@@ -1,17 +1,19 @@
 # Copyright 1999-2009 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: MiscFunctionsProcess.py 13993 2009-08-11 07:47:27Z zmedico $
+# $Id: MiscFunctionsProcess.py 15657 2010-03-02 20:59:00Z zmedico $
 
-from _emerge.SpawnProcess import SpawnProcess
+from _emerge.AbstractEbuildProcess import AbstractEbuildProcess
 import portage
 from portage import os
+from portage.package.ebuild.doebuild import spawn, \
+	_doebuild_exit_status_check_and_log, _doebuild_exit_status_unlink
 
-class MiscFunctionsProcess(SpawnProcess):
+class MiscFunctionsProcess(AbstractEbuildProcess):
 	"""
 	Spawns misc-functions.sh with an existing ebuild environment.
 	"""
 
-	__slots__ = ("commands", "phase", "pkg", "settings")
+	__slots__ = ('commands',)
 
 	def _start(self):
 		settings = self.settings
@@ -23,19 +25,19 @@ class MiscFunctionsProcess(SpawnProcess):
 		self.args = [portage._shell_quote(misc_sh_binary)] + self.commands
 		self.logfile = settings.get("PORTAGE_LOG_FILE")
 
-		portage._doebuild_exit_status_unlink(
+		_doebuild_exit_status_unlink(
 			settings.get("EBUILD_EXIT_STATUS_FILE"))
 
-		SpawnProcess._start(self)
+		AbstractEbuildProcess._start(self)
 
 	def _spawn(self, args, **kwargs):
 		settings = self.settings
 		debug = settings.get("PORTAGE_DEBUG") == "1"
-		return portage.spawn(" ".join(args), settings,
+		return spawn(" ".join(args), settings,
 			debug=debug, **kwargs)
 
 	def _set_returncode(self, wait_retval):
-		SpawnProcess._set_returncode(self, wait_retval)
-		self.returncode = portage._doebuild_exit_status_check_and_log(
+		AbstractEbuildProcess._set_returncode(self, wait_retval)
+		self.returncode = _doebuild_exit_status_check_and_log(
 			self.settings, self.phase, self.returncode)
 
