@@ -1,6 +1,6 @@
 # Copyright 1999-2009 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: Scheduler.py 14540 2009-10-10 01:58:52Z zmedico $
+# $Id: Scheduler.py 15708 2010-03-02 21:13:39Z zmedico $
 
 from __future__ import print_function
 
@@ -10,7 +10,9 @@ import sys
 import textwrap
 import time
 import weakref
+
 import portage
+from portage import StringIO
 from portage import os
 from portage import _encodings
 from portage import _unicode_decode
@@ -22,6 +24,8 @@ bad = create_color_func("BAD")
 from portage._sets import SETPREFIX
 from portage._sets.base import InternalPackageSet
 from portage.util import writemsg, writemsg_level
+from portage.package.ebuild.digestcheck import digestcheck
+from portage.package.ebuild.digestgen import digestgen
 
 from _emerge.BinpkgPrefetcher import BinpkgPrefetcher
 from _emerge.Blocker import Blocker
@@ -38,6 +42,7 @@ from _emerge._find_deep_system_runtime_deps import _find_deep_system_runtime_dep
 from _emerge._flush_elog_mod_echo import _flush_elog_mod_echo
 from _emerge.JobStatusDisplay import JobStatusDisplay
 from _emerge.MergeListItem import MergeListItem
+from _emerge.MiscFunctionsProcess import MiscFunctionsProcess
 from _emerge.Package import Package
 from _emerge.PackageMerge import PackageMerge
 from _emerge.PollScheduler import PollScheduler
@@ -285,7 +290,8 @@ class Scheduler(PollScheduler):
 		@returns: True if background mode is enabled, False otherwise.
 		"""
 		background = (self._max_jobs is True or \
-			self._max_jobs > 1 or "--quiet" in self.myopts) and \
+			self._max_jobs > 1 or "--quiet" in self.myopts \
+			or "--quiet-build" in self.myopts) and \
 			not bool(self._opts_no_background.intersection(self.myopts))
 
 		if background:
@@ -351,6 +357,9 @@ class Scheduler(PollScheduler):
 		Find system packages and their deep runtime dependencies. Before being
 		merged, these packages go to merge_wait_queue, to be merged when no
 		other packages are building.
+		NOTE: This can only find deep system deps if the system set has been
+		added to the graph and traversed deeply (the depgraph "complete"
+		parameter will do this, triggered by emerge --complete-graph option).
 		"""
 		deep_system_deps = self._deep_system_deps
 		deep_system_deps.clear()
@@ -426,7 +435,7 @@ class Scheduler(PollScheduler):
 		Schedule a setup phase on the merge queue, in order to
 		serialize unsandboxed access to the live filesystem.
 		"""
-		self._task_queues.merge.addFront(setup_phase)
+		self._task_queues.merge.add(setup_phase)
 		self._schedule()
 
 	def _schedule_unpack(self, unpack_phase):
@@ -495,23 +504,19 @@ class Scheduler(PollScheduler):
 	def _dblink_elog(self, pkg_dblink, phase, func, msgs):
 
 		log_path = pkg_dblink.settings.get("PORTAGE_LOG_FILE")
-		log_file = None
-		out = sys.stdout
 		background = self._background
+		out = StringIO()
 
-		if background and log_path is not None:
-			log_file = codecs.open(_unicode_encode(log_path,
-				encoding=_encodings['fs'], errors='strict'),
-				mode='a', encoding=_encodings['content'],
-				errors='backslashreplace')
-			out = log_file
+		for msg in msgs:
+			func(msg, phase=phase, key=pkg_dblink.mycpv, out=out)
 
-		try:
-			for msg in msgs:
-				func(msg, phase=phase, key=pkg_dblink.mycpv, out=out)
-		finally:
-			if log_file is not None:
-				log_file.close()
+		out_str = out.getvalue()
+
+		if not background:
+			portage.util.writemsg_stdout(out_str, noiselevel=-1)
+
+		if log_path is not None:
+			self._append_to_log_path(log_path, out_str)
 
 	def _dblink_emerge_log(self, msg):
 		self._logger.log(msg)
@@ -544,9 +549,14 @@ class Scheduler(PollScheduler):
 		background = self._background
 		log_path = settings.get("PORTAGE_LOG_FILE")
 
-		ebuild_phase = EbuildPhase(background=background,
-			pkg=pkg, phase=phase, scheduler=scheduler,
-			settings=settings, tree=pkg_dblink.treetype)
+		if phase in ('die_hooks', 'success_hooks'):
+			ebuild_phase = MiscFunctionsProcess(background=background,
+				commands=[phase], phase=phase, pkg=pkg,
+				scheduler=scheduler, settings=settings)
+		else:
+			ebuild_phase = EbuildPhase(background=background,
+				pkg=pkg, phase=phase, scheduler=scheduler,
+				settings=settings, tree=pkg_dblink.treetype)
 		ebuild_phase.start()
 		ebuild_phase.wait()
 
@@ -586,7 +596,7 @@ class Scheduler(PollScheduler):
 			if ebuild_path is None:
 				raise AssertionError("ebuild not found for '%s'" % x.cpv)
 			pkgsettings['O'] = os.path.dirname(ebuild_path)
-			if not portage.digestgen([], pkgsettings, myportdb=portdb):
+			if not digestgen(mysettings=pkgsettings, myportdb=portdb):
 				writemsg_level(
 					"!!! Unable to generate manifest for '%s'.\n" \
 					% x.cpv, level=logging.ERROR, noiselevel=-1)
@@ -632,7 +642,7 @@ class Scheduler(PollScheduler):
 			if ebuild_path is None:
 				raise AssertionError("ebuild not found for '%s'" % x.cpv)
 			quiet_config["O"] = os.path.dirname(ebuild_path)
-			if not portage.digestcheck([], quiet_config, strict=True):
+			if not digestcheck([], quiet_config, strict=True):
 				failures |= 1
 
 		if failures:
@@ -1485,23 +1495,6 @@ class Scheduler(PollScheduler):
 		"""
 		print(colorize("GOOD", "*** Resuming merge..."))
 
-		if self._show_list():
-			if "--tree" in self.myopts:
-				portage.writemsg_stdout("\n" + \
-					darkgreen("These are the packages that " + \
-					"would be merged, in reverse order:\n\n"))
-
-			else:
-				portage.writemsg_stdout("\n" + \
-					darkgreen("These are the packages that " + \
-					"would be merged, in order:\n\n"))
-
-		show_spinner = "--quiet" not in self.myopts and \
-			"--nodeps" not in self.myopts
-
-		if show_spinner:
-			print("Calculating dependencies  ", end=' ')
-
 		myparams = create_depgraph_params(self.myopts, None)
 		success = False
 		e = None
@@ -1517,9 +1510,6 @@ class Scheduler(PollScheduler):
 			mydepgraph = e.depgraph
 			dropped_tasks = set()
 
-		if show_spinner:
-			print("\b\b... done!")
-
 		if e is not None:
 			def unsatisfied_resume_dep_msg():
 				mydepgraph.display_problems()
@@ -1624,7 +1614,7 @@ class Scheduler(PollScheduler):
 		logger = self._logger
 		pkg_count = self._pkg_count
 		root_config = pkg.root_config
-		world_set = root_config.sets["world"]
+		world_set = root_config.sets["selected"]
 		world_locked = False
 		if hasattr(world_set, "lock"):
 			world_set.lock()
