@@ -1,6 +1,6 @@
 # Copyright 1999-2009 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: actions.py 14549 2009-10-10 03:08:40Z zmedico $
+# $Id: actions.py 15687 2010-03-02 21:07:53Z zmedico $
 
 from __future__ import print_function
 
@@ -24,8 +24,10 @@ from itertools import chain
 import portage
 from portage import os
 from portage import digraph
+from portage import _unicode_decode
 from portage.cache.cache_errors import CacheError
 from portage.const import NEWS_LIB_PATH
+from portage.dbapi.dep_expand import dep_expand
 from portage.output import blue, bold, colorize, create_color_func, darkgreen, \
 	red, yellow
 good = create_color_func("GOOD")
@@ -33,6 +35,7 @@ bad = create_color_func("BAD")
 from portage._sets import load_default_config, SETPREFIX
 from portage._sets.base import InternalPackageSet
 from portage.util import cmp_sort_key, writemsg, writemsg_level
+from portage._global_updates import _global_updates
 
 from _emerge.clear_caches import clear_caches
 from _emerge.countdown import countdown
@@ -134,6 +137,7 @@ def action_build(settings, trees, mtimedb,
 	pretend = "--pretend" in myopts
 	fetchonly = "--fetchonly" in myopts or "--fetch-all-uri" in myopts
 	ask = "--ask" in myopts
+	enter_invalid = '--ask-enter-invalid' in myopts
 	nodeps = "--nodeps" in myopts
 	oneshot = "--oneshot" in myopts or "--onlydeps" in myopts
 	tree = "--tree" in myopts
@@ -165,36 +169,10 @@ def action_build(settings, trees, mtimedb,
 			writemsg("%s%s\n" % (prefix, line))
 		writemsg(prefix + "\n")
 
-	if "--quiet" not in myopts and \
-		("--pretend" in myopts or "--ask" in myopts or \
-		"--tree" in myopts or "--verbose" in myopts):
-		action = ""
-		if "--fetchonly" in myopts or "--fetch-all-uri" in myopts:
-			action = "fetched"
-		elif "--buildpkgonly" in myopts:
-			action = "built"
-		else:
-			action = "merged"
-		if "--tree" in myopts and action != "fetched": # Tree doesn't work with fetching
-			print()
-			print(darkgreen("These are the packages that would be %s, in reverse order:") % action)
-			print()
-		else:
-			print()
-			print(darkgreen("These are the packages that would be %s, in order:") % action)
-			print()
-
-	show_spinner = "--quiet" not in myopts and "--nodeps" not in myopts
-	if not show_spinner:
-		spinner.update = spinner.update_quiet
-
 	if resume:
 		favorites = mtimedb["resume"].get("favorites")
 		if not isinstance(favorites, list):
 			favorites = []
-
-		if show_spinner:
-			print("Calculating dependencies  ", end=' ')
 		myparams = create_depgraph_params(myopts, myaction)
 
 		resume_data = mtimedb["resume"]
@@ -215,8 +193,7 @@ def action_build(settings, trees, mtimedb,
 			depgraph.UnsatisfiedResumeDep) as e:
 			if isinstance(e, depgraph.UnsatisfiedResumeDep):
 				mydepgraph = e.depgraph
-			if show_spinner:
-				print()
+
 			from textwrap import wrap
 			from portage.output import EOutput
 			out = EOutput()
@@ -268,9 +245,6 @@ def action_build(settings, trees, mtimedb,
 					"the operation manually."
 				for line in wrap(msg, 72):
 					out.eerror(line)
-		else:
-			if show_spinner:
-				print("\b\b... done!")
 
 		if success:
 			if dropped_tasks:
@@ -298,24 +272,15 @@ def action_build(settings, trees, mtimedb,
 			print(darkgreen("emerge: It seems we have nothing to resume..."))
 			return os.EX_OK
 
-		if "--quiet" not in myopts and "--nodeps" not in myopts:
-			print("Calculating dependencies  ", end=' ')
-			sys.stdout.flush()
-
 		myparams = create_depgraph_params(myopts, myaction)
 		try:
 			success, mydepgraph, favorites = backtrack_depgraph(
 				settings, trees, myopts, myparams, myaction, myfiles, spinner)
 		except portage.exception.PackageSetNotFound as e:
-			if show_spinner:
-				print("\b\b... done!")
 			root_config = trees[settings["ROOT"]]["root_config"]
 			display_missing_pkg_set(root_config, e.value)
 			return 1
 
-		if show_spinner:
-			print("\b\b... done!")
-
 		if not success:
 			mydepgraph.display_problems()
 			return 1
@@ -379,7 +344,7 @@ def action_build(settings, trees, mtimedb,
 			else:
 				prompt="Would you like to merge these packages?"
 		print()
-		if "--ask" in myopts and userquery(prompt) == "No":
+		if "--ask" in myopts and userquery(prompt, enter_invalid) == "No":
 			print()
 			print("Quitting.")
 			print()
@@ -479,6 +444,7 @@ def action_build(settings, trees, mtimedb,
 		return retval
 
 def action_config(settings, trees, myopts, myfiles):
+	enter_invalid = '--ask-enter-invalid' in myopts
 	if len(myfiles) != 1:
 		print(red("!!! config can only take a single package atom at this time\n"))
 		sys.exit(1)
@@ -508,7 +474,7 @@ def action_config(settings, trees, myopts, myfiles):
 				print(options[-1]+") "+pkg)
 			print("X) Cancel")
 			options.append("X")
-			idx = userquery("Selection?", options)
+			idx = userquery("Selection?", enter_invalid, responses=options)
 			if idx == "X":
 				sys.exit(0)
 			pkg = pkgs[int(idx)-1]
@@ -523,7 +489,7 @@ def action_config(settings, trees, myopts, myfiles):
 
 	print()
 	if "--ask" in myopts:
-		if userquery("Ready to configure "+pkg+"?") == "No":
+		if userquery("Ready to configure %s?" % pkg, enter_invalid) == "No":
 			sys.exit(0)
 	else:
 		print("Configuring pkg...")
@@ -617,7 +583,7 @@ def action_depclean(settings, trees, ldpath_mtimes,
 
 	print("Packages installed:   " + str(len(vardb.cpv_all())))
 	print("Packages in world:    " + \
-		str(len(root_config.sets["world"].getAtoms())))
+		str(len(root_config.sets["selected"].getAtoms())))
 	print("Packages in system:   " + \
 		str(len(root_config.sets["system"].getAtoms())))
 	print("Required packages:    "+str(req_pkg_count))
@@ -633,32 +599,47 @@ def calc_depclean(settings, trees, ldpath_mtimes,
 	xterm_titles = "notitles" not in settings.features
 	myroot = settings["ROOT"]
 	root_config = trees[myroot]["root_config"]
-	getSetAtoms = root_config.setconfig.getSetAtoms
-	vardb = trees[myroot]["vartree"].dbapi
+	psets = root_config.setconfig.psets
 	deselect = myopts.get('--deselect') != 'n'
 
-	required_set_names = ("system", "world")
+	required_set_stack = ["world"]
 	required_sets = {}
 	set_args = []
 
-	for s in required_set_names:
-		required_sets[s] = InternalPackageSet(
-			initial_atoms=getSetAtoms(s))
+	# Recursively create InternalPackageSet instances for world
+	# and any sets nested within it.
+	while required_set_stack:
+		s = required_set_stack.pop()
+		if s in required_sets:
+			continue
+		pset = psets.get(s)
+		if pset is not None:
+			required_sets[s] = InternalPackageSet(
+				initial_atoms=pset.getAtoms())
+			for n in pset.getNonAtoms():
+				if n.startswith(SETPREFIX):
+					required_set_stack.append(n[len(SETPREFIX):])
 
-
-	# When removing packages, use a temporary version of world
-	# which excludes packages that are intended to be eligible for
+	# When removing packages, use a temporary version of world 'selected'
+	# set which excludes packages that are intended to be eligible for
 	# removal.
-	world_temp_set = required_sets["world"]
-	system_set = required_sets["system"]
+	selected_set = required_sets["selected"]
+	# Rename 'selected' to 'world', since 'selected' is currently
+	# considered to be an internal set and we don't want it to be
+	# displayed to the user as a reverse dependency.
+	required_sets["world"] = required_sets.pop("selected")
+	protected_set = InternalPackageSet()
+	protected_set_name = '____depclean_protected_set____'
+	required_sets[protected_set_name] = protected_set
+	system_set = required_sets.get("system")
 
-	if not system_set or not world_temp_set:
+	if not system_set or not selected_set:
 
 		if not system_set:
 			writemsg_level("!!! You have no system list.\n",
 				level=logging.ERROR, noiselevel=-1)
 
-		if not world_temp_set:
+		if not selected_set:
 			writemsg_level("!!! You have no world file.\n",
 					level=logging.WARNING, noiselevel=-1)
 
@@ -674,6 +655,7 @@ def calc_depclean(settings, trees, ldpath_mtimes,
 	writemsg_level("\nCalculating dependencies  ")
 	resolver_params = create_depgraph_params(myopts, "remove")
 	resolver = depgraph(settings, trees, myopts, resolver_params, spinner)
+	resolver._load_vdb()
 	vardb = resolver._frozen_config.trees[myroot]["vartree"].dbapi
 
 	if action == "depclean":
@@ -681,7 +663,7 @@ def calc_depclean(settings, trees, ldpath_mtimes,
 		if args_set:
 
 			if deselect:
-				world_temp_set.clear()
+				selected_set.clear()
 
 			# Pull in everything that's installed but not matched
 			# by an argument atom since we don't want to clean any
@@ -691,23 +673,23 @@ def calc_depclean(settings, trees, ldpath_mtimes,
 
 				try:
 					if args_set.findAtomForPackage(pkg) is None:
-						world_temp_set.add("=" + pkg.cpv)
+						protected_set.add("=" + pkg.cpv)
 						continue
 				except portage.exception.InvalidDependString as e:
 					show_invalid_depstring_notice(pkg,
 						pkg.metadata["PROVIDE"], str(e))
 					del e
-					world_temp_set.add("=" + pkg.cpv)
+					protected_set.add("=" + pkg.cpv)
 					continue
 
 	elif action == "prune":
 
 		if deselect:
-			world_temp_set.clear()
+			selected_set.clear()
 
 		# Pull in everything that's installed since we don't
 		# to prune a package if something depends on it.
-		world_temp_set.update(vardb.cp_all())
+		protected_set.update(vardb.cp_all())
 
 		if not args_set:
 
@@ -730,7 +712,7 @@ def calc_depclean(settings, trees, ldpath_mtimes,
 			highest_version = pkgs_for_cp[-1]
 			if pkg == highest_version:
 				# pkg is the highest version
-				world_temp_set.add("=" + pkg.cpv)
+				protected_set.add("=" + pkg.cpv)
 				continue
 
 			if len(pkgs_for_cp) <= 1:
@@ -740,27 +722,16 @@ def calc_depclean(settings, trees, ldpath_mtimes,
 
 			try:
 				if args_set.findAtomForPackage(pkg) is None:
-					world_temp_set.add("=" + pkg.cpv)
+					protected_set.add("=" + pkg.cpv)
 					continue
 			except portage.exception.InvalidDependString as e:
 				show_invalid_depstring_notice(pkg,
 					pkg.metadata["PROVIDE"], str(e))
 				del e
-				world_temp_set.add("=" + pkg.cpv)
+				protected_set.add("=" + pkg.cpv)
 				continue
 
-	set_args = {}
-	for s, package_set in required_sets.items():
-		set_atom = SETPREFIX + s
-		set_arg = SetArg(arg=set_atom, set=package_set,
-			root_config=resolver._frozen_config.roots[myroot])
-		set_args[s] = set_arg
-		for atom in set_arg.set:
-			resolver._dynamic_config._dep_stack.append(
-				Dependency(atom=atom, root=myroot, parent=set_arg))
-			resolver._dynamic_config.digraph.add(set_arg, None)
-
-	success = resolver._complete_graph()
+	success = resolver._complete_graph(required_sets={myroot:required_sets})
 	writemsg_level("\b\b... done!\n")
 
 	resolver.display_problems()
@@ -844,6 +815,13 @@ def calc_depclean(settings, trees, ldpath_mtimes,
 			return -1
 
 	def create_cleanlist():
+
+		# Never display the special internal protected_set.
+		for node in graph:
+			if isinstance(node, SetArg) and node.name == protected_set_name:
+				graph.remove(node)
+				break
+
 		pkgs_to_remove = []
 
 		if action == "depclean":
@@ -871,9 +849,6 @@ def calc_depclean(settings, trees, ldpath_mtimes,
 						show_parents(pkg)
 
 		elif action == "prune":
-			# Prune really uses all installed instead of world. It's not
-			# a real reverse dependency so don't display it as such.
-			graph.remove(set_args["world"])
 
 			for atom in args_set:
 				for pkg in vardb.match_pkgs(atom):
@@ -996,10 +971,11 @@ def calc_depclean(settings, trees, ldpath_mtimes,
 	return 0, [], False, required_pkgs_total
 
 def action_deselect(settings, trees, opts, atoms):
+	enter_invalid = '--ask-enter-invalid' in opts
 	root_config = trees[settings['ROOT']]['root_config']
-	world_set = root_config.sets['world']
+	world_set = root_config.sets['selected']
 	if not hasattr(world_set, 'update'):
-		writemsg_level("World set does not appear to be mutable.\n",
+		writemsg_level("World @selected set does not appear to be mutable.\n",
 			level=logging.ERROR, noiselevel=-1)
 		return 1
 
@@ -1038,7 +1014,7 @@ def action_deselect(settings, trees, opts, atoms):
 			if '--ask' in opts:
 				prompt = "Would you like to remove these " + \
 					"packages from your world favorites?"
-				if userquery(prompt) == 'No':
+				if userquery(prompt, enter_invalid) == 'No':
 					return os.EX_OK
 
 			remaining = set(world_set)
@@ -1119,7 +1095,8 @@ def action_info(settings, trees, myopts, myfiles):
 		myvars = ['GENTOO_MIRRORS', 'CONFIG_PROTECT', 'CONFIG_PROTECT_MASK',
 		          'PORTDIR', 'DISTDIR', 'PKGDIR', 'PORTAGE_TMPDIR',
 		          'PORTDIR_OVERLAY', 'USE', 'CHOST', 'CFLAGS', 'CXXFLAGS',
-		          'ACCEPT_KEYWORDS', 'SYNC', 'FEATURES', 'EMERGE_DEFAULT_OPTS']
+		          'ACCEPT_KEYWORDS', 'ACCEPT_LICENSE', 'SYNC', 'FEATURES',
+		          'EMERGE_DEFAULT_OPTS']
 
 		myvars.extend(portage.util.grabfile(settings["PORTDIR"]+"/profiles/info_vars"))
 
@@ -1316,8 +1293,10 @@ def action_metadata(settings, portdb, myopts, porttrees=None):
 
 	porttrees = [tree_data.path for tree_data in porttrees_data]
 
-	isatty = sys.stdout.isatty()
-	quiet = not isatty or '--quiet' in myopts
+	quiet = settings.get('TERM') == 'dumb' or \
+		'--quiet' in myopts or \
+		not sys.stdout.isatty()
+
 	onProgress = None
 	if not quiet:
 		progressBar = portage.output.TermProgressBar()
@@ -1533,6 +1512,7 @@ def action_search(root_config, myopts, myfiles, spinner):
 			searchinstance.output()
 
 def action_sync(settings, trees, mtimedb, myopts, myaction):
+	enter_invalid = '--ask-enter-invalid' in myopts
 	xterm_titles = "notitles" not in settings.features
 	emergelog(xterm_titles, " === sync")
 	portdb = trees[settings["ROOT"]]["porttree"].dbapi
@@ -1778,7 +1758,9 @@ def action_sync(settings, trees, mtimedb, myopts, myaction):
 
 			if (retries==0):
 				if "--ask" in myopts:
-					if userquery("Do you want to sync your Portage tree with the mirror at\n" + blue(dosyncuri) + bold("?"))=="No":
+					if userquery("Do you want to sync your Portage tree " + \
+						"with the mirror at\n" + blue(dosyncuri) + bold("?"),
+						enter_invalid) == "No":
 						print()
 						print("Quitting.")
 						print()
@@ -1991,6 +1973,7 @@ def action_sync(settings, trees, mtimedb, myopts, myaction):
 
 	# Reload the whole config from scratch.
 	settings, trees, mtimedb = load_emerge_config(trees=trees)
+	adjust_configs(myopts, trees)
 	root_config = trees[settings["ROOT"]]["root_config"]
 	portdb = trees[settings["ROOT"]]["porttree"].dbapi
 
@@ -2001,10 +1984,11 @@ def action_sync(settings, trees, mtimedb, myopts, myaction):
 		# the only one that's been synced here.
 		action_metadata(settings, portdb, myopts, porttrees=[myportdir])
 
-	if portage._global_updates(trees, mtimedb["updates"]):
+	if _global_updates(trees, mtimedb["updates"]):
 		mtimedb.commit()
 		# Reload the whole config from scratch.
 		settings, trees, mtimedb = load_emerge_config(trees=trees)
+		adjust_configs(myopts, trees)
 		portdb = trees[settings["ROOT"]]["porttree"].dbapi
 		root_config = trees[settings["ROOT"]]["root_config"]
 
@@ -2054,7 +2038,7 @@ def action_uninstall(settings, trees, ldpath_mtimes,
 
 			try:
 				valid_atoms.append(
-					portage.dep_expand(x, mydb=vardb, settings=settings))
+					dep_expand(x, mydb=vardb, settings=settings))
 			except portage.exception.AmbiguousPackageName as e:
 				msg = "The short ebuild name \"" + x + \
 					"\" is ambiguous.  Please specify " + \
@@ -2122,6 +2106,17 @@ def action_uninstall(settings, trees, ldpath_mtimes,
 	if files and not valid_atoms:
 		return 1
 
+	if action == 'unmerge' and \
+		'--quiet' not in opts and \
+		'--quiet-unmerge-warn' not in opts:
+		msg = "This action can remove important packages! " + \
+			"In order to be safer, use " + \
+			"`emerge -pv --depclean <atom>` to check for " + \
+			"reverse dependencies before removing packages."
+		out = portage.output.EOutput()
+		for line in textwrap.wrap(msg, 72):
+			out.ewarn(line)
+
 	if action in ('clean', 'unmerge') or \
 		(action == 'prune' and "--nodeps" in opts):
 		# When given a list of atoms, unmerge them in the order given.
@@ -2137,6 +2132,13 @@ def action_uninstall(settings, trees, ldpath_mtimes,
 
 	return rval
 
+def adjust_configs(myopts, trees):
+	for myroot in trees:
+		mysettings =  trees[myroot]["vartree"].settings
+		mysettings.unlock()
+		adjust_config(myopts, mysettings)
+		mysettings.lock()
+
 def adjust_config(myopts, settings):
 	"""Make emerge specific adjustments to the config."""
 
@@ -2180,7 +2182,7 @@ def adjust_config(myopts, settings):
 	settings["EMERGE_WARNING_DELAY"] = str(EMERGE_WARNING_DELAY)
 	settings.backup_changes("EMERGE_WARNING_DELAY")
 
-	if "--quiet" in myopts:
+	if "--quiet" in myopts or "--quiet-build" in myopts:
 		settings["PORTAGE_QUIET"]="1"
 		settings.backup_changes("PORTAGE_QUIET")
 
@@ -2226,7 +2228,8 @@ def adjust_config(myopts, settings):
 			portage.output.havecolor = 0
 			settings["NOCOLOR"] = "true"
 		settings.backup_changes("NOCOLOR")
-	elif not sys.stdout.isatty() and settings.get("NOCOLOR") != "no":
+	elif settings.get('TERM') == 'dumb' or \
+		not sys.stdout.isatty():
 		portage.output.havecolor = 0
 		settings["NOCOLOR"] = "true"
 		settings.backup_changes("NOCOLOR")
@@ -2313,7 +2316,7 @@ def git_sync_timestamps(settings, portdir):
 		portage._shell_quote(portdir)]
 	import subprocess
 	proc = subprocess.Popen(args, stdout=subprocess.PIPE)
-	modified_files = set(l.rstrip("\n") for l in proc.stdout)
+	modified_files = set(_unicode_decode(l).rstrip("\n") for l in proc.stdout)
 	rval = proc.wait()
 	if rval != os.EX_OK:
 		return rval
@@ -2403,7 +2406,7 @@ def git_sync_timestamps(settings, portdir):
 			if ec in updated_ec_mtimes:
 				continue
 			ec_path = os.path.join(ec_dir, ec + ".eclass")
-			current_mtime = long(os.stat(ec_path).st_mtime)
+			current_mtime = os.stat(ec_path)[stat.ST_MTIME]
 			if current_mtime != ec_mtime:
 				os.utime(ec_path, (ec_mtime, ec_mtime))
 			updated_ec_mtimes[ec] = ec_mtime
@@ -2420,6 +2423,7 @@ def load_emerge_config(trees=None):
 
 	for root, root_trees in trees.items():
 		settings = root_trees["vartree"].settings
+		settings._init_dirs()
 		setconfig = load_default_config(settings, root_trees)
 		root_trees["root_config"] = RootConfig(settings, root_trees, setconfig)
 
