@@ -1,9 +1,10 @@
 # Copyright 1999-2009 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: clear_caches.py 14538 2009-10-10 01:33:10Z zmedico $
+# $Id: clear_caches.py 15686 2010-03-02 21:07:13Z zmedico $
 
 import gc
 import portage
+from portage.util.listdir import dircache
 
 def clear_caches(trees):
 	for d in trees.values():
@@ -11,5 +12,5 @@ def clear_caches(trees):
 		d["porttree"].dbapi._aux_cache.clear()
 		d["bintree"].dbapi._aux_cache.clear()
 		d["bintree"].dbapi._clear_cache()
-	portage.dircache.clear()
+	dircache.clear()
 	gc.collect()
