@@ -1,6 +1,6 @@
 # Copyright 1999-2009 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: help.py 14556 2009-10-10 10:49:59Z zmedico $
+# $Id: help.py 15687 2010-03-02 21:07:53Z zmedico $
 
 from __future__ import print_function
 
@@ -42,7 +42,7 @@ def help(myopts, havecolor=1):
 		print("              emerge to display detailed help.")
 		print()
 		print(turquoise("Actions:"))
-		print("       "+green("--clean")+" ("+green("-c")+" short option)")
+		print("       "+green("--clean"))
 		print("              Cleans the system by removing outdated packages which will not")
 		print("              remove functionalities or prevent your system from working.")
 		print("              The arguments can be in several different formats :")
@@ -66,7 +66,7 @@ def help(myopts, havecolor=1):
 		print("              emerge process has completed.  This usually entails configuration")
 		print("              file setup or other similar setups that the user may wish to run.")
 		print()
-		print("       "+green("--depclean"))
+		print("       "+green("--depclean")+" ("+green("-c")+" short option)")
 
 		paragraph = "Cleans the system by removing packages that are " + \
 		"not associated with explicitly merged packages. Depclean works " + \
@@ -252,16 +252,36 @@ def help(myopts, havecolor=1):
 		print("              be displayed as USE=\"-bar dar -foo\"")
 		print()
 		print("       "+green("--ask")+" ("+green("-a")+" short option)")
-		print("              before performing the merge, display what ebuilds and tbz2s will")
-		print("              be installed, in the same format as when using --pretend; then")
-		print("              ask whether to continue with the merge or abort. Using --ask is")
-		print("              more efficient than using --pretend and then executing the same")
-		print("              command without --pretend, as dependencies will only need to be")
-		print("              calculated once. WARNING: If the \"Enter\" key is pressed at the")
-		print("              prompt (with no other input), it is interpreted as acceptance of")
-		print("              the first choice.  Note that the input buffer is not cleared prior")
-		print("              to the prompt, so an accidental press of the \"Enter\" key at any")
-		print("              time prior to the prompt will be interpreted as a choice!")
+		desc = "Before performing the action, display what will take place (server info for " + \
+			"--sync, --pretend output for merge, and so forth), then ask " + \
+			"whether to proceed with the action or abort.  Using --ask is more " + \
+			"efficient than using --pretend and then executing the same command " + \
+			"without --pretend, as dependencies will only need to be calculated once. " + \
+			"WARNING: If the \"Enter\" key is pressed at the prompt (with no other input), " + \
+			"it is interpreted as acceptance of the first choice.  Note that the input " + \
+			"buffer is not cleared prior to the prompt, so an accidental press of the " + \
+			"\"Enter\" key at any time prior to the prompt will be interpreted as a choice! " + \
+			"Use the --ask-enter-invalid option if you want a single \"Enter\" key " + \
+			"press to be interpreted as invalid input."
+		for line in wrap(desc, desc_width):
+			print(desc_indent + line)
+		print()
+		print("        " + green("--ask-enter-invalid"))
+		desc = "When used together with the --ask option, " + \
+			"interpret a single \"Enter\" key press as " + \
+			"invalid input. This helps prevent accidental " + \
+			"acceptance of the first choice. This option is " + \
+			"intended to be set in the make.conf(5) " + \
+			"EMERGE_DEFAULT_OPTS variable."
+		for line in wrap(desc, desc_width):
+			print(desc_indent + line)
+		print()
+		print("       " + green("--backtrack") + " " + turquoise("COUNT"))
+		desc = "Specifies an integer number of times to backtrack if " + \
+			"dependency calculation fails due to a conflict or an " + \
+			"unsatisfied dependency (default: '5')."
+		for line in wrap(desc, desc_width):
+			print(desc_indent + line)
 		print()
 		print("        " + green("--binpkg-respect-use") + \
 			" < " + turquoise("y") + " | " + turquoise("n") + " >")
@@ -419,8 +439,11 @@ def help(myopts, havecolor=1):
 			print(desc_indent + line)
 		print()
 		print("       "+green("--newuse")+" ("+green("-N")+" short option)")
-		print("              Tells emerge to include installed packages where USE flags have ")
-		print("              changed since installation.")
+		desc = "Tells emerge to include installed packages where USE " + \
+			"flags have changed since compilation. This option " + \
+			"also implies the --selective option."
+		for line in wrap(desc, desc_width):
+			print(desc_indent + line)
 		print()
 		print("       "+green("--noconfmem"))
 		print("              Portage keeps track of files that have been placed into")
@@ -474,6 +497,30 @@ def help(myopts, havecolor=1):
 		print("              Effects vary, but the general outcome is a reduced or condensed")
 		print("              output from portage's displays.")
 		print()
+		print("       "+green("--quiet-build"))
+		desc = "Redirect all build output to logs alone, and do not " + \
+			"display it on stdout."
+		for line in wrap(desc, desc_width):
+			print(desc_indent + line)
+		print()
+		print("       "+green("--quiet-unmerge-warn"))
+		desc = "Disable the warning message that's shown prior to " + \
+			"--unmerge actions. This option is intended " + \
+			"to be set in the make.conf(5) " + \
+			"EMERGE_DEFAULT_OPTS variable."
+		for line in wrap(desc, desc_width):
+			print(desc_indent + line)
+		print()
+		print("       "+green("--rebuilt-binaries") + "[=%s]" % turquoise("n"))
+		desc = "Replace installed packages with binary packages that have " + \
+			"been rebuilt. Rebuilds are detected by comparison of " + \
+			"BUILD_TIME package metadata. This option is enabled " + \
+			"automatically when using binary packages " + \
+			"(--usepkgonly or --getbinpkgonly) together with " + \
+			"--update and --deep."
+		for line in wrap(desc, desc_width):
+			print(desc_indent + line)
+		print()
 		print("       "+green("--reinstall ") + turquoise("changed-use"))
 		print("              Tells emerge to include installed packages where USE flags have")
 		print("              changed since installation.  Unlike --newuse, this option does")
@@ -499,6 +546,14 @@ def help(myopts, havecolor=1):
 		for line in wrap(desc, desc_width):
 			print(desc_indent + line)
 		print()
+		print("       " + green("--select") + "[=%s]" % turquoise("n"))
+		desc = "Add specified packages to the world set (inverse of " + \
+			"--oneshot). This is useful if you want to " + \
+			"use EMERGE_DEFAULT_OPTS to make " + \
+			"--oneshot behavior default."
+		for line in wrap(desc, desc_width):
+			print(desc_indent + line)
+		print()
 		print("       " + green("--selective") + "[=%s]" % turquoise("n"))
 		desc = "This is similar to the --noreplace option, except that it " + \
 			"does not take precedence over options such as --newuse. " + \
@@ -527,6 +582,21 @@ def help(myopts, havecolor=1):
 		print("              a package's dependencies follow the package. Only really useful")
 		print("              in combination with --emptytree, --update or --deep.")
 		print()
+		print("       " + green("--unordered-display"))
+		desc = "By default the displayed merge list is sorted using the " + \
+			"order in which the packages will be merged. When " + \
+			"--tree is used together with this option, this " + \
+			"constraint is removed, hopefully leading to a more " + \
+			"readable dependency tree."
+		for line in wrap(desc, desc_width):
+			print(desc_indent + line)
+		print()
+		print("       " + green("--use-ebuild-visibility") + "[=%s]" % turquoise("n"))
+		desc = "Use unbuilt ebuild metadata for visibility " + \
+			"checks on built packages."
+		for line in wrap(desc, desc_width):
+			print(desc_indent + line)
+		print()
 		print("       "+green("--usepkg")+ "[=%s]" % turquoise("n") + " ("+green("-k")+" short option)")
 		print("              Tell emerge to use binary packages (from $PKGDIR) if they are")
 		print("              available, thus possibly avoiding some time-consuming compiles.")
