@@ -1,11 +1,12 @@
 # Copyright 1999-2009 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: main.py 14549 2009-10-10 03:08:40Z zmedico $
+# $Id: main.py 15687 2010-03-02 21:07:53Z zmedico $
 
 from __future__ import print_function
 
 import logging
 import signal
+import stat
 import sys
 import textwrap
 import platform
@@ -31,14 +32,17 @@ import portage.util
 import portage.locks
 import portage.exception
 from portage.data import secpass
+from portage.dbapi.dep_expand import dep_expand
 from portage.util import normalize_path as normpath
 from portage.util import writemsg, writemsg_level, writemsg_stdout
 from portage._sets import SETPREFIX
+from portage._global_updates import _global_updates
 
 from _emerge.actions import action_config, action_sync, action_metadata, \
 	action_regen, action_search, action_uninstall, action_info, action_build, \
-	adjust_config, chk_updated_cfg_files, display_missing_pkg_set, \
+	adjust_configs, chk_updated_cfg_files, display_missing_pkg_set, \
 	display_news_notification, getportageversion, load_emerge_config
+import _emerge
 from _emerge.emergelog import emergelog
 from _emerge._flush_elog_mod_echo import _flush_elog_mod_echo
 from _emerge.is_valid_package_atom import is_valid_package_atom
@@ -49,6 +53,7 @@ if sys.hexversion >= 0x3000000:
 
 options=[
 "--ask",          "--alphabetical",
+"--ask-enter-invalid",
 "--buildpkg",     "--buildpkgonly",
 "--changed-use",
 "--changelog",    "--columns",
@@ -62,10 +67,14 @@ options=[
 "--nodeps",       "--noreplace",
 "--nospinner",    "--oneshot",
 "--onlydeps",     "--pretend",
-"--quiet",        "--resume",
+"--quiet",
+"--quiet-build",
+"--quiet-unmerge-warn",
+"--resume",
 "--searchdesc",
 "--skipfirst",
 "--tree",
+"--unordered-display",
 "--update",
 "--verbose",
 ]
@@ -74,7 +83,8 @@ shortmapping={
 "1":"--oneshot",
 "a":"--ask",
 "b":"--buildpkg",  "B":"--buildpkgonly",
-"c":"--clean",     "C":"--unmerge",
+"c":"--depclean",
+"C":"--unmerge",
 "d":"--debug",
 "e":"--emptytree",
 "f":"--fetchonly", "F":"--fetch-all-uri",
@@ -101,7 +111,7 @@ def chk_updated_info_files(root, infodirs, prev_mtimes, retval):
 				continue
 			inforoot=normpath(root+z)
 			if os.path.isdir(inforoot):
-				infomtime = long(os.stat(inforoot).st_mtime)
+				infomtime = os.stat(inforoot)[stat.ST_MTIME]
 				if inforoot not in prev_mtimes or \
 					prev_mtimes[inforoot] != infomtime:
 						regen_infodirs.append(inforoot)
@@ -191,7 +201,7 @@ def chk_updated_info_files(root, infodirs, prev_mtimes, retval):
 						del e
 
 				#update mtime so we can potentially avoid regenerating.
-				prev_mtimes[inforoot] = long(os.stat(inforoot).st_mtime)
+				prev_mtimes[inforoot] = os.stat(inforoot)[stat.ST_MTIME]
 
 			if badcount:
 				out.eerror("Processed %d info files; %d errors." % \
@@ -310,8 +320,11 @@ def insert_optional_args(args):
 		'--getbinpkgonly'        : ('n',),
 		'--jobs'       : valid_integers,
 		'--keep-going'           : ('n',),
+		'--rebuilt-binaries'     : ('n',),
 		'--root-deps'  : ('rdeps',),
+		'--select'               : ('n',),
 		'--selective'            : ('n',),
+		"--use-ebuild-visibility": ('n',),
 		'--usepkg'               : ('n',),
 		'--usepkgonly'           : ('n',),
 	}
@@ -427,6 +440,15 @@ def parse_opts(tmpcmdline, silent=False):
 			"help":"temporarily override ACCEPT_PROPERTIES",
 			"action":"store"
 		},
+
+		"--backtrack": {
+
+			"help"   : "Specifies how many times to backtrack if dependency " + \
+				"calculation fails ",
+
+			"action" : "store"
+		},
+
 		"--config-root": {
 			"help":"specify the location for portage configuration files",
 			"action":"store"
@@ -524,6 +546,13 @@ def parse_opts(tmpcmdline, silent=False):
 			"choices"  : ("True", "n")
 		},
 
+		"--rebuilt-binaries": {
+			"help"     : "replace installed packages with binary " + \
+			             "packages that have been rebuilt",
+			"type"     : "choice",
+			"choices"  : ("True", "n")
+		},
+
 		"--root": {
 		 "help"   : "specify the target root filesystem for merging packages",
 		 "action" : "store"
@@ -535,6 +564,13 @@ def parse_opts(tmpcmdline, silent=False):
 			"choices" :("True", "rdeps")
 		},
 
+		"--select": {
+			"help"    : "add specified packages to the world set " + \
+			            "(inverse of --oneshot)",
+			"type"    : "choice",
+			"choices" : ("True", "n")
+		},
+
 		"--selective": {
 			"help"    : "similar to the --noreplace but does not take " + \
 			            "precedence over options such as --newuse",
@@ -542,6 +578,12 @@ def parse_opts(tmpcmdline, silent=False):
 			"choices" : ("True", "n")
 		},
 
+		"--use-ebuild-visibility": {
+			"help"     : "use unbuilt ebuild metadata for visibility checks on built packages",
+			"type"     : "choice",
+			"choices"  : ("True", "n")
+		},
+
 		"--usepkg": {
 			"shortopt" : "-k",
 			"help"     : "use binary packages",
@@ -623,12 +665,36 @@ def parse_opts(tmpcmdline, silent=False):
 	else:
 		myoptions.keep_going = None
 
+	if myoptions.rebuilt_binaries in ("True",):
+		myoptions.rebuilt_binaries = True
+
 	if myoptions.root_deps == "True":
 		myoptions.root_deps = True
 
+	if myoptions.select == "True":
+		myoptions.select = True
+		myoptions.oneshot = False
+	elif myoptions.select == "n":
+		myoptions.oneshot = True
+
 	if myoptions.selective == "True":
 		myoptions.selective = True
 
+	if myoptions.backtrack is not None:
+
+		try:
+			backtrack = int(myoptions.backtrack)
+		except (OverflowError, ValueError):
+			backtrack = -1
+
+		if backtrack < 0:
+			backtrack = None
+			if not silent:
+				writemsg("!!! Invalid --backtrack parameter: '%s'\n" % \
+					(myoptions.backtrack,), noiselevel=-1)
+
+		myoptions.backtrack = backtrack
+
 	if myoptions.deep is not None:
 		deep = None
 		if myoptions.deep == "True":
@@ -680,6 +746,11 @@ def parse_opts(tmpcmdline, silent=False):
 
 		myoptions.load_average = load_average
 
+	if myoptions.use_ebuild_visibility in ("True",):
+		myoptions.use_ebuild_visibility = True
+	else:
+		myoptions.use_ebuild_visibility = None
+
 	if myoptions.usepkg in ("True",):
 		myoptions.usepkg = True
 	else:
@@ -765,6 +836,56 @@ def ionice(settings):
 		out.eerror("PORTAGE_IONICE_COMMAND returned %d" % (rval,))
 		out.eerror("See the make.conf(5) man page for PORTAGE_IONICE_COMMAND usage instructions.")
 
+def setconfig_fallback(root_config):
+	from portage._sets.base import DummyPackageSet
+	from portage._sets.files import WorldSelectedSet
+	from portage._sets.profiles import PackagesSystemSet
+	setconfig = root_config.setconfig
+	setconfig.psets['world'] = DummyPackageSet(atoms=['@selected', '@system'])
+	setconfig.psets['selected'] = WorldSelectedSet(root_config.root)
+	setconfig.psets['system'] = \
+		PackagesSystemSet(root_config.settings.profiles)
+	root_config.sets = setconfig.getSets()
+
+def get_missing_sets(root_config):
+	# emerge requires existence of "world", "selected", and "system"
+	missing_sets = []
+
+	for s in ("selected", "system", "world",):
+		if s not in root_config.sets:
+			missing_sets.append(s)
+
+	return missing_sets
+
+def missing_sets_warning(root_config, missing_sets):
+	if len(missing_sets) > 2:
+		missing_sets_str = ", ".join('"%s"' % s for s in missing_sets[:-1])
+		missing_sets_str += ', and "%s"' % missing_sets[-1]
+	elif len(missing_sets) == 2:
+		missing_sets_str = '"%s" and "%s"' % tuple(missing_sets)
+	else:
+		missing_sets_str = '"%s"' % missing_sets[-1]
+	msg = ["emerge: incomplete set configuration, " + \
+		"missing set(s): %s" % missing_sets_str]
+	if root_config.sets:
+		msg.append("        sets defined: %s" % ", ".join(root_config.sets))
+	msg.append("        This usually means that '%s'" % \
+		(os.path.join(portage.const.GLOBAL_CONFIG_PATH, "sets.conf"),))
+	msg.append("        is missing or corrupt.")
+	msg.append("        Falling back to default world and system set configuration!!!")
+	for line in msg:
+		writemsg_level(line + "\n", level=logging.ERROR, noiselevel=-1)
+
+def ensure_required_sets(trees):
+	warning_shown = False
+	for root_trees in trees.values():
+		missing_sets = get_missing_sets(root_trees["root_config"])
+		if missing_sets and not warning_shown:
+			warning_shown = True
+			missing_sets_warning(root_trees["root_config"], missing_sets)
+		if missing_sets:
+			setconfig_fallback(root_trees["root_config"])
+
 def expand_set_arguments(myfiles, myaction, root_config):
 
 	if myaction != "search":
@@ -832,7 +953,7 @@ def repo_name_duplicate_check(trees):
 	for root, root_trees in trees.items():
 		if 'porttree' in root_trees:
 			portdb = root_trees['porttree'].dbapi
-			if portdb.mysettings.get('PORTAGE_REPO_DUPLICATE_WARN') != '0':
+			if portdb.settings.get('PORTAGE_REPO_DUPLICATE_WARN') != '0':
 				for repo_name, paths in portdb._ignored_repos:
 					k = (root, repo_name, portdb.getRepositoryPath(repo_name))
 					ignored_repos.setdefault(k, []).extend(paths)
@@ -864,6 +985,7 @@ def config_protect_check(trees):
 			msg = "!!! CONFIG_PROTECT is empty"
 			if root != "/":
 				msg += " for '%s'" % root
+			msg += "\n"
 			writemsg_level(msg, level=logging.WARN, noiselevel=-1)
 
 def profile_check(trees, myaction):
@@ -883,6 +1005,16 @@ def profile_check(trees, myaction):
 		return 1
 	return os.EX_OK
 
+def check_procfs():
+	procfs_path = '/proc'
+	if platform.system() not in ("Linux",) or \
+		os.path.ismount(procfs_path):
+		return os.EX_OK
+	msg = "It seems that %s is not mounted. You have been warned." % procfs_path
+	writemsg_level("".join("!!! %s\n" % l for l in textwrap.wrap(msg, 70)),
+		level=logging.ERROR, noiselevel=-1)
+	return 1
+
 def emerge_main():
 	global portage	# NFC why this is necessary now - genone
 	portage._disable_legacy_globals()
@@ -911,7 +1043,8 @@ def emerge_main():
 	if rval != os.EX_OK:
 		return rval
 
-	if portage._global_updates(trees, mtimedb["updates"]):
+	if myaction not in ('help', 'info', 'version') and \
+		_global_updates(trees, mtimedb["updates"]):
 		mtimedb.commit()
 		# Reload the whole config from scratch.
 		settings, trees, mtimedb = load_emerge_config(trees=trees)
@@ -934,13 +1067,7 @@ def emerge_main():
 		settings, trees, mtimedb = load_emerge_config(trees=trees)
 		portdb = trees[settings["ROOT"]]["porttree"].dbapi
 
-	for myroot in trees:
-		mysettings =  trees[myroot]["vartree"].settings
-		mysettings.unlock()
-		adjust_config(myopts, mysettings)
-		mysettings.lock()
-		del myroot, mysettings
-
+	adjust_configs(myopts, trees)
 	apply_priorities(settings)
 
 	spinner = stdout_spinner()
@@ -952,11 +1079,45 @@ def emerge_main():
 		#repo_name_check(trees)
 		repo_name_duplicate_check(trees)
 		config_protect_check(trees)
+	check_procfs()
+
+	if "getbinpkg" in settings.features:
+		myopts["--getbinpkg"] = True
+
+	if "--getbinpkgonly" in myopts:
+		myopts["--getbinpkg"] = True
+
+	if "--getbinpkgonly" in myopts:
+		myopts["--usepkgonly"] = True
+
+	if "--getbinpkg" in myopts:
+		myopts["--usepkg"] = True
+
+	if "--usepkgonly" in myopts:
+		myopts["--usepkg"] = True
+
+	if "buildpkg" in settings.features or "--buildpkgonly" in myopts:
+		myopts["--buildpkg"] = True
+
+	if "--buildpkgonly" in myopts:
+		# --buildpkgonly will not merge anything, so
+		# it cancels all binary package options.
+		for opt in ("--getbinpkg", "--getbinpkgonly",
+			"--usepkg", "--usepkgonly"):
+			myopts.pop(opt, None)
 
 	for mytrees in trees.values():
 		mydb = mytrees["porttree"].dbapi
 		# Freeze the portdbapi for performance (memoize all xmatch results).
 		mydb.freeze()
+
+		if "--usepkg" in myopts:
+			# Populate the bintree with current --getbinpkg setting.
+			# This needs to happen before expand_set_arguments(), in case
+			# any sets use the bintree.
+			mytrees["bintree"].populate(
+				getbinpkgs="--getbinpkg" in myopts)
+
 	del mytrees, mydb
 
 	if "moo" in myfiles:
@@ -983,6 +1144,8 @@ def emerge_main():
 
 	root_config = trees[settings["ROOT"]]["root_config"]
 
+	ensure_required_sets(trees)
+
 	# only expand sets for actions taking package arguments
 	oldargs = myfiles[:]
 	if myaction in ("clean", "config", "depclean", "info", "prune", "unmerge", None):
@@ -1010,42 +1173,12 @@ def emerge_main():
 		spinner.update = spinner.update_quiet
 		portage.util.noiselimit = -1
 
-	# Always create packages if FEATURES=buildpkg
-	# Imply --buildpkg if --buildpkgonly
-	if ("buildpkg" in settings.features) or ("--buildpkgonly" in myopts):
-		if "--buildpkg" not in myopts:
-			myopts["--buildpkg"] = True
-
-	# Always try and fetch binary packages if FEATURES=getbinpkg
-	if ("getbinpkg" in settings.features):
-		myopts["--getbinpkg"] = True
-
-	if "--buildpkgonly" in myopts:
-		# --buildpkgonly will not merge anything, so
-		# it cancels all binary package options.
-		for opt in ("--getbinpkg", "--getbinpkgonly",
-			"--usepkg", "--usepkgonly"):
-			myopts.pop(opt, None)
-
 	if "--fetch-all-uri" in myopts:
 		myopts["--fetchonly"] = True
 
 	if "--skipfirst" in myopts and "--resume" not in myopts:
 		myopts["--resume"] = True
 
-	if ("--getbinpkgonly" in myopts) and not ("--usepkgonly" in myopts):
-		myopts["--usepkgonly"] = True
-
-	if ("--getbinpkgonly" in myopts) and not ("--getbinpkg" in myopts):
-		myopts["--getbinpkg"] = True
-
-	if ("--getbinpkg" in myopts) and not ("--usepkg" in myopts):
-		myopts["--usepkg"] = True
-
-	# Also allow -K to apply --usepkg/-k
-	if ("--usepkgonly" in myopts) and not ("--usepkg" in myopts):
-		myopts["--usepkg"] = True
-
 	# Allow -p to remove --ask
 	if "--pretend" in myopts:
 		myopts.pop("--ask", None)
@@ -1065,7 +1198,9 @@ def emerge_main():
 			portage.debug.set_trace(True)
 
 	if not ("--quiet" in myopts):
-		if not sys.stdout.isatty() or ("--nospinner" in myopts):
+		if '--nospinner' in myopts or \
+			settings.get('TERM') == 'dumb' or \
+			not sys.stdout.isatty():
 			spinner.update = spinner.update_basic
 
 	if myaction == 'version':
@@ -1134,9 +1269,7 @@ def emerge_main():
 		operations.  This helps minimize parallel emerge.log entries that can
 		confuse log parsers.  We especially want it disabled during
 		parallel-fetch, which uses --resume --fetchonly."""
-		global emergelog
-		def emergelog(*pargs, **kargs):
-			pass
+		_emerge.emergelog._disable = True
 
 	else:
 		if 'EMERGE_LOG_DIR' in settings:
@@ -1225,7 +1358,7 @@ def emerge_main():
 			if is_valid_package_atom(x):
 				try:
 					valid_atoms.append(
-						portage.dep_expand(x, mydb=vardb, settings=settings))
+						dep_expand(x, mydb=vardb, settings=settings))
 				except portage.exception.AmbiguousPackageName as e:
 					msg = "The short ebuild name \"" + x + \
 						"\" is ambiguous.  Please specify " + \
