@@ -1,6 +1,6 @@
 # Copyright 1999-2009 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: unmerge.py 14549 2009-10-10 03:08:40Z zmedico $
+# $Id: unmerge.py 15611 2010-03-02 20:37:38Z zmedico $
 
 from __future__ import print_function
 
@@ -27,6 +27,7 @@ def unmerge(root_config, myopts, unmerge_action,
 	if clean_world:
 		clean_world = myopts.get('--deselect') != 'n'
 	quiet = "--quiet" in myopts
+	enter_invalid = '--ask-enter-invalid' in myopts
 	settings = root_config.settings
 	sets = root_config.sets
 	vartree = root_config.trees["vartree"]
@@ -236,7 +237,7 @@ def unmerge(root_config, myopts, unmerge_action,
 					slotmap[myslot][localtree.dbapi.cpv_counter(mypkg)] = mypkg
 
 				for mypkg in vartree.dbapi.cp_list(
-					portage.dep_getkey(mymatch[0])):
+					portage.cpv_getkey(mymatch[0])):
 					myslot = vartree.getslot(mypkg)
 					if myslot not in slotmap:
 						slotmap[myslot] = {}
@@ -396,7 +397,8 @@ def unmerge(root_config, myopts, unmerge_action,
 		#we're done... return
 		return 0
 	if "--ask" in myopts:
-		if userquery("Would you like to unmerge these packages?")=="No":
+		if userquery("Would you like to unmerge these packages?",
+			enter_invalid) == "No":
 			# enter pretend mode for correct formatting of results
 			myopts["--pretend"] = True
 			print()
@@ -424,22 +426,22 @@ def unmerge(root_config, myopts, unmerge_action,
 					raise UninstallFailure(retval)
 				sys.exit(retval)
 			else:
-				if clean_world and hasattr(sets["world"], "cleanPackage")\
-						and hasattr(sets["world"], "lock"):
-					sets["world"].lock()
-					if hasattr(sets["world"], "load"):
-						sets["world"].load()
-					sets["world"].cleanPackage(vartree.dbapi, y)
-					sets["world"].unlock()
+				if clean_world and hasattr(sets["selected"], "cleanPackage")\
+						and hasattr(sets["selected"], "lock"):
+					sets["selected"].lock()
+					if hasattr(sets["selected"], "load"):
+						sets["selected"].load()
+					sets["selected"].cleanPackage(vartree.dbapi, y)
+					sets["selected"].unlock()
 				emergelog(xterm_titles, " >>> unmerge success: "+y)
 
-	if clean_world and hasattr(sets["world"], "remove")\
-			and hasattr(sets["world"], "lock"):
-		sets["world"].lock()
+	if clean_world and hasattr(sets["selected"], "remove")\
+			and hasattr(sets["selected"], "lock"):
+		sets["selected"].lock()
 		# load is called inside remove()
 		for s in root_config.setconfig.active:
-			sets["world"].remove(SETPREFIX+s)
-		sets["world"].unlock()
+			sets["selected"].remove(SETPREFIX + s)
+		sets["selected"].unlock()
 
 	return 1
 
