@@ -1,6 +1,6 @@
 # Copyright 2007 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: __init__.py 14540 2009-10-10 01:58:52Z zmedico $
+# $Id: __init__.py 15726 2010-03-02 22:55:54Z zmedico $
 
 from __future__ import print_function
 
@@ -34,20 +34,29 @@ class SetConfigError(Exception):
 
 class SetConfig(object):
 	def __init__(self, paths, settings, trees):
-		self._parser = SafeConfigParser()
+		self._parser = SafeConfigParser(
+			defaults={
+				"PORTAGE_CONFIGROOT" : settings["PORTAGE_CONFIGROOT"],
+				"ROOT" : settings["ROOT"],
+			})
 		#self._parser.read(paths)
 		# The "paths" argument is ignored and the config for
 		# system and world sets is hardcoded below.
 		parser = self._parser
 
+		parser.add_section("world")
+		parser.set("world", "class", "portage._sets.base.DummyPackageSet")
+		parser.set("world", "world-candidate", "False")
+		parser.set("world", "packages", "@selected @system")
+
+		parser.add_section("selected")
+		parser.set("selected", "class", "portage._sets.files.WorldSelectedSet")
+		parser.set("selected", "world-candidate", "False")
+
 		parser.add_section("system")
 		parser.set("system", "class", "portage._sets.profiles.PackagesSystemSet")
 		parser.set("system", "world-candidate", "False")
 
-		parser.add_section("world")
-		parser.set("world", "class", "portage._sets.files.WorldSet")
-		parser.set("world", "world-candidate", "False")
-
 		self.errors = []
 		self.psets = {}
 		self.trees = trees
@@ -167,41 +176,21 @@ class SetConfig(object):
 			raise PackageSetNotFound(setname)
 		myatoms = myset.getAtoms()
 		parser = self._parser
-		extend = set()
-		remove = set()
-		intersect = set()
-		
+
 		if ignorelist is None:
 			ignorelist = set()
-		if not setname in ignorelist:
-			if parser.has_option(myset.creator, "extend"):
-				extend.update(parser.get(myset.creator, "extend").split())
-			if parser.has_option(myset.creator, "remove"):
-				remove.update(parser.get(myset.creator, "remove").split())
-			if parser.has_option(myset.creator, "intersect"):
-				intersect.update(parser.get(myset.creator, "intersect").split())
-						
+
 		ignorelist.add(setname)
 		for n in myset.getNonAtoms():
 			if n.startswith(SETPREFIX):
 				s = n[len(SETPREFIX):]
 				if s in self.psets:
-					extend.add(n[len(SETPREFIX):])
+					if s not in ignorelist:
+						myatoms.update(self.getSetAtoms(s,
+							ignorelist=ignorelist))
 				else:
 					raise PackageSetNotFound(s)
 
-		for s in ignorelist:
-			extend.discard(s)
-			remove.discard(s)
-			intersect.discard(s)
-		
-		for s in extend:
-			myatoms.update(self.getSetAtoms(s, ignorelist=ignorelist))
-		for s in remove:
-			myatoms.difference_update(self.getSetAtoms(s, ignorelist=ignorelist))
-		for s in intersect:
-			myatoms.intersection_update(self.getSetAtoms(s, ignorelist=ignorelist))
-
 		return myatoms
 
 def load_default_config(settings, trees):
@@ -211,15 +200,3 @@ def load_default_config(settings, trees):
 	setconfigpaths.append(os.path.join(settings["PORTAGE_CONFIGROOT"],
 		USER_CONFIG_PATH, "sets.conf"))
 	return SetConfig(setconfigpaths, settings, trees)
-
-# adhoc test code
-if __name__ == "__main__":
-	import portage
-	sc = load_default_config(portage.settings, portage.db["/"])
-	l, e = sc.getSets()
-	for x in l:
-		print(x+":")
-		print("DESCRIPTION = %s" % l[x].getMetadata("Description"))
-		for n in sorted(l[x].getAtoms()):
-			print("- "+n)
-		print()
