@@ -1,6 +1,6 @@
 # Copyright 2007 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: files.py 14544 2009-10-10 02:28:46Z zmedico $
+# $Id: files.py 15621 2010-03-02 20:43:32Z zmedico $
 
 import errno
 import re
@@ -19,15 +19,15 @@ from portage._sets.base import PackageSet, EditablePackageSet
 from portage._sets import SetConfigError, SETPREFIX, get_boolean
 from portage.env.loaders import ItemFileLoader, KeyListFileLoader
 from portage.env.validators import ValidAtomValidator
-from portage import dep_getkey, cpv_getkey
+from portage import cpv_getkey
 
-__all__ = ["WorldSet",]
+__all__ = ["WorldSelectedSet",]
 
-class WorldSet(EditablePackageSet):
+class WorldSelectedSet(EditablePackageSet):
 	description = "Set of packages that were directly installed by the user"
 	
 	def __init__(self, root):
-		super(WorldSet, self).__init__()
+		super(WorldSelectedSet, self).__init__()
 		# most attributes exist twice as atoms and non-atoms are stored in 
 		# separate files
 		self._lock = None
@@ -44,6 +44,7 @@ class WorldSet(EditablePackageSet):
 
 	def load(self):
 		atoms = []
+		nonatoms = []
 		atoms_changed = False
 		# load atoms and non-atoms from different files so the worldfile is 
 		# backwards-compatible with older versions and other PMs, even though 
@@ -70,7 +71,7 @@ class WorldSet(EditablePackageSet):
 			atoms.extend(self._atoms)
 
 		if atoms_changed:
-			self._setAtoms(atoms)
+			self._setAtoms(atoms+nonatoms)
 		
 	def _ensure_dirs(self):
 		ensure_dirs(os.path.dirname(self._filename), gid=portage_gid, mode=0o2750, mask=0o2)
@@ -95,7 +96,7 @@ class WorldSet(EditablePackageSet):
 		mykey = cpv_getkey(cpv)
 		newworldlist = []
 		for x in worldlist:
-			if dep_getkey(x) == mykey:
+			if x.cp == mykey:
 				matches = vardb.match(x, use_cache=0)
 				if not matches:
 					#zap our world entry
@@ -114,5 +115,5 @@ class WorldSet(EditablePackageSet):
 		self.replace(newworldlist)
 
 	def singleBuilder(self, options, settings, trees):
-		return WorldSet(settings["ROOT"])
+		return WorldSelectedSet(settings["ROOT"])
 	singleBuilder = classmethod(singleBuilder)
