@@ -1,7 +1,7 @@
 # Copyright: 2005 Gentoo Foundation
 # Author(s): Brian Harring (ferringb@gentoo.org)
 # License: GPL2
-# $Id: flat_hash.py 14416 2009-09-24 22:15:48Z zmedico $
+# $Id: flat_hash.py 15536 2010-03-02 19:50:41Z zmedico $
 
 import codecs
 from portage.cache import fs_template
@@ -48,7 +48,7 @@ class database(fs_template.FsBased):
 				if '_mtime_' not in d:
 					# Backward compatibility with old cache
 					# that uses mtime mangling.
-					d['_mtime_'] = long(_os.fstat(myf.fileno()).st_mtime)
+					d['_mtime_'] = _os.fstat(myf.fileno())[stat.ST_MTIME]
 				return d
 			finally:
 				myf.close()
@@ -120,24 +120,32 @@ class database(fs_template.FsBased):
 
 	def __iter__(self):
 		"""generator for walking the dir struct"""
-		dirs = [self.location]
+		dirs = [(0, self.location)]
 		len_base = len(self.location)
-		while len(dirs):
+		while dirs:
+			depth, dir_path = dirs.pop()
 			try:
-				dir_list = os.listdir(dirs[0])
+				dir_list = os.listdir(dir_path)
 			except OSError as e:
 				if e.errno != errno.ENOENT:
 					raise
 				del e
-				dirs.pop(0)
 				continue
 			for l in dir_list:
 				if l.endswith(".cpickle"):
 					continue
-				p = os.path.join(dirs[0],l)
-				st = os.lstat(p)
+				p = os.path.join(dir_path, l)
+				try:
+					st = os.lstat(p)
+				except OSError:
+					# Cache entry disappeared.
+					continue
 				if stat.S_ISDIR(st.st_mode):
-					dirs.append(p)
+					# Only recurse 1 deep, in order to avoid iteration over
+					# entries from another nested cache instance. This can
+					# happen if the user nests an overlay inside
+					# /usr/portage/local as in bug #302764.
+					if depth < 1:
+						dirs.append((depth+1, p))
 					continue
 				yield p[len_base+1:]
-			dirs.pop(0)
