@@ -1,10 +1,11 @@
 # Copyright: 2005 Gentoo Foundation
 # Author(s): Brian Harring (ferringb@gentoo.org)
 # License: GPL2
-# $Id: metadata.py 14398 2009-09-24 05:48:33Z zmedico $
+# $Id: metadata.py 15137 2009-12-24 04:01:10Z zmedico $
 
 import errno
 import re
+import stat
 import sys
 from portage import os
 from portage import _encodings
@@ -110,7 +111,7 @@ class database(flat_hash.database):
 			except EnvironmentError:
 				pass
 			else:
-				existing_mtime = long(existing_st.st_mtime)
+				existing_mtime = existing_st[stat.ST_MTIME]
 				if values['_mtime_'] == existing_mtime and \
 					existing_content == new_content:
 					return
