@@ -1,24 +1,30 @@
-# Copyright 1998-2007 Gentoo Foundation
+# Copyright 1998-2009 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: __init__.py 12158 2008-12-04 23:57:32Z zmedico $
+# $Id: __init__.py 15676 2010-03-02 21:01:55Z zmedico $
 
 __all__ = ["dbapi"]
 
-import os
 import re
-from portage.dep import match_from_list
-from portage.locks import unlockfile
-from portage.output import red
-from portage.util import writemsg
-from portage import auxdbkeys, dep_expand
-from portage.versions import catpkgsplit, pkgcmp
 
+import portage
+portage.proxy.lazyimport.lazyimport(globals(),
+	'portage.dbapi.dep_expand:dep_expand@_dep_expand',
+	'portage.dep:match_from_list',
+	'portage.locks:unlockfile',
+	'portage.output:colorize',
+	'portage.util:cmp_sort_key,writemsg',
+	'portage.versions:catsplit,catpkgsplit,vercmp',
+)
+
+from portage import os
+from portage import auxdbkeys
+from portage.localization import _
 
 class dbapi(object):
 	_category_re = re.compile(r'^\w[-.+\w]*$')
 	_pkg_dir_name_re = re.compile(r'^\w[-+\w]*$')
 	_categories = None
-	_iuse_implicit = None
+	_iuse_implicit_re = None
 	_use_mutable = False
 	_known_keys = frozenset(x for x in auxdbkeys
 		if not x.startswith("UNUSED_0"))
@@ -34,19 +40,15 @@ class dbapi(object):
 		"""
 		if self._categories is not None:
 			return self._categories
-		categories = set()
-		cat_pattern = re.compile(r'(.*)/.*')
-		for cp in self.cp_all():
-			categories.add(cat_pattern.match(cp).group(1))
-		self._categories = list(categories)
-		self._categories.sort()
+		self._categories = tuple(sorted(set(catsplit(x)[0] \
+			for x in self.cp_all())))
 		return self._categories
 
 	def close_caches(self):
 		pass
 
 	def cp_list(self, cp, use_cache=1):
-		return
+		raise NotImplementedError(self)
 
 	def _cpv_sort_ascending(self, cpv_list):
 		"""
@@ -57,14 +59,12 @@ class dbapi(object):
 			# If the cpv includes explicit -r0, it has to be preserved
 			# for consistency in findname and aux_get calls, so use a
 			# dict to map strings back to their original values.
-			str_map = {}
-			for i, cpv in enumerate(cpv_list):
-				mysplit = tuple(catpkgsplit(cpv)[1:])
-				str_map[mysplit] = cpv
-				cpv_list[i] = mysplit
-			cpv_list.sort(pkgcmp)
-			for i, mysplit in enumerate(cpv_list):
-				cpv_list[i] = str_map[mysplit]
+			ver_map = {}
+			for cpv in cpv_list:
+				ver_map[cpv] = '-'.join(catpkgsplit(cpv)[2:])
+			def cmp_cpv(cpv1, cpv2):
+				return vercmp(ver_map[cpv1], ver_map[cpv2])
+			cpv_list.sort(key=cmp_sort_key(cmp_cpv))
 
 	def cpv_all(self):
 		"""Return all CPVs in the db
@@ -122,7 +122,7 @@ class dbapi(object):
 		Returns:
 			a list of packages that match origdep
 		"""
-		mydep = dep_expand(origdep, mydb=self, settings=self.settings)
+		mydep = _dep_expand(origdep, mydb=self, settings=self.settings)
 		return list(self._iter_match(mydep,
 			self.cp_list(mydep.cp, use_cache=use_cache)))
 
@@ -147,20 +147,20 @@ class dbapi(object):
 		1) Check for required IUSE intersection (need implicit IUSE here).
 		2) Check enabled/disabled flag states.
 		"""
-		if self._iuse_implicit is None:
-			self._iuse_implicit = self.settings._get_implicit_iuse()
+		if self._iuse_implicit_re is None:
+			self._iuse_implicit_re = re.compile("^(%s)$" % \
+				"|".join(self.settings._get_implicit_iuse()))
+		iuse_implicit_re = self._iuse_implicit_re
 		for cpv in cpv_iter:
 			try:
 				iuse, slot, use = self.aux_get(cpv, ["IUSE", "SLOT", "USE"])
 			except KeyError:
 				continue
 			use = use.split()
-			iuse = self._iuse_implicit.union(
-				re.escape(x.lstrip("+-")) for x in iuse.split())
-			iuse_re = re.compile("^(%s)$" % "|".join(iuse))
+			iuse = frozenset(x.lstrip('+-') for x in iuse.split())
 			missing_iuse = False
 			for x in atom.use.required:
-				if iuse_re.match(x) is None:
+				if x not in iuse and iuse_implicit_re.match(x) is None:
 					missing_iuse = True
 					break
 			if missing_iuse:
@@ -172,7 +172,7 @@ class dbapi(object):
 					continue
 			else:
 				# Check masked and forced flags for repoman.
-				mysettings = getattr(self, "mysettings", None)
+				mysettings = getattr(self, 'settings', None)
 				if mysettings is not None and not mysettings.local_config:
 
 					pkg = "%s:%s" % (cpv, slot)
@@ -189,14 +189,15 @@ class dbapi(object):
 	def invalidentry(self, mypath):
 		if mypath.endswith('portage_lockfile'):
 			if "PORTAGE_MASTER_PID" not in os.environ:
-				writemsg("Lockfile removed: %s\n" % mypath, 1)
+				writemsg(_("Lockfile removed: %s\n") % mypath, 1)
 				unlockfile((mypath, None, None))
 			else:
 				# Nothing we can do about it. We're probably sandboxed.
 				pass
 		elif '/-MERGING-' in mypath:
 			if os.path.exists(mypath):
-				writemsg(red("INCOMPLETE MERGE:")+" "+mypath+"\n", noiselevel=-1)
+				writemsg(colorize("BAD", _("INCOMPLETE MERGE:"))+" %s\n" % mypath,
+					noiselevel=-1)
 		else:
 			writemsg("!!! Invalid db entry: %s\n" % mypath, noiselevel=-1)
 
@@ -218,14 +219,13 @@ class dbapi(object):
 		aux_get = self.aux_get
 		aux_update = self.aux_update
 		update_keys = ["DEPEND", "RDEPEND", "PDEPEND", "PROVIDE"]
-		from itertools import izip
 		from portage.update import update_dbentries
 		if onUpdate:
 			onUpdate(maxval, 0)
 		if onProgress:
 			onProgress(maxval, 0)
 		for i, cpv in enumerate(cpv_all):
-			metadata = dict(izip(update_keys, aux_get(cpv, update_keys)))
+			metadata = dict(zip(update_keys, aux_get(cpv, update_keys)))
 			metadata_updates = update_dbentries(updates, metadata)
 			if metadata_updates:
 				aux_update(cpv, metadata_updates)
