@@ -1,26 +1,45 @@
-# Copyright 1998-2007 Gentoo Foundation
+# Copyright 1998-2009 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: bintree.py 11952 2008-11-15 18:53:05Z zmedico $
+# $Id: bintree.py 15696 2010-03-02 21:09:35Z zmedico $
 
 __all__ = ["bindbapi", "binarytree"]
 
+import portage
+portage.proxy.lazyimport.lazyimport(globals(),
+	'portage.dbapi.dep_expand:dep_expand',
+	'portage.dep:dep_getkey,isjustname,match_from_list',
+	'portage.output:EOutput,colorize',
+	'portage.package.ebuild.doebuild:_vdb_use_conditional_atoms',
+	'portage.update:update_dbentries',
+	'portage.util:ensure_dirs,normalize_path,writemsg,writemsg_stdout',
+	'portage.util.listdir:listdir',
+	'portage.versions:best,catpkgsplit,catsplit',
+)
+
 from portage.cache.mappings import slot_dict_class
-from portage.dep import isvalidatom, isjustname, dep_getkey, match_from_list
 from portage.dbapi.virtual import fakedbapi
 from portage.exception import InvalidPackageName, \
 	PermissionDenied, PortageException
-from portage.output import green, EOutput
-from portage.util import ensure_dirs, normalize_path, writemsg, writemsg_stdout
-from portage.versions import best, catpkgsplit, catsplit
-from portage.update import update_dbentries
+from portage.localization import _
 
-from portage import dep_expand, listdir, _check_distfile, _movefile
+from portage import _movefile
+from portage import os
+from portage import _encodings
+from portage import _unicode_decode
+from portage import _unicode_encode
+from portage.package.ebuild.fetch import _check_distfile
 
-import portage.xpak, portage.getbinpkg
-
-import os, errno, stat
+import codecs
+import errno
 import re
-from itertools import chain, izip
+import stat
+import sys
+import warnings
+from itertools import chain
+
+if sys.hexversion >= 0x3000000:
+	basestring = str
+	long = int
 
 class bindbapi(fakedbapi):
 	_known_keys = frozenset(list(fakedbapi._known_keys) + \
@@ -33,7 +52,7 @@ class bindbapi(fakedbapi):
 		self.cpdict={}
 		# Selectively cache metadata in order to optimize dep matching.
 		self._aux_cache_keys = set(
-			["CHOST", "DEPEND", "EAPI", "IUSE", "KEYWORDS",
+			["BUILD_TIME", "CHOST", "DEPEND", "EAPI", "IUSE", "KEYWORDS",
 			"LICENSE", "PDEPEND", "PROPERTIES", "PROVIDE",
 			"RDEPEND", "repository", "RESTRICT", "SLOT", "USE"])
 		self._aux_cache_slot_dict = slot_dict_class(self._aux_cache_keys)
@@ -44,6 +63,14 @@ class bindbapi(fakedbapi):
 			self.bintree.populate()
 		return fakedbapi.match(self, *pargs, **kwargs)
 
+	def cpv_inject(self, cpv, **kwargs):
+		self._aux_cache.pop(cpv, None)
+		fakedbapi.cpv_inject(self, cpv, **kwargs)
+
+	def cpv_remove(self, cpv):
+		self._aux_cache.pop(cpv, None)
+		fakedbapi.cpv_remove(self, cpv)
+
 	def aux_get(self, mycpv, wants):
 		if self.bintree and not self.bintree.populated:
 			self.bintree.populate()
@@ -62,7 +89,15 @@ class bindbapi(fakedbapi):
 			tbz2_path = self.bintree.getname(mycpv)
 			if not os.path.exists(tbz2_path):
 				raise KeyError(mycpv)
-			getitem = portage.xpak.tbz2(tbz2_path).getfile
+			metadata_bytes = portage.xpak.tbz2(tbz2_path).get_data()
+			def getitem(k):
+				v = metadata_bytes.get(_unicode_encode(k,
+					encoding=_encodings['repo.content'],
+					errors='backslashreplace'))
+				if v is not None:
+					v = _unicode_decode(v,
+						encoding=_encodings['repo.content'], errors='replace')
+				return v
 		else:
 			getitem = self.bintree._remotepkgs[mycpv].get
 		mydata = {}
@@ -75,15 +110,16 @@ class bindbapi(fakedbapi):
 			# or the tbz2 is corrupt.
 			if myval:
 				mydata[x] = " ".join(myval.split())
-		if "EAPI" in mykeys:
-			if not mydata.setdefault("EAPI", "0"):
-				mydata["EAPI"] = "0"
+
+		if not mydata.setdefault('EAPI', _unicode_decode('0')):
+			mydata['EAPI'] = _unicode_decode('0')
+
 		if cache_me:
 			aux_cache = self._aux_cache_slot_dict()
 			for x in self._aux_cache_keys:
-				aux_cache[x] = mydata.get(x, "")
+				aux_cache[x] = mydata.get(x, _unicode_decode(''))
 			self._aux_cache[mycpv] = aux_cache
-		return [mydata.get(x, "") for x in wants]
+		return [mydata.get(x, _unicode_decode('')) for x in wants]
 
 	def aux_update(self, cpv, values):
 		if not self.bintree.populated:
@@ -93,11 +129,19 @@ class bindbapi(fakedbapi):
 			raise KeyError(cpv)
 		mytbz2 = portage.xpak.tbz2(tbz2path)
 		mydata = mytbz2.get_data()
-		mydata.update(values)
-		for k, v in mydata.items():
+
+		for k, v in values.items():
+			k = _unicode_encode(k,
+				encoding=_encodings['repo.content'], errors='backslashreplace')
+			v = _unicode_encode(v,
+				encoding=_encodings['repo.content'], errors='backslashreplace')
+			mydata[k] = v
+
+		for k, v in list(mydata.items()):
 			if not v:
 				del mydata[k]
 		mytbz2.recompose_mem(portage.xpak.xpak_mem(mydata))
+		# inject will clear stale caches via cpv_inject.
 		self.bintree.inject(cpv)
 
 	def cp_list(self, *pargs, **kwargs):
@@ -117,20 +161,8 @@ class bindbapi(fakedbapi):
 
 class binarytree(object):
 	"this tree scans for a list of all packages available in PKGDIR"
-	def __init__(self, root, pkgdir, virtual=None, settings=None, clone=None):
-		if clone:
-			writemsg("binartree.__init__(): deprecated " + \
-				"use of clone parameter\n", noiselevel=-1)
-			# XXX This isn't cloning. It's an instance of the same thing.
-			self.root = clone.root
-			self.pkgdir = clone.pkgdir
-			self.dbapi = clone.dbapi
-			self.populated = clone.populated
-			self.tree = clone.tree
-			self.remotepkgs = clone.remotepkgs
-			self.invalids = clone.invalids
-			self.settings = clone.settings
-		else:
+	def __init__(self, root, pkgdir, virtual=None, settings=None):
+		if True:
 			self.root = root
 			#self.pkgdir=settings["PKGDIR"]
 			self.pkgdir = normalize_path(pkgdir)
@@ -142,7 +174,7 @@ class binarytree(object):
 			self._remote_has_index = False
 			self._remote_base_uri = None
 			self._remotepkgs = None # remote metadata indexed by cpv
-			self.remotepkgs = {}  # indexed by tbz2 name (deprecated)
+			self.__remotepkgs = {}  # indexed by tbz2 name (deprecated)
 			self.invalids = []
 			self.settings = settings
 			self._pkg_paths = {}
@@ -155,17 +187,20 @@ class binarytree(object):
 			self._pkgindex_keys = self.dbapi._aux_cache_keys.copy()
 			self._pkgindex_keys.update(["CPV", "MTIME", "SIZE"])
 			self._pkgindex_aux_keys = \
-				["CHOST", "DEPEND", "DESCRIPTION", "EAPI",
+				["BUILD_TIME", "CHOST", "DEPEND", "DESCRIPTION", "EAPI",
 				"IUSE", "KEYWORDS", "LICENSE", "PDEPEND", "PROPERTIES",
 				"PROVIDE", "RDEPEND", "repository", "SLOT", "USE"]
 			self._pkgindex_aux_keys = list(self._pkgindex_aux_keys)
 			self._pkgindex_use_evaluated_keys = \
 				("LICENSE", "RDEPEND", "DEPEND",
 				"PDEPEND", "PROPERTIES", "PROVIDE")
-			self._pkgindex_header_keys = set(["ACCEPT_KEYWORDS", "CBUILD",
+			self._pkgindex_header_keys = set([
+				"ACCEPT_KEYWORDS", "ACCEPT_LICENSE",
+				"ACCEPT_PROPERTIES", "CBUILD",
 				"CHOST", "CONFIG_PROTECT", "CONFIG_PROTECT_MASK", "FEATURES",
 				"GENTOO_MIRRORS", "INSTALL_MASK", "SYNC", "USE"])
 			self._pkgindex_default_pkg_data = {
+				"BUILD_TIME"         : "",
 				"DEPEND"  : "",
 				"EAPI"    : "0",
 				"IUSE"    : "",
@@ -199,39 +234,56 @@ class binarytree(object):
 				chain(*self._pkgindex_translated_keys)
 			))
 
+	def _get_remotepkgs(self):
+		warnings.warn("Use binarytree._remotepkgs insead of binarytree.remotepkgs",
+			DeprecationWarning)
+		return self.__remotepkgs
+
+	def _set_remotepkgs(self, remotepkgs):
+		warnings.warn("Use binarytree._remotepkgs insead of binarytree.remotepkgs",
+			DeprecationWarning)
+		self.__remotepkgs = remotepkgs
+
+	def _del_remotepkgs(self):
+		warnings.warn("Use binarytree._remotepkgs insead of binarytree.remotepkgs",
+			DeprecationWarning)
+		del self.__remotepkgs
+
+	remotepkgs = property(_get_remotepkgs, _set_remotepkgs, _del_remotepkgs,
+		"Deprecated self.remotepkgs, only for backward compatibility")
+
 	def move_ent(self, mylist):
 		if not self.populated:
 			self.populate()
 		origcp = mylist[1]
 		newcp = mylist[2]
 		# sanity check
-		for cp in [origcp, newcp]:
-			if not (isvalidatom(cp) and isjustname(cp)):
-				raise InvalidPackageName(cp)
-		origcat = origcp.split("/")[0]
-		mynewcat = newcp.split("/")[0]
+		for atom in (origcp, newcp):
+			if not isjustname(atom):
+				raise InvalidPackageName(str(atom))
+		mynewcat = catsplit(newcp)[0]
 		origmatches=self.dbapi.cp_list(origcp)
 		moves = 0
 		if not origmatches:
 			return moves
 		for mycpv in origmatches:
-
-			mycpsplit = catpkgsplit(mycpv)
-			mynewcpv = newcp + "-" + mycpsplit[2]
-			if mycpsplit[3] != "r0":
-				mynewcpv += "-" + mycpsplit[3]
-			myoldpkg = mycpv.split("/")[1]
-			mynewpkg = mynewcpv.split("/")[1]
+			mycpv_cp = portage.cpv_getkey(mycpv)
+			if mycpv_cp != origcp:
+				# Ignore PROVIDE virtual match.
+				continue
+			mynewcpv = mycpv.replace(mycpv_cp, str(newcp), 1)
+			myoldpkg = catsplit(mycpv)[1]
+			mynewpkg = catsplit(mynewcpv)[1]
 
 			if (mynewpkg != myoldpkg) and os.path.exists(self.getname(mynewcpv)):
-				writemsg("!!! Cannot update binary: Destination exists.\n",
+				writemsg(_("!!! Cannot update binary: Destination exists.\n"),
 					noiselevel=-1)
 				writemsg("!!! "+mycpv+" -> "+mynewcpv+"\n", noiselevel=-1)
 				continue
 
 			tbz2path = self.getname(mycpv)
 			if os.path.exists(tbz2path) and not os.access(tbz2path,os.W_OK):
-				writemsg("!!! Cannot update readonly binary: "+mycpv+"\n",
+				writemsg(_("!!! Cannot update readonly binary: %s\n") % mycpv,
 					noiselevel=-1)
 				continue
 
@@ -240,13 +292,21 @@ class binarytree(object):
 			mydata = mytbz2.get_data()
 			updated_items = update_dbentries([mylist], mydata)
 			mydata.update(updated_items)
-			mydata["PF"] = mynewpkg + "\n"
-			mydata["CATEGORY"] = mynewcat+"\n"
+			mydata[_unicode_encode('PF',
+				encoding=_encodings['repo.content'])] = \
+				_unicode_encode(mynewpkg + "\n",
+				encoding=_encodings['repo.content'])
+			mydata[_unicode_encode('CATEGORY',
+				encoding=_encodings['repo.content'])] = \
+				_unicode_encode(mynewcat + "\n",
+				encoding=_encodings['repo.content'])
 			if mynewpkg != myoldpkg:
-				ebuild_data = mydata.get(myoldpkg+".ebuild")
+				ebuild_data = mydata.pop(_unicode_encode(myoldpkg + '.ebuild',
+					encoding=_encodings['repo.content']), None)
 				if ebuild_data is not None:
-					mydata[mynewpkg+".ebuild"] = ebuild_data
-					del mydata[myoldpkg+".ebuild"]
+					mydata[_unicode_encode(mynewpkg + '.ebuild',
+						encoding=_encodings['repo.content'])] = ebuild_data
+
 			mytbz2.recompose_mem(portage.xpak.xpak_mem(mydata))
 
 			self.dbapi.cpv_remove(mycpv)
@@ -276,7 +336,7 @@ class binarytree(object):
 			os.unlink(mylink)
 		try:
 			os.rmdir(os.path.join(self.pkgdir, mycat))
-		except OSError, e:
+		except OSError as e:
 			if e.errno not in (errno.ENOENT,
 				errno.ENOTEMPTY, errno.EEXIST):
 				raise
@@ -291,7 +351,7 @@ class binarytree(object):
 		self._ensure_dir(os.path.dirname(full_path))
 		try:
 			os.unlink(full_path)
-		except OSError, e:
+		except OSError as e:
 			if e.errno != errno.ENOENT:
 				raise
 			del e
@@ -322,19 +382,33 @@ class binarytree(object):
 		myfile = mypkg + ".tbz2"
 		mypath = os.path.join("All", myfile)
 		dest_path = os.path.join(self.pkgdir, mypath)
-		if os.path.exists(dest_path):
+
+		try:
+			st = os.lstat(dest_path)
+		except OSError:
+			st = None
+		else:
+			if stat.S_ISLNK(st.st_mode):
+				st = None
+				try:
+					os.unlink(dest_path)
+				except OSError:
+					if os.path.exists(dest_path):
+						raise
+
+		if st is not None:
 			# For invalid packages, other_cat could be None.
-			other_cat = portage.xpak.tbz2(dest_path).getfile("CATEGORY")
+			other_cat = portage.xpak.tbz2(dest_path).getfile(
+				_unicode_encode("CATEGORY",
+				encoding=_encodings['repo.content']))
 			if other_cat:
+				other_cat = _unicode_decode(other_cat,
+					encoding=_encodings['repo.content'], errors='replace')
 				other_cat = other_cat.strip()
 				other_cpv = other_cat + "/" + mypkg
 				self._move_from_all(other_cpv)
 				self.inject(other_cpv)
-		"""The file may or may not exist. Move it if necessary and update
-		internal state for future calls to getname()."""
 		self._move_to_all(cpv)
-		if os.path.exists(full_path):
-			self.inject(cpv)
 
 	def _ensure_dir(self, path):
 		"""
@@ -349,7 +423,7 @@ class binarytree(object):
 			ensure_dirs(path)
 			return
 		pkgdir_gid = pkgdir_st.st_gid
-		pkgdir_grp_mode = 02070 & pkgdir_st.st_mode
+		pkgdir_grp_mode = 0o2070 & pkgdir_st.st_mode
 		try:
 			ensure_dirs(path, gid=pkgdir_gid, mode=pkgdir_grp_mode, mask=0)
 		except PortageException:
@@ -361,17 +435,18 @@ class binarytree(object):
 		for future getname() calls."""
 		mycat, mypkg = catsplit(cpv)
 		myfile = mypkg + ".tbz2"
+		self._pkg_paths[cpv] = os.path.join("All", myfile)
 		src_path = os.path.join(self.pkgdir, mycat, myfile)
 		try:
 			mystat = os.lstat(src_path)
-		except OSError, e:
+		except OSError as e:
 			mystat = None
 		if mystat and stat.S_ISREG(mystat.st_mode):
 			self._ensure_dir(os.path.join(self.pkgdir, "All"))
 			dest_path = os.path.join(self.pkgdir, "All", myfile)
 			_movefile(src_path, dest_path, mysettings=self.settings)
 			self._create_symlink(cpv)
-		self._pkg_paths[cpv] = os.path.join("All", myfile)
+			self.inject(cpv)
 
 	def _move_from_all(self, cpv):
 		"""Move a package from ${PKGDIR}/All/${PF}.tbz2 to
@@ -386,8 +461,15 @@ class binarytree(object):
 		_movefile(src_path, dest_path, mysettings=self.settings)
 		self._pkg_paths[cpv] = mypath
 
-	def populate(self, getbinpkgs=0, getbinpkgsonly=0):
+	def populate(self, getbinpkgs=0, getbinpkgsonly=None):
 		"populates the binarytree"
+
+		if getbinpkgsonly is not None:
+			warnings.warn(
+				"portage.dbapi.bintree.binarytree.populate(): " + \
+				"getbinpkgsonly parameter is deprecated",
+				DeprecationWarning, stacklevel=2)
+
 		if self._populating:
 			return
 		from portage.locks import lockfile, unlockfile
@@ -397,17 +479,24 @@ class binarytree(object):
 				pkgindex_lock = lockfile(self._pkgindex_file,
 					wantnewlockfile=1)
 			self._populating = True
-			self._populate(getbinpkgs, getbinpkgsonly)
+			self._populate(getbinpkgs)
 		finally:
 			if pkgindex_lock:
 				unlockfile(pkgindex_lock)
 			self._populating = False
 
-	def _populate(self, getbinpkgs=0, getbinpkgsonly=0):
+	def _populate(self, getbinpkgs=0):
 		if (not os.path.isdir(self.pkgdir) and not getbinpkgs):
 			return 0
 
-		if not getbinpkgsonly:
+		# Clear all caches in case populate is called multiple times
+		# as may be the case when _global_updates calls populate()
+		# prior to performing package moves since it only wants to
+		# operate on local packages (getbinpkgs=0).
+		self._remotepkgs = None
+		self.dbapi._clear_cache()
+		self.dbapi._aux_cache.clear()
+		if True:
 			pkg_paths = {}
 			self._pkg_paths = pkg_paths
 			dirs = listdir(self.pkgdir, dirsonly=True, EmptyOnError=True)
@@ -415,18 +504,8 @@ class binarytree(object):
 				dirs.remove("All")
 			dirs.sort()
 			dirs.insert(0, "All")
-			pkgindex = self._new_pkgindex()
+			pkgindex = self._load_pkgindex()
 			pf_index = None
-			try:
-				f = open(self._pkgindex_file)
-			except EnvironmentError:
-				pass
-			else:
-				try:
-					pkgindex.read(f)
-				finally:
-					f.close()
-					del f
 			if not self._pkgindex_version_supported(pkgindex):
 				pkgindex = self._new_pkgindex()
 			header = pkgindex.header
@@ -463,7 +542,7 @@ class binarytree(object):
 						match = None
 						for d in possibilities:
 							try:
-								if long(d["MTIME"]) != long(s.st_mtime):
+								if long(d["MTIME"]) != s[stat.ST_MTIME]:
 									continue
 							except (KeyError, ValueError):
 								continue
@@ -501,20 +580,28 @@ class binarytree(object):
 								self.dbapi._aux_cache[mycpv] = aux_cache
 							continue
 					if not os.access(full_path, os.R_OK):
-						writemsg("!!! Permission denied to read " + \
-							"binary package: '%s'\n" % full_path,
+						writemsg(_("!!! Permission denied to read " \
+							"binary package: '%s'\n") % full_path,
 							noiselevel=-1)
 						self.invalids.append(myfile[:-5])
 						continue
-					mytbz2 = portage.xpak.tbz2(full_path)
-					# For invalid packages, mycat could be None.
-					mycat = mytbz2.getfile("CATEGORY")
-					mypf = mytbz2.getfile("PF")
-					slot = mytbz2.getfile("SLOT")
+					metadata_bytes = portage.xpak.tbz2(full_path).get_data()
+					mycat = _unicode_decode(metadata_bytes.get(
+						_unicode_encode("CATEGORY",
+						encoding=_encodings['repo.content']), ""),
+						encoding=_encodings['repo.content'], errors='replace')
+					mypf = _unicode_decode(metadata_bytes.get(
+						_unicode_encode("PF",
+						encoding=_encodings['repo.content']), ""),
+						encoding=_encodings['repo.content'], errors='replace')
+					slot = _unicode_decode(metadata_bytes.get(
+						_unicode_encode("SLOT",
+						encoding=_encodings['repo.content']), ""),
+						encoding=_encodings['repo.content'], errors='replace')
 					mypkg = myfile[:-5]
 					if not mycat or not mypf or not slot:
 						#old-style or corrupt package
-						writemsg("\n!!! Invalid binary package: '%s'\n" % full_path,
+						writemsg(_("\n!!! Invalid binary package: '%s'\n") % full_path,
 							noiselevel=-1)
 						missing_keys = []
 						if not mycat:
@@ -526,10 +613,10 @@ class binarytree(object):
 						msg = []
 						if missing_keys:
 							missing_keys.sort()
-							msg.append("Missing metadata key(s): %s." % \
+							msg.append(_("Missing metadata key(s): %s.") % \
 								", ".join(missing_keys))
-						msg.append(" This binary package is not " + \
-							"recoverable and should be deleted.")
+						msg.append(_(" This binary package is not " \
+							"recoverable and should be deleted."))
 						from textwrap import wrap
 						for line in wrap("".join(msg), 72):
 							writemsg("!!! %s\n" % line, noiselevel=-1)
@@ -546,10 +633,10 @@ class binarytree(object):
 						# All is first, so it's preferred.
 						continue
 					if not self.dbapi._category_re.match(mycat):
-						writemsg(("!!! Binary package has an " + \
+						writemsg(_("!!! Binary package has an " \
 							"unrecognized category: '%s'\n") % full_path,
 							noiselevel=-1)
-						writemsg(("!!! '%s' has a category that is not" + \
+						writemsg(_("!!! '%s' has a category that is not" \
 							" listed in %setc/portage/categories\n") % \
 							(mycpv, self.settings["PORTAGE_CONFIGROOT"]),
 							noiselevel=-1)
@@ -560,7 +647,7 @@ class binarytree(object):
 					d = metadata.get(mycpv, {})
 					if d:
 						try:
-							if long(d["MTIME"]) != long(s.st_mtime):
+							if long(d["MTIME"]) != s[stat.ST_MTIME]:
 								d.clear()
 						except (KeyError, ValueError):
 							d.clear()
@@ -573,15 +660,15 @@ class binarytree(object):
 
 					d["CPV"] = mycpv
 					d["SLOT"] = slot
-					d["MTIME"] = str(long(s.st_mtime))
+					d["MTIME"] = str(s[stat.ST_MTIME])
 					d["SIZE"] = str(s.st_size)
 
-					d.update(izip(self._pkgindex_aux_keys,
+					d.update(zip(self._pkgindex_aux_keys,
 						self.dbapi.aux_get(mycpv, self._pkgindex_aux_keys)))
 					try:
 						self._eval_use_flags(mycpv, d)
 					except portage.exception.InvalidDependString:
-						writemsg("!!! Invalid binary package: '%s'\n" % \
+						writemsg(_("!!! Invalid binary package: '%s'\n") % \
 							self.getname(mycpv), noiselevel=-1)
 						self.dbapi.cpv_remove(mycpv)
 						del pkg_paths[mycpv]
@@ -598,15 +685,16 @@ class binarytree(object):
 							aux_cache[k] = d[k]
 						self.dbapi._aux_cache[mycpv] = aux_cache
 
+			for cpv in list(metadata):
+				if cpv not in pkg_paths:
+					del metadata[cpv]
+
 			# Do not bother to write the Packages index if $PKGDIR/All/ exists
 			# since it will provide no benefit due to the need to read CATEGORY
 			# from xpak.
 			if update_pkgindex and os.access(self.pkgdir, os.W_OK):
-				stale = [cpv for cpv in metadata if cpv not in self._pkg_paths]
-				for cpv in stale:
-					del metadata[cpv]
 				del pkgindex.packages[:]
-				pkgindex.packages.extend(metadata.itervalues())
+				pkgindex.packages.extend(iter(metadata.values()))
 				self._update_pkgindex_header(pkgindex.header)
 				from portage.util import atomic_ofstream
 				f = atomic_ofstream(self._pkgindex_file)
@@ -616,76 +704,121 @@ class binarytree(object):
 					f.close()
 
 		if getbinpkgs and not self.settings["PORTAGE_BINHOST"]:
-			writemsg("!!! PORTAGE_BINHOST unset, but use is requested.\n",
+			writemsg(_("!!! PORTAGE_BINHOST unset, but use is requested.\n"),
 				noiselevel=-1)
 
-		if getbinpkgs and \
-			"PORTAGE_BINHOST" in self.settings and \
-			not self._remotepkgs:
-
+		if getbinpkgs and 'PORTAGE_BINHOST' in self.settings:
 			base_url = self.settings["PORTAGE_BINHOST"]
 			from portage.const import CACHE_PATH
-			from urlparse import urlparse
+			try:
+				from urllib.parse import urlparse
+			except ImportError:
+				from urlparse import urlparse
 			urldata = urlparse(base_url)
-			pkgindex_file = os.path.join(CACHE_PATH, "binhost",
+			pkgindex_file = os.path.join(self.settings["ROOT"], CACHE_PATH, "binhost",
 				urldata[1] + urldata[2], "Packages")
 			pkgindex = self._new_pkgindex()
 			try:
-				f = open(pkgindex_file)
+				f = codecs.open(_unicode_encode(pkgindex_file,
+					encoding=_encodings['fs'], errors='strict'),
+					mode='r', encoding=_encodings['repo.content'],
+					errors='replace')
 				try:
 					pkgindex.read(f)
 				finally:
 					f.close()
-			except EnvironmentError, e:
+			except EnvironmentError as e:
 				if e.errno != errno.ENOENT:
 					raise
 			local_timestamp = pkgindex.header.get("TIMESTAMP", None)
-			import urllib, urlparse
+			try:
+				from urllib.request import urlopen as urllib_request_urlopen
+			except ImportError:
+				from urllib import urlopen as urllib_request_urlopen
 			rmt_idx = self._new_pkgindex()
 			try:
-				f = urllib.urlopen(urlparse.urljoin(base_url, "Packages"))
+				# urlparse.urljoin() only works correctly with recognized
+				# protocols and requires the base url to have a trailing
+				# slash, so join manually...
+				f = urllib_request_urlopen(base_url.rstrip("/") + "/Packages")
+				f_dec = codecs.iterdecode(f,
+					_encodings['repo.content'], errors='replace')
 				try:
-					rmt_idx.readHeader(f)
+					rmt_idx.readHeader(f_dec)
 					remote_timestamp = rmt_idx.header.get("TIMESTAMP", None)
 					if not remote_timestamp:
 						# no timestamp in the header, something's wrong
 						pkgindex = None
 					else:
 						if not self._pkgindex_version_supported(rmt_idx):
-							writemsg("\n\n!!! Binhost package index version" + \
-							" is not supported: '%s'\n" % \
+							writemsg(_("\n\n!!! Binhost package index version" \
+							" is not supported: '%s'\n") % \
 							rmt_idx.header.get("VERSION"), noiselevel=-1)
 							pkgindex = None
 						elif local_timestamp != remote_timestamp:
-							rmt_idx.readBody(f)
+							rmt_idx.readBody(f_dec)
 							pkgindex = rmt_idx
 				finally:
 					f.close()
-			except EnvironmentError, e:
-				writemsg("\n\n!!! Error fetching binhost package" + \
-					" info from '%s'\n" % base_url)
+			except EnvironmentError as e:
+				writemsg(_("\n\n!!! Error fetching binhost package" \
+					" info from '%s'\n") % base_url)
 				writemsg("!!! %s\n\n" % str(e))
 				del e
 				pkgindex = None
 			if pkgindex is rmt_idx:
 				pkgindex.modified = False # don't update the header
 				from portage.util import atomic_ofstream, ensure_dirs
-				ensure_dirs(os.path.dirname(pkgindex_file))
-				f = atomic_ofstream(pkgindex_file)
 				try:
+					ensure_dirs(os.path.dirname(pkgindex_file))
+					f = atomic_ofstream(pkgindex_file)
 					pkgindex.write(f)
-				finally:
 					f.close()
+				except PortageException:
+					if os.access(os.path.join(
+						self.settings["ROOT"], CACHE_PATH), os.W_OK):
+						raise
+					# The current user doesn't have permission to cache the
+					# file, but that's alright.
 			if pkgindex:
 				self._remotepkgs = {}
 				for d in pkgindex.packages:
 					self._remotepkgs[d["CPV"]] = d
 				self._remote_has_index = True
 				self._remote_base_uri = pkgindex.header.get("URI", base_url)
-				self.remotepkgs = {}
+				self.__remotepkgs = {}
 				for cpv in self._remotepkgs:
 					self.dbapi.cpv_inject(cpv)
 				self.populated = 1
+				if True:
+					# Remote package instances override local package
+					# if they are not identical.
+					hash_names = ["SIZE"] + self._pkgindex_hashes
+					for cpv, local_metadata in metadata.items():
+						remote_metadata = self._remotepkgs.get(cpv)
+						if remote_metadata is None:
+							continue
+						# Use digests to compare identity.
+						identical = True
+						for hash_name in hash_names:
+							local_value = local_metadata.get(hash_name)
+							if local_value is None:
+								continue
+							remote_value = remote_metadata.get(hash_name)
+							if remote_value is None:
+								continue
+							if local_value != remote_value:
+								identical = False
+								break
+						if identical:
+							del self._remotepkgs[cpv]
+						else:
+							# Override the local package in the aux_get cache.
+							self.dbapi._aux_cache[cpv] = remote_metadata
+				else:
+					# Local package instances override remote instances.
+					for cpv in metadata:
+						self._remotepkgs.pop(cpv, None)
 				return
 			self._remotepkgs = {}
 			try:
@@ -696,46 +829,62 @@ class binarytree(object):
 				chunk_size = 3000
 			writemsg_stdout("\n")
 			writemsg_stdout(
-				green("Fetching bininfo from ") + \
+				colorize("GOOD", _("Fetching bininfo from ")) + \
 				re.sub(r'//(.+):.+@(.+)/', r'//\1:*password*@\2/', base_url) + "\n")
-			self.remotepkgs = portage.getbinpkg.dir_get_metadata(
+			self.__remotepkgs = portage.getbinpkg.dir_get_metadata(
 				self.settings["PORTAGE_BINHOST"], chunk_size=chunk_size)
 			#writemsg(green("  -- DONE!\n\n"))
 
-			for mypkg in self.remotepkgs.keys():
-				if "CATEGORY" not in self.remotepkgs[mypkg]:
+			for mypkg in list(self.__remotepkgs):
+				if "CATEGORY" not in self.__remotepkgs[mypkg]:
 					#old-style or corrupt package
-					writemsg("!!! Invalid remote binary package: "+mypkg+"\n",
+					writemsg(_("!!! Invalid remote binary package: %s\n") % mypkg,
 						noiselevel=-1)
-					del self.remotepkgs[mypkg]
+					del self.__remotepkgs[mypkg]
 					continue
-				mycat = self.remotepkgs[mypkg]["CATEGORY"].strip()
+				mycat = self.__remotepkgs[mypkg]["CATEGORY"].strip()
 				fullpkg = mycat+"/"+mypkg[:-5]
+
+				if fullpkg in metadata:
+					# When using this old protocol, comparison with the remote
+					# package isn't supported, so the local package is always
+					# preferred even if getbinpkgsonly is enabled.
+					continue
+
 				if not self.dbapi._category_re.match(mycat):
-					writemsg(("!!! Remote binary package has an " + \
+					writemsg(_("!!! Remote binary package has an " \
 						"unrecognized category: '%s'\n") % fullpkg,
 						noiselevel=-1)
-					writemsg(("!!! '%s' has a category that is not" + \
+					writemsg(_("!!! '%s' has a category that is not" \
 						" listed in %setc/portage/categories\n") % \
 						(fullpkg, self.settings["PORTAGE_CONFIGROOT"]),
 						noiselevel=-1)
 					continue
-				mykey = dep_getkey(fullpkg)
+				mykey = portage.cpv_getkey(fullpkg)
 				try:
 					# invalid tbz2's can hurt things.
-					#print "cpv_inject("+str(fullpkg)+")"
 					self.dbapi.cpv_inject(fullpkg)
-					metadata = self.remotepkgs[mypkg]
-					for k, v in metadata.items():
-						metadata[k] = v.strip()
-					self._remotepkgs[fullpkg] = metadata
-					#print "  -- Injected"
-				except SystemExit, e:
+					remote_metadata = self.__remotepkgs[mypkg]
+					for k, v in remote_metadata.items():
+						remote_metadata[k] = v.strip()
+
+					# Eliminate metadata values with names that digestCheck
+					# uses, since they are not valid when using the old
+					# protocol. Typically this is needed for SIZE metadata
+					# which corresponds to the size of the unpacked files
+					# rather than the binpkg file size, triggering digest
+					# verification failures as reported in bug #303211.
+					remote_metadata.pop('SIZE', None)
+					for k in portage.checksum.hashfunc_map:
+						remote_metadata.pop(k, None)
+
+					self._remotepkgs[fullpkg] = remote_metadata
+				except SystemExit as e:
 					raise
 				except:
-					writemsg("!!! Failed to inject remote binary package:"+str(fullpkg)+"\n",
+					writemsg(_("!!! Failed to inject remote binary package: %s\n") % fullpkg,
 						noiselevel=-1)
-					del self.remotepkgs[mypkg]
+					del self.__remotepkgs[mypkg]
 					continue
 		self.populated=1
 
@@ -758,27 +907,27 @@ class binarytree(object):
 			full_path = filename
 		try:
 			s = os.stat(full_path)
-		except OSError, e:
+		except OSError as e:
 			if e.errno != errno.ENOENT:
 				raise
 			del e
-			writemsg("!!! Binary package does not exist: '%s'\n" % full_path,
+			writemsg(_("!!! Binary package does not exist: '%s'\n") % full_path,
 				noiselevel=-1)
 			return
 		mytbz2 = portage.xpak.tbz2(full_path)
 		slot = mytbz2.getfile("SLOT")
 		if slot is None:
-			writemsg("!!! Invalid binary package: '%s'\n" % full_path,
+			writemsg(_("!!! Invalid binary package: '%s'\n") % full_path,
 				noiselevel=-1)
 			return
 		slot = slot.strip()
 		self.dbapi.cpv_inject(cpv)
-		self.dbapi._aux_cache.pop(cpv, None)
 
 		# Reread the Packages index (in case it's been changed by another
 		# process) and then updated it, all while holding a lock.
 		from portage.locks import lockfile, unlockfile
 		pkgindex_lock = None
+		created_symlink = False
 		try:
 			pkgindex_lock = lockfile(self._pkgindex_file,
 				wantnewlockfile=1)
@@ -789,36 +938,37 @@ class binarytree(object):
 			if self._all_directory and \
 				self.getname(cpv).split(os.path.sep)[-2] == "All":
 				self._create_symlink(cpv)
-			pkgindex = self._new_pkgindex()
-			try:
-				f = open(self._pkgindex_file)
-			except EnvironmentError:
-				pass
-			else:
-				try:
-					pkgindex.read(f)
-				finally:
-					f.close()
-					del f
+				created_symlink = True
+			pkgindex = self._load_pkgindex()
+
 			if not self._pkgindex_version_supported(pkgindex):
 				pkgindex = self._new_pkgindex()
 
 			try:
 				d = self._pkgindex_entry(cpv)
 			except portage.exception.InvalidDependString:
-				writemsg("!!! Invalid binary package: '%s'\n" % \
+				writemsg(_("!!! Invalid binary package: '%s'\n") % \
 					self.getname(cpv), noiselevel=-1)
 				self.dbapi.cpv_remove(cpv)
 				del self._pkg_paths[cpv]
 				return
 
 			# If found, remove package(s) with duplicate path.
-			for i in xrange(len(pkgindex.packages) - 1, -1, -1):
+			path = d.get("PATH", "")
+			for i in range(len(pkgindex.packages) - 1, -1, -1):
 				d2 = pkgindex.packages[i]
-				if d2["CPV"] != cpv:
-					continue
-				if d2.get("PATH", "") == d.get("PATH", ""):
+				if path and path == d2.get("PATH"):
+					# Handle path collisions in $PKGDIR/All
+					# when CPV is not identical.
 					del pkgindex.packages[i]
+				elif cpv == d2.get("CPV"):
+					if path == d2.get("PATH", ""):
+						del pkgindex.packages[i]
+					elif created_symlink and not d2.get("PATH", ""):
+						# Delete entry for the package that was just
+						# overwritten by a symlink to this package.
+						del pkgindex.packages[i]
+
 			pkgindex.packages.append(d)
 
 			self._update_pkgindex_header(pkgindex.header)
@@ -832,6 +982,11 @@ class binarytree(object):
 			if pkgindex_lock:
 				unlockfile(pkgindex_lock)
 
+		if self._remotepkgs is not None:
+			# When a remote package is downloaded and injected,
+			# update state so self.isremote() returns False.
+			self._remotepkgs.pop(cpv, None)
+
 	def _pkgindex_entry(self, cpv):
 		"""
 		Performs checksums and evaluates USE flag conditionals.
@@ -843,7 +998,7 @@ class binarytree(object):
 		pkg_path = self.getname(cpv)
 		from portage.checksum import perform_multiple_checksums
 
-		d = dict(izip(self._pkgindex_aux_keys,
+		d = dict(zip(self._pkgindex_aux_keys,
 			self.dbapi.aux_get(cpv, self._pkgindex_aux_keys)))
 
 		d.update(perform_multiple_checksums(
@@ -851,7 +1006,7 @@ class binarytree(object):
 
 		d["CPV"] = cpv
 		st = os.stat(pkg_path)
-		d["MTIME"] = str(long(st.st_mtime))
+		d["MTIME"] = str(st[stat.ST_MTIME])
 		d["SIZE"] = str(st.st_size)
 
 		rel_path = self._pkg_paths[cpv]
@@ -903,7 +1058,7 @@ class binarytree(object):
 		return False
 
 	def _eval_use_flags(self, cpv, metadata):
-		use = metadata["USE"].split()
+		use = frozenset(metadata["USE"].split())
 		raw_use = use
 		iuse = set(f.lstrip("-+") for f in metadata["IUSE"].split())
 		use = [f for f in use if f in iuse]
@@ -917,10 +1072,20 @@ class binarytree(object):
 				deps = use_reduce(deps, uselist=raw_use)
 				deps = paren_normalize(deps)
 				deps = paren_enclose(deps)
-			except portage.exception.InvalidDependString, e:
+			except portage.exception.InvalidDependString as e:
 				writemsg("%s: %s\n" % (k, str(e)),
 					noiselevel=-1)
 				raise
+			if k in _vdb_use_conditional_atoms:
+				v_split = []
+				for x in deps.split():
+					try:
+						x = portage.dep.Atom(x)
+					except portage.exception.InvalidAtom:
+						v_split.append(x)
+					else:
+						v_split.append(str(x.evaluate_conditionals(raw_use)))
+				deps = ' '.join(v_split)
 			metadata[k] = deps
 
 	def exists_specific(self, cpv):
@@ -972,21 +1137,13 @@ class binarytree(object):
 		downloaded (or it is only partially downloaded)."""
 		if self._remotepkgs is None or pkgname not in self._remotepkgs:
 			return False
-		pkg_path = self.getname(pkgname)
-		if os.path.exists(pkg_path) and \
-			os.path.basename(pkg_path) not in self.invalids:
-			return False
+		# Presence in self._remotepkgs implies that it's remote. When a
+		# package is downloaded, state is updated by self.inject().
 		return True
 
-	def get_use(self, pkgname):
-		writemsg("deprecated use of binarytree.get_use()," + \
-			" use dbapi.aux_get() instead", noiselevel=-1)
-		return self.dbapi.aux_get(pkgname, ["USE"])[0].split()
-
 	def gettbz2(self, pkgname):
 		"""Fetches the package from a remote site, if necessary.  Attempts to
 		resume if the file appears to be partially downloaded."""
-		print "Fetching '"+str(pkgname)+"'"
 		tbz2_path = self.getname(pkgname)
 		tbz2name = os.path.basename(tbz2_path)
 		resume = False
@@ -995,12 +1152,15 @@ class binarytree(object):
 				return
 			else:
 				resume = True
-				writemsg("Resuming download of this tbz2, but it is possible that it is corrupt.\n",
+				writemsg(_("Resuming download of this tbz2, but it is possible that it is corrupt.\n"),
 					noiselevel=-1)
 		
 		mydest = os.path.dirname(self.getname(pkgname))
 		self._ensure_dir(mydest)
-		from urlparse import urlparse
+		try:
+			from urllib.parse import urlparse
+		except ImportError:
+			from urlparse import urlparse
 		# urljoin doesn't work correctly with unrecognized protocols like sftp
 		if self._remote_has_index:
 			rel_url = self._remotepkgs[pkgname].get("PATH")
@@ -1028,7 +1188,10 @@ class binarytree(object):
 	def _load_pkgindex(self):
 		pkgindex = self._new_pkgindex()
 		try:
-			f = open(self._pkgindex_file)
+			f = codecs.open(_unicode_encode(self._pkgindex_file,
+				encoding=_encodings['fs'], errors='strict'),
+				mode='r', encoding=_encodings['repo.content'],
+				errors='replace')
 		except EnvironmentError:
 			pass
 		else:
@@ -1074,8 +1237,8 @@ class binarytree(object):
 			try:
 				digests["size"] = int(metadata["SIZE"])
 			except ValueError:
-				writemsg("!!! Malformed SIZE attribute in remote " + \
-				"metadata for '%s'\n" % cpv)
+				writemsg(_("!!! Malformed SIZE attribute in remote " \
+				"metadata for '%s'\n") % cpv)
 
 		if not digests:
 			return False
@@ -1096,8 +1259,8 @@ class binarytree(object):
 		myslot = ""
 		try:
 			myslot = self.dbapi.aux_get(mycatpkg,["SLOT"])[0]
-		except SystemExit, e:
+		except SystemExit as e:
 			raise
-		except Exception, e:
+		except Exception as e:
 			pass
 		return myslot
