@@ -1,22 +1,25 @@
 # Copyright 1998-2009 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: porttree.py 14531 2009-10-09 22:13:04Z zmedico $
+# $Id: porttree.py 15695 2010-03-02 21:09:26Z zmedico $
 
-from __future__ import print_function
-
-__all__ = ["portdbapi", "close_portdbapi_caches", "portagetree"]
+__all__ = [
+	"close_portdbapi_caches", "FetchlistDict", "portagetree", "portdbapi"
+]
 
 import portage
 portage.proxy.lazyimport.lazyimport(globals(),
 	'portage.checksum',
-	'portage.dep:dep_getkey,match_from_list,paren_reduce,use_reduce',
+	'portage.dbapi.dep_expand:dep_expand',
+	'portage.dep:dep_getkey,flatten,match_from_list,paren_reduce,use_reduce',
 	'portage.env.loaders:KeyValuePairFileLoader',
+	'portage.package.ebuild.doebuild:doebuild',
 	'portage.util:ensure_dirs,writemsg,writemsg_level',
+	'portage.util.listdir:listdir',
 	'portage.versions:best,catpkgsplit,_pkgsplit@pkgsplit,ver_regexp',
 )
 
 from portage.cache.cache_errors import CacheError
-from portage.cache.mappings import slot_dict_class
+from portage.cache.mappings import Mapping
 from portage.const import REPO_NAME_LOC
 from portage.data import portage_gid, secpass
 from portage.dbapi import dbapi
@@ -25,13 +28,14 @@ from portage.exception import PortageException, \
 from portage.localization import _
 from portage.manifest import Manifest
 
-from portage import eclass_cache, auxdbkeys, doebuild, flatten, \
-	listdir, dep_expand, eapi_is_supported, key_expand, dep_check, \
+from portage import eclass_cache, auxdbkeys, \
+	eapi_is_supported, dep_check, \
 	_eapi_is_deprecated
 from portage import os
 from portage import _encodings
 from portage import _unicode_decode
 from portage import _unicode_encode
+from portage import OrderedDict
 
 import os as _os
 import codecs
@@ -125,38 +129,60 @@ class portdbapi(dbapi):
 	_use_mutable = True
 
 	def _get_settings(self):
-		return self.mysettings
+		warnings.warn("Use portdbapi.settings instead of portdbapi.mysettings",
+			DeprecationWarning)
+		return self.settings
 
 	def _set_settings(self, settings):
-		self.mysettings = settings
+		warnings.warn("Use portdbapi.settings instead of portdbapi.mysettings",
+			DeprecationWarning)
+		self.settings = settings
 
 	def _del_settings (self):
-		del self.mysettings
+		warnings.warn("Use portdbapi.settings instead of portdbapi.mysettings",
+			DeprecationWarning)
+		del self.settings
 
-	settings = property(_get_settings, _set_settings, _del_settings,
-		"Define self.settings as an alias for self.mysettings, " + \
-		"for conformity with other dbapi classes.")
+	mysettings = property(_get_settings, _set_settings, _del_settings,
+		"Deprecated self.mysettings, only for backward compatibility")
 
 	@property
 	def _categories(self):
 		return self.settings.categories
 
-	def __init__(self, porttree_root, mysettings=None):
+	def __init__(self, _unused_param=None, mysettings=None):
+		"""
+		@param _unused_param: deprecated, use mysettings['PORTDIR'] instead
+		@type _unused_param: None
+		@param mysettings: an immutable config instance
+		@type mysettings: portage.config
+		"""
 		portdbapi.portdbapi_instances.append(self)
 
 		from portage import config
 		if mysettings:
-			self.mysettings = mysettings
+			self.settings = mysettings
 		else:
 			from portage import settings
-			self.mysettings = config(clone=settings)
+			self.settings = config(clone=settings)
+
+		porttree_root = self.settings['PORTDIR']
+
+		# always show this warning after this parameter
+		# is unused in stable portage
+		if _unused_param is not None and _unused_param != porttree_root:
+			warnings.warn("The first parameter of the " + \
+				"portage.dbapi.porttree.portdbapi" + \
+				" constructor is now unused. Use " + \
+				"mysettings['PORTDIR'] instead.",
+				DeprecationWarning, stacklevel=2)
 
 		# This is strictly for use in aux_get() doebuild calls when metadata
 		# is generated by the depend phase.  It's safest to use a clone for
 		# this purpose because doebuild makes many changes to the config
 		# instance that is passed in.
-		self.doebuild_settings = config(clone=self.mysettings)
-		self.depcachedir = os.path.realpath(self.mysettings.depcachedir)
+		self.doebuild_settings = config(clone=self.settings)
+		self.depcachedir = os.path.realpath(self.settings.depcachedir)
 
 		if os.environ.get("SANDBOX_ON") == "1":
 			# Make api consumers exempt from sandbox violations
@@ -169,7 +195,7 @@ class portdbapi(dbapi):
 
 		porttrees = [os.path.realpath(porttree_root)]
 		porttrees.extend(os.path.realpath(x) for x in \
-			self.mysettings.get('PORTDIR_OVERLAY', '').split())
+			self.settings.get('PORTDIR_OVERLAY', '').split())
 		treemap = {}
 		repository_map = {}
 		self.treemap = treemap
@@ -225,7 +251,7 @@ class portdbapi(dbapi):
 		self._have_root_eclass_dir = os.path.isdir(
 			os.path.join(self.porttree_root, "eclass"))
 
-		self.metadbmodule = self.mysettings.load_best_module("portdbapi.metadbmodule")
+		self.metadbmodule = self.settings.load_best_module("portdbapi.metadbmodule")
 
 		#if the portdbapi is "frozen", then we assume that we can cache everything (that no updates to it are happening)
 		self.xcache = {}
@@ -233,7 +259,7 @@ class portdbapi(dbapi):
 
 		self._repo_info = {}
 		eclass_dbs = {porttree_root : self.eclassdb}
-		local_repo_configs = self.mysettings._local_repo_configs
+		local_repo_configs = self.settings._local_repo_configs
 		default_loc_repo_config = None
 		repo_aliases = {}
 		if local_repo_configs is not None:
@@ -248,7 +274,7 @@ class portdbapi(dbapi):
 								"'%s' alias in " \
 								"'%s'\n") % (alias, repo_name,
 								overridden_alias,
-								self.mysettings._local_repo_conf_path),
+								self.settings._local_repo_conf_path),
 								level=logging.WARNING, noiselevel=-1)
 						repo_aliases[alias] = repo_name
 
@@ -307,7 +333,7 @@ class portdbapi(dbapi):
 							writemsg_level(_("Unavailable repository '%s' " \
 								"referenced by eclass-overrides entry in " \
 								"'%s'\n") % (other_name,
-								self.mysettings._local_repo_conf_path),
+								self.settings._local_repo_conf_path),
 								level=logging.ERROR, noiselevel=-1)
 							continue
 						porttrees.append(other_path)
@@ -325,7 +351,7 @@ class portdbapi(dbapi):
 
 			self._repo_info[path] = _repo_info(repo_name, path, eclass_db)
 
-		self.auxdbmodule = self.mysettings.load_best_module("portdbapi.auxdbmodule")
+		self.auxdbmodule = self.settings.load_best_module("portdbapi.auxdbmodule")
 		self.auxdb = {}
 		self._pregen_auxdb = {}
 		self._init_cache_dirs()
@@ -362,7 +388,7 @@ class portdbapi(dbapi):
 					self.depcachedir, x, filtered_auxdbkeys, **cache_kwargs)
 				if self.auxdbmodule is metadata_overlay.database:
 					self.auxdb[x].db_ro.ec = self._repo_info[x].eclass_db
-		if "metadata-transfer" not in self.mysettings.features:
+		if "metadata-transfer" not in self.settings.features:
 			for x in self.porttrees:
 				if x in self._pregen_auxdb:
 					continue
@@ -430,6 +456,18 @@ class portdbapi(dbapi):
 			return self.treemap[repository_id]
 		return None
 
+	def getRepositoryName(self, canonical_repo_path):
+		"""
+		This is the inverse of getRepositoryPath().
+		@param canonical_repo_path: the canonical path of a repository, as
+			resolved by os.path.realpath()
+		@type canonical_repo_path: String
+		@returns: The repo_name for the corresponding repository, or None
+			if the path does not correspond a known repository
+		@rtype: String or None
+		"""
+		return self._repository_map.get(canonical_repo_path)
+
 	def getRepositories(self):
 		"""
 		This function is required for GLEP 42 compliance; it will return a list of
@@ -609,7 +647,11 @@ class portdbapi(dbapi):
 				return [aux_cache.get(x, "") for x in mylist]
 			cache_me = True
 		global auxdbkeys, auxdbkeylen
-		cat,pkg = mycpv.split("/", 1)
+		try:
+			cat, pkg = mycpv.split("/", 1)
+		except ValueError:
+			# Missing slash. Can't find ebuild so raise KeyError.
+			raise KeyError(mycpv)
 
 		myebuild, mylocation = self.findname2(mycpv, mytree)
 
@@ -668,7 +710,7 @@ class portdbapi(dbapi):
 		mydata["repository"] = self._repository_map.get(mylocation, "")
 
 		mydata["INHERITED"] = ' '.join(mydata.get("_eclasses_", []))
-		mydata["_mtime_"] = long(st.st_mtime)
+		mydata["_mtime_"] = st[stat.ST_MTIME]
 
 		eapi = mydata.get("EAPI")
 		if not eapi:
@@ -730,7 +772,7 @@ class portdbapi(dbapi):
 			matchall=(useflags is None))
 		myuris = flatten(myuris)
 
-		uri_map = {}
+		uri_map = OrderedDict()
 
 		myuris.reverse()
 		while myuris:
@@ -755,42 +797,18 @@ class portdbapi(dbapi):
 
 		return uri_map
 
-	def getfetchlist(self, mypkg, useflags=None, mysettings=None,
-		all=0, mytree=None):
-
-		writemsg("!!! pordbapi.getfetchlist() is deprecated, " + \
-			"use getFetchMap() instead.\n", noiselevel=-1)
-
-		if all:
-			useflags = None
-		elif useflags is None:
-			if mysettings is None:
-				mysettings = self.doebuild_settings
-			mysettings.setcpv(mypkg, mydb=self)
-			useflags = mysettings["PORTAGE_USE"].split()
-		uri_map = self.getFetchMap(mypkg, useflags=useflags, mytree=mytree)
-
-		all_uris = []
-		all_files = []
-		for filename, uris in uri_map.items():
-			for uri in uris:
-				all_uris.append(uri)
-				all_files.append(filename)
-
-		return [all_uris, all_files]
-
 	def getfetchsizes(self, mypkg, useflags=None, debug=0):
 		# returns a filename:size dictionnary of remaining downloads
 		myebuild = self.findname(mypkg)
 		if myebuild is None:
 			raise AssertionError("ebuild not found for '%s'" % mypkg)
 		pkgdir = os.path.dirname(myebuild)
-		mf = Manifest(pkgdir, self.mysettings["DISTDIR"])
+		mf = Manifest(pkgdir, self.settings["DISTDIR"])
 		checksums = mf.getDigests()
 		if not checksums:
 			if debug: 
-				print("[empty/missing/bad digest]: "+mypkg)
-			return None
+				writemsg("[empty/missing/bad digest]: %s\n" % (mypkg,))
+			return {}
 		filesdict={}
 		myfiles = self.getFetchMap(mypkg, useflags=useflags)
 		#XXX: maybe this should be improved: take partial downloads
@@ -800,7 +818,7 @@ class portdbapi(dbapi):
 				if debug:
 					writemsg(_("[bad digest]: missing %(file)s for %(pkg)s\n") % {"file":myfile, "pkg":mypkg})
 				continue
-			file_path = os.path.join(self.mysettings["DISTDIR"], myfile)
+			file_path = os.path.join(self.settings["DISTDIR"], myfile)
 			mystat = None
 			try:
 				mystat = os.stat(file_path)
@@ -830,7 +848,7 @@ class portdbapi(dbapi):
 		if myebuild is None:
 			raise AssertionError("ebuild not found for '%s'" % mypkg)
 		pkgdir = os.path.dirname(myebuild)
-		mf = Manifest(pkgdir, self.mysettings["DISTDIR"])
+		mf = Manifest(pkgdir, self.settings["DISTDIR"])
 		mysums = mf.getDigests()
 
 		failures = {}
@@ -841,7 +859,7 @@ class portdbapi(dbapi):
 			else:
 				try:
 					ok, reason = portage.checksum.verify_all(
-						os.path.join(self.mysettings["DISTDIR"], x), mysums[x])
+						os.path.join(self.settings["DISTDIR"], x), mysums[x])
 				except FileNotFound as e:
 					ok = False
 					reason = _("File Not Found: '%s'") % (e,)
@@ -863,11 +881,22 @@ class portdbapi(dbapi):
 		else:
 			return 0
 
-	def cp_all(self):
-		"returns a list of all keys in our tree"
+	def cp_all(self, categories=None, trees=None):
+		"""
+		This returns a list of all keys in our tree or trees
+		@param categories: optional list of categories to search or 
+			defaults to self.settings.categories
+		@param trees: optional list of trees to search the categories in or
+			defaults to self.porttrees
+		@rtype list of [cat/pkg,...]
+		"""
 		d = {}
-		for x in self.mysettings.categories:
-			for oroot in self.porttrees:
+		if categories is None:
+			categories = self.settings.categories
+		if trees is None:
+			trees = self.porttrees
+		for x in categories:
+			for oroot in trees:
 				for y in listdir(oroot+"/"+x, EmptyOnError=1, ignorecvs=1, dirsonly=1):
 					if not self._pkg_dir_name_re.match(y) or \
 						y == "CVS":
@@ -927,7 +956,7 @@ class portdbapi(dbapi):
 		if invalid_category and d:
 			writemsg(_("\n!!! '%s' has a category that is not listed in " \
 				"%setc/portage/categories\n") % \
-				(mycp, self.mysettings["PORTAGE_CONFIGROOT"]), noiselevel=-1)
+				(mycp, self.settings["PORTAGE_CONFIGROOT"]), noiselevel=-1)
 			mylist = []
 		else:
 			mylist = list(d)
@@ -965,7 +994,7 @@ class portdbapi(dbapi):
 		if not mydep:
 			#this stuff only runs on first call of xmatch()
 			#create mydep, mykey from origdep
-			mydep = dep_expand(origdep, mydb=self, settings=self.mysettings)
+			mydep = dep_expand(origdep, mydb=self, settings=self.settings)
 			mykey = mydep.cp
 
 		if level == "list-visible":
@@ -994,7 +1023,7 @@ class portdbapi(dbapi):
 			else:
 				mylist = match_from_list(mydep, self.cp_list(mykey))
 			myval = ""
-			settings = self.mysettings
+			settings = self.settings
 			local_config = settings.local_config
 			aux_keys = list(self._aux_cache_keys)
 			if level == "minimum-visible":
@@ -1060,8 +1089,8 @@ class portdbapi(dbapi):
 			else:
 				myval = list(self._iter_match(mydep, self.cp_list(mykey)))
 		else:
-			print("ERROR: xmatch doesn't handle", level, "query!")
-			raise KeyError
+			raise AssertionError(
+				"Invalid level argument: '%s'" % level)
 
 		if self.frozen and (level not in ["match-list", "bestmatch-list"]):
 			self.xcache[level][mydep] = myval
@@ -1081,8 +1110,8 @@ class portdbapi(dbapi):
 
 		db_keys = ["SLOT"]
 		visible = []
-		getMaskAtom = self.mysettings._getMaskAtom
-		getProfileMaskAtom = self.mysettings._getProfileMaskAtom
+		getMaskAtom = self.settings._getMaskAtom
+		getProfileMaskAtom = self.settings._getProfileMaskAtom
 		for cpv in mylist:
 			try:
 				metadata = dict(zip(db_keys, self.aux_get(cpv, db_keys)))
@@ -1106,9 +1135,9 @@ class portdbapi(dbapi):
 		newlist=[]
 		aux_keys = list(self._aux_cache_keys)
 		metadata = {}
-		local_config = self.mysettings.local_config
-		chost = self.mysettings.get('CHOST', '')
-		accept_chost = self.mysettings._accept_chost
+		local_config = self.settings.local_config
+		chost = self.settings.get('CHOST', '')
+		accept_chost = self.settings._accept_chost
 		for mycpv in mylist:
 			metadata.clear()
 			try:
@@ -1126,7 +1155,7 @@ class portdbapi(dbapi):
 				continue
 			if _eapi_is_deprecated(eapi):
 				continue
-			if self.mysettings._getMissingKeywords(mycpv, metadata):
+			if self.settings._getMissingKeywords(mycpv, metadata):
 				continue
 			if local_config:
 				metadata['CHOST'] = chost
@@ -1137,9 +1166,9 @@ class portdbapi(dbapi):
 					self.doebuild_settings.setcpv(mycpv, mydb=metadata)
 					metadata['USE'] = self.doebuild_settings['PORTAGE_USE']
 				try:
-					if self.mysettings._getMissingLicenses(mycpv, metadata):
+					if self.settings._getMissingLicenses(mycpv, metadata):
 						continue
-					if self.mysettings._getMissingProperties(mycpv, metadata):
+					if self.settings._getMissingProperties(mycpv, metadata):
 						continue
 				except InvalidDependString:
 					continue
@@ -1151,7 +1180,7 @@ def close_portdbapi_caches():
 		i.close_caches()
 
 class portagetree(object):
-	def __init__(self, root="/", virtual=None, clone=None, settings=None):
+	def __init__(self, root="/", virtual=None, settings=None):
 		"""
 		Constructor for a PortageTree
 		
@@ -1159,27 +1188,18 @@ class portagetree(object):
 		@type root: String/Path
 		@param virtual: UNUSED
 		@type virtual: No Idea
-		@param clone: Set this if you want a copy of Clone
-		@type clone: Existing portagetree Instance
 		@param settings: Portage Configuration object (portage.settings)
 		@type settings: Instance of portage.config
 		"""
 
-		if clone:
-			writemsg("portagetree.__init__(): deprecated " + \
-				"use of clone parameter\n", noiselevel=-1)
-			self.root = clone.root
-			self.portroot = clone.portroot
-			self.pkglines = clone.pkglines
-		else:
+		if True:
 			self.root = root
 			if settings is None:
 				from portage import settings
 			self.settings = settings
 			self.portroot = settings["PORTDIR"]
 			self.virtual = virtual
-			self.dbapi = portdbapi(
-				settings["PORTDIR"], mysettings=settings)
+			self.dbapi = portdbapi(mysettings=settings)
 
 	def dep_bestmatch(self,mydep):
 		"compatibility method"
@@ -1211,20 +1231,6 @@ class portagetree(object):
 		psplit = pkgsplit(mysplit[1])
 		return "/".join([self.portroot, mysplit[0], psplit[0], mysplit[1]])+".ebuild"
 
-	def resolve_specific(self, myspec):
-		warnings.warn(
-			"portage.dbapi.porttree.portagetree.resolve_specific() is deprecated",
-			DeprecationWarning)
-		cps = catpkgsplit(myspec)
-		if not cps:
-			return None
-		mykey = key_expand(cps[0]+"/"+cps[1], mydb=self.dbapi,
-			settings=self.settings)
-		mykey = mykey + "-" + cps[2]
-		if cps[3] != "r0":
-			mykey = mykey + "-" + cps[3]
-		return mykey
-
 	def depcheck(self, mycheck, use="yes", myusesplit=None):
 		return dep_check(mycheck, self.dbapi, use=use, myuse=myusesplit)
 
@@ -1238,3 +1244,44 @@ class portagetree(object):
 		except Exception as e:
 			pass
 		return myslot
+
+class FetchlistDict(Mapping):
+	"""
+	This provide a mapping interface to retrieve fetch lists. It's used
+	to allow portage.manifest.Manifest to access fetch lists via a standard
+	mapping interface rather than use the dbapi directly.
+	"""
+	def __init__(self, pkgdir, settings, mydbapi):
+		"""pkgdir is a directory containing ebuilds and settings is passed into
+		portdbapi.getfetchlist for __getitem__ calls."""
+		self.pkgdir = pkgdir
+		self.cp = os.sep.join(pkgdir.split(os.sep)[-2:])
+		self.settings = settings
+		self.mytree = os.path.realpath(os.path.dirname(os.path.dirname(pkgdir)))
+		self.portdb = mydbapi
+
+	def __getitem__(self, pkg_key):
+		"""Returns the complete fetch list for a given package."""
+		return list(self.portdb.getFetchMap(pkg_key, mytree=self.mytree))
+
+	def __contains__(self, cpv):
+		return cpv in self.__iter__()
+
+	def has_key(self, pkg_key):
+		"""Returns true if the given package exists within pkgdir."""
+		return pkg_key in self
+
+	def __iter__(self):
+		return iter(self.portdb.cp_list(self.cp, mytree=self.mytree))
+
+	def __len__(self):
+		"""This needs to be implemented in order to avoid
+		infinite recursion in some cases."""
+		return len(self.portdb.cp_list(self.cp, mytree=self.mytree))
+
+	def keys(self):
+		"""Returns keys for all packages within pkgdir"""
+		return self.portdb.cp_list(self.cp, mytree=self.mytree)
+
+	if sys.hexversion >= 0x3000000:
+		keys = __iter__
