@@ -1,7 +1,7 @@
 # getbinpkg.py -- Portage binary-package helper functions
 # Copyright 2003-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: getbinpkg.py 14531 2009-10-09 22:13:04Z zmedico $
+# $Id: getbinpkg.py 15246 2010-01-29 18:47:51Z zmedico $
 
 from portage.output import colorize
 from portage.cache.mappings import slot_dict_class
@@ -638,7 +638,7 @@ def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=
 			self.out.flush()
 
 	cache_stats = CacheStats(out)
-	have_tty = out.isatty()
+	have_tty = os.environ.get('TERM') != 'dumb' and out.isatty()
 	if have_tty:
 		cache_stats.display()
 	binpkg_filenames = set()
