@@ -1,19 +1,27 @@
 # portage: Lock management code
 # Copyright 2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: locks.py 12177 2008-12-07 20:17:27Z zmedico $
+# $Id: locks.py 15269 2010-01-29 18:53:15Z zmedico $
 
 __all__ = ["lockdir", "unlockdir", "lockfile", "unlockfile", \
 	"hardlock_name", "hardlink_is_mine", "hardlink_lockfile", \
 	"unhardlink_lockfile", "hardlock_cleanup"]
 
-import errno, os, stat, time, types
+import errno
+import stat
+import sys
+import time
+from portage import os
 from portage.exception import DirectoryNotFound, FileNotFound, \
-	InvalidData, TryAgain
+	InvalidData, TryAgain, OperationNotPermitted, PermissionDenied
 from portage.data import portage_gid
+from portage.output import EOutput
 from portage.util import writemsg
 from portage.localization import _
 
+if sys.hexversion >= 0x3000000:
+	basestring = str
+
 HARDLINK_FD = -2
 
 # Used by emerge in order to disable the "waiting for lock" message
@@ -27,19 +35,21 @@ def unlockdir(mylock):
 
 def lockfile(mypath, wantnewlockfile=0, unlinkfile=0,
 	waiting_msg=None, flags=0):
-	"""Creates all dirs upto, the given dir. Creates a lockfile
-	for the given directory as the file: directoryname+'.portage_lockfile'."""
+	"""
+	If wantnewlockfile is True then this creates a lockfile in the parent
+	directory as the file: '.' + basename + '.portage_lockfile'.
+	"""
 	import fcntl
 
 	if not mypath:
-		raise InvalidData("Empty path given")
+		raise InvalidData(_("Empty path given"))
 
-	if type(mypath) == types.StringType and mypath[-1] == '/':
+	if isinstance(mypath, basestring) and mypath[-1] == '/':
 		mypath = mypath[:-1]
 
-	if type(mypath) == types.FileType:
+	if hasattr(mypath, 'fileno'):
 		mypath = mypath.fileno()
-	if type(mypath) == types.IntType:
+	if isinstance(mypath, int):
 		lockfilename    = mypath
 		wantnewlockfile = 0
 		unlinkfile      = 0
@@ -50,32 +60,46 @@ def lockfile(mypath, wantnewlockfile=0, unlinkfile=0,
 		unlinkfile   = 1
 	else:
 		lockfilename = mypath
-	
-	if type(mypath) == types.StringType:
+
+	if isinstance(mypath, basestring):
 		if not os.path.exists(os.path.dirname(mypath)):
 			raise DirectoryNotFound(os.path.dirname(mypath))
-		if not os.path.exists(lockfilename):
-			old_mask=os.umask(000)
-			myfd = os.open(lockfilename, os.O_CREAT|os.O_RDWR,0660)
+		preexisting = os.path.exists(lockfilename)
+		old_mask = os.umask(000)
+		try:
 			try:
-				if os.stat(lockfilename).st_gid != portage_gid:
-					os.chown(lockfilename,os.getuid(),portage_gid)
-			except OSError, e:
-				if e[0] == 2: # No such file or directory
-					return lockfile(mypath, wantnewlockfile=wantnewlockfile,
-						unlinkfile=unlinkfile, waiting_msg=waiting_msg,
-						flags=flags)
+				myfd = os.open(lockfilename, os.O_CREAT|os.O_RDWR, 0o660)
+			except OSError as e:
+				func_call = "open('%s')" % lockfilename
+				if e.errno == OperationNotPermitted.errno:
+					raise OperationNotPermitted(func_call)
+				elif e.errno == PermissionDenied.errno:
+					raise PermissionDenied(func_call)
 				else:
-					writemsg("Cannot chown a lockfile. This could cause inconvenience later.\n");
-			os.umask(old_mask)
-		else:
-			myfd = os.open(lockfilename, os.O_CREAT|os.O_RDWR,0660)
+					raise
 
-	elif type(mypath) == types.IntType:
+			if not preexisting:
+				try:
+					if os.stat(lockfilename).st_gid != portage_gid:
+						os.chown(lockfilename, -1, portage_gid)
+				except OSError as e:
+					if e.errno in (errno.ENOENT, errno.ESTALE):
+						return lockfile(mypath,
+							wantnewlockfile=wantnewlockfile,
+							unlinkfile=unlinkfile, waiting_msg=waiting_msg,
+							flags=flags)
+					else:
+						writemsg(_("Cannot chown a lockfile: '%s'\n") % \
+							lockfilename, noiselevel=-1)
+
+		finally:
+			os.umask(old_mask)
+
+	elif isinstance(mypath, int):
 		myfd = mypath
 
 	else:
-		raise ValueError("Unknown type passed in '%s': '%s'" % \
+		raise ValueError(_("Unknown type passed in '%s': '%s'") % \
 			(type(mypath), mypath))
 
 	# try for a non-blocking lock, if it's held, throw a message
@@ -83,7 +107,7 @@ def lockfile(mypath, wantnewlockfile=0, unlinkfile=0,
 	locking_method = fcntl.lockf
 	try:
 		fcntl.lockf(myfd,fcntl.LOCK_EX|fcntl.LOCK_NB)
-	except IOError, e:
+	except IOError as e:
 		if "errno" not in dir(e):
 			raise
 		if e.errno in (errno.EACCES, errno.EAGAIN):
@@ -92,19 +116,21 @@ def lockfile(mypath, wantnewlockfile=0, unlinkfile=0,
 				raise TryAgain(mypath)
 
 			global _quiet
-			if _quiet:
-				pass
-			elif waiting_msg is None:
+			out = EOutput()
+			out.quiet = _quiet
+			if waiting_msg is None:
 				if isinstance(mypath, int):
-					writemsg("waiting for lock on fd %i\n" % myfd,
-						noiselevel=-1)
+					waiting_msg = _("waiting for lock on fd %i") % myfd
 				else:
-					writemsg("waiting for lock on %s\n" % lockfilename,
-						noiselevel=-1)
-			elif waiting_msg:
-				writemsg(waiting_msg + "\n", noiselevel=-1)
+					waiting_msg = _("waiting for lock on %s\n") % lockfilename
+			out.ebegin(waiting_msg)
 			# try for the exclusive lock now.
-			fcntl.lockf(myfd,fcntl.LOCK_EX)
+			try:
+				fcntl.lockf(myfd, fcntl.LOCK_EX)
+			except EnvironmentError as e:
+				out.eend(1, str(e))
+				raise
+			out.eend(os.EX_OK)
 		elif e.errno == errno.ENOLCK:
 			# We're not allowed to lock on this FS.
 			os.close(myfd)
@@ -125,11 +151,11 @@ def lockfile(mypath, wantnewlockfile=0, unlinkfile=0,
 			raise
 
 		
-	if type(lockfilename) == types.StringType and \
+	if isinstance(lockfilename, basestring) and \
 		myfd != HARDLINK_FD and _fstat_nlink(myfd) == 0:
 		# The file was deleted on us... Keep trying to make one...
 		os.close(myfd)
-		writemsg("lockfile recurse\n",1)
+		writemsg(_("lockfile recurse\n"), 1)
 		lockfilename, myfd, unlinkfile, locking_method = lockfile(
 			mypath, wantnewlockfile=wantnewlockfile, unlinkfile=unlinkfile,
 			waiting_msg=waiting_msg, flags=flags)
@@ -146,8 +172,8 @@ def _fstat_nlink(fd):
 	"""
 	try:
 		return os.fstat(fd).st_nlink
-	except EnvironmentError, e:
-		if e.errno == errno.ENOENT:
+	except EnvironmentError as e:
+		if e.errno in (errno.ENOENT, errno.ESTALE):
 			# Some filesystems such as CIFS return
 			# ENOENT which means st_nlink == 0.
 			return 0
@@ -170,21 +196,22 @@ def unlockfile(mytuple):
 		return True
 	
 	# myfd may be None here due to myfd = mypath in lockfile()
-	if type(lockfilename) == types.StringType and not os.path.exists(lockfilename):
-		writemsg("lockfile does not exist '%s'\n" % lockfilename,1)
+	if isinstance(lockfilename, basestring) and \
+		not os.path.exists(lockfilename):
+		writemsg(_("lockfile does not exist '%s'\n") % lockfilename,1)
 		if myfd is not None:
 			os.close(myfd)
 		return False
 
 	try:
 		if myfd is None:
-			myfd = os.open(lockfilename, os.O_WRONLY,0660)
+			myfd = os.open(lockfilename, os.O_WRONLY,0o660)
 			unlinkfile = 1
 		locking_method(myfd,fcntl.LOCK_UN)
 	except OSError:
-		if type(lockfilename) == types.StringType:
+		if isinstance(lockfilename, basestring):
 			os.close(myfd)
-		raise IOError("Failed to unlock file '%s'\n" % lockfilename)
+		raise IOError(_("Failed to unlock file '%s'\n") % lockfilename)
 
 	try:
 		# This sleep call was added to allow other processes that are
@@ -197,23 +224,23 @@ def unlockfile(mytuple):
 			locking_method(myfd,fcntl.LOCK_EX|fcntl.LOCK_NB)
 			# We won the lock, so there isn't competition for it.
 			# We can safely delete the file.
-			writemsg("Got the lockfile...\n",1)
+			writemsg(_("Got the lockfile...\n"), 1)
 			if _fstat_nlink(myfd) == 1:
 				os.unlink(lockfilename)
-				writemsg("Unlinked lockfile...\n",1)
+				writemsg(_("Unlinked lockfile...\n"), 1)
 				locking_method(myfd,fcntl.LOCK_UN)
 			else:
-				writemsg("lockfile does not exist '%s'\n" % lockfilename,1)
+				writemsg(_("lockfile does not exist '%s'\n") % lockfilename, 1)
 				os.close(myfd)
 				return False
-	except Exception, e:
-		writemsg("Failed to get lock... someone took it.\n",1)
+	except Exception as e:
+		writemsg(_("Failed to get lock... someone took it.\n"), 1)
 		writemsg(str(e)+"\n",1)
 
 	# why test lockfilename?  because we may have been handed an
 	# fd originally, and the caller might not like having their
 	# open fd closed automatically on them.
-	if type(lockfilename) == types.StringType:
+	if isinstance(lockfilename, basestring):
 		os.close(myfd)
 
 	return True
@@ -245,7 +272,7 @@ def hardlink_lockfile(lockfilename, max_wait=14400):
 	
 	while(time.time() < (start_time + max_wait)):
 		# We only need it to exist.
-		myfd = os.open(myhardlock, os.O_CREAT|os.O_RDWR,0660)
+		myfd = os.open(myhardlock, os.O_CREAT|os.O_RDWR,0o660)
 		os.close(myfd)
 	
 		if not os.path.exists(myhardlock):
@@ -269,10 +296,10 @@ def hardlink_lockfile(lockfilename, max_wait=14400):
 		else:
 			reported_waiting = True
 			from portage.const import PORTAGE_BIN_PATH
-			msg = "\nWaiting on (hardlink) lockfile:" + \
-				" (one '.' per 3 seconds)\n" + \
-				"%s/clean_locks can fix stuck locks.\n" % PORTAGE_BIN_PATH + \
-				"Lockfile: %s\n" % lockfilename
+			msg = _("\nWaiting on (hardlink) lockfile: (one '.' per 3 seconds)\n"
+				"%(bin_path)s/clean_locks can fix stuck locks.\n"
+				"Lockfile: %(lockfilename)s\n") % \
+				{"bin_path": PORTAGE_BIN_PATH, "lockfilename": lockfilename}
 			writemsg(msg, noiselevel=-1)
 		time.sleep(3)
 	
@@ -319,7 +346,7 @@ def hardlock_cleanup(path, remove_all_locks=False):
 				mycount += 1
 
 
-	results.append("Found %(count)s locks" % {"count":mycount})
+	results.append(_("Found %(count)s locks") % {"count":mycount})
 	
 	for x in mylist:
 		if myhost in mylist[x] or remove_all_locks:
