@@ -1,7 +1,7 @@
 # portage: news management code
 # Copyright 2006 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: news.py 14327 2009-09-21 16:07:07Z arfrever $
+# $Id: news.py 15541 2010-03-02 19:51:44Z zmedico $
 
 __all__ = ["NewsManager", "NewsItem", "DisplayRestriction",
 	"DisplayProfileRestriction", "DisplayKeywordRestriction",
@@ -59,9 +59,9 @@ class NewsManager(object):
 		portdir = portdb.porttree_root
 		profiles_base = os.path.join(portdir, 'profiles') + os.path.sep
 		profile_path = None
-		if portdb.mysettings.profile_path:
+		if portdb.settings.profile_path:
 			profile_path = normalize_path(
-				os.path.realpath(portdb.mysettings.profile_path))
+				os.path.realpath(portdb.settings.profile_path))
 			if profile_path.startswith(profiles_base):
 				profile_path = profile_path[len(profiles_base):]
 		self._profile_path = profile_path
