@@ -1,6 +1,6 @@
 # Copyright 1998-2009 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: output.py 14327 2009-09-21 16:07:07Z arfrever $
+# $Id: output.py 15245 2010-01-29 18:47:30Z zmedico $
 
 __docformat__ = "epytext"
 
@@ -251,11 +251,15 @@ def xtermTitle(mystr, raw=False):
 			mystr = mystr[:_max_xtermTitle_len]
 		if not raw:
 			mystr = '\x1b]0;%s\x07' % mystr
-		if sys.hexversion < 0x3000000 and isinstance(mystr, unicode):
-			# avoid potential UnicodeEncodeError
-			mystr = mystr.encode(_encodings['stdio'], 'backslashreplace')
-		sys.stderr.write(mystr)
-		sys.stderr.flush()
+
+		# avoid potential UnicodeEncodeError
+		mystr = _unicode_encode(mystr,
+			encoding=_encodings['stdio'], errors='backslashreplace')
+		f = sys.stderr
+		if sys.hexversion >= 0x3000000:
+			f = f.buffer
+		f.write(mystr)
+		f.flush()
 
 default_xterm_title = None
 
@@ -266,7 +270,10 @@ def xtermTitleReset():
 		if prompt_command == "":
 			default_xterm_title = ""
 		elif prompt_command is not None:
-			if dotitles and "TERM" in os.environ and sys.stderr.isatty():
+			if dotitles and \
+				'TERM' in os.environ and \
+				_legal_terms_re.match(os.environ['TERM']) is not None and \
+				sys.stderr.isatty():
 				from portage.process import find_binary, spawn
 				shell = os.environ.get("SHELL")
 				if not shell or not os.access(shell, os.EX_OK):
@@ -374,11 +381,12 @@ class ConsoleStyleFile(object):
 			self._write(self.write_listener, s)
 
 	def _write(self, f, s):
-		if sys.hexversion < 0x3000000 and \
-			isinstance(s, unicode) and \
-			f in (sys.stdout, sys.stderr):
-			# avoid potential UnicodeEncodeError
-			s = s.encode(_encodings['stdio'], 'backslashreplace')
+		# avoid potential UnicodeEncodeError
+		if f in (sys.stdout, sys.stderr):
+			s = _unicode_encode(s,
+				encoding=_encodings['stdio'], errors='backslashreplace')
+			if sys.hexversion >= 0x3000000:
+				f = f.buffer
 		f.write(s)
 
 	def writelines(self, lines):
@@ -484,9 +492,11 @@ class EOutput(object):
 		sys.stderr.flush()
 
 	def _write(self, f, s):
-		if sys.hexversion < 0x3000000 and isinstance(s, unicode):
-			# avoid potential UnicodeEncodeError
-			s = s.encode(_encodings['stdio'], 'backslashreplace')
+		# avoid potential UnicodeEncodeError
+		s = _unicode_encode(s,
+			encoding=_encodings['stdio'], errors='backslashreplace')
+		if sys.hexversion >= 0x3000000:
+			f = f.buffer
 		f.write(s)
 		f.flush()
 
