@@ -1,12 +1,17 @@
 # tests/__init__.py -- Portage Unit Test functionality
 # Copyright 2006 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: __init__.py 14421 2009-09-25 05:29:02Z arfrever $
+# $Id: __init__.py 15006 2009-12-10 01:04:18Z zmedico $
 
 import sys
 import time
 import unittest
 
+try:
+	from unittest.runner import _TextTestResult # new in python-2.7
+except ImportError:
+	from unittest import _TextTestResult
+
 from portage import os
 from portage import _encodings
 from portage import _unicode_encode
@@ -69,7 +74,7 @@ def getTests(path, base_path):
 		result.append(unittest.TestLoader().loadTestsFromModule(mod))
 	return result
 
-class TextTestResult(unittest._TextTestResult):
+class TextTestResult(_TextTestResult):
 	"""
 	We need a subclass of unittest._TextTestResult to handle tests with TODO
 
@@ -77,9 +82,9 @@ class TextTestResult(unittest._TextTestResult):
 	that are marked TODO; these can be displayed later
 	by the test runner.
 	"""
-	
+
 	def __init__(self, stream, descriptions, verbosity):
-		unittest._TextTestResult.__init__( self, stream, descriptions, verbosity )
+		super(TextTestResult, self).__init__(stream, descriptions, verbosity)
 		self.todoed = []
 
 	def addTodo(self, test, info):
@@ -88,14 +93,14 @@ class TextTestResult(unittest._TextTestResult):
 			self.stream.writeln("TODO")
 		elif self.dots:
 			self.stream.write(".")
-	
+
 	def printErrors(self):
 		if self.dots or self.showAll:
 			self.stream.writeln()
 			self.printErrorList('ERROR', self.errors)
 			self.printErrorList('FAIL', self.failures)
 			self.printErrorList('TODO', self.todoed)
-	
+
 class TestCase(unittest.TestCase):
 	"""
 	We need a way to mark a unit test as "ok to fail"
