@@ -1,9 +1,9 @@
 # test_PackageUseFile.py -- Portage Unit Testing Functionality
 # Copyright 2007 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: test_PackageUseFile.py 6182 2007-03-06 07:35:22Z antarus $
+# $Id: test_PackageUseFile.py 14327 2009-09-21 16:07:07Z arfrever $
 
-import os
+from portage import os
 from portage.tests import TestCase
 from portage.env.config import PackageUseFile
 from tempfile import mkstemp
@@ -22,7 +22,7 @@ class PackageUseFileTestCase(TestCase):
 		try:
 			f = PackageUseFile(self.fname)
 			f.load()
-			for cpv, use in f.iteritems():
+			for cpv, use in f.items():
 				self.assertEqual( cpv, self.cpv )
 				[flag for flag in use if self.assertTrue(flag in self.useflags)]
 		finally:
@@ -30,10 +30,9 @@ class PackageUseFileTestCase(TestCase):
 
 	def BuildFile(self):
 		fd, self.fname = mkstemp()
-		f = os.fdopen(fd, 'wb')
+		f = os.fdopen(fd, 'w')
 		f.write("%s %s" % (self.cpv, ' '.join(self.useflags)))
 		f.close()
 	
 	def NukeFile(self):
-		import os
 		os.unlink(self.fname)
