@@ -1,14 +1,13 @@
 # Copyright 1998-2008 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: test_spawn.py 8474 2007-11-09 03:35:38Z zmedico $
+# $Id: test_poll.py 14078 2009-08-17 22:18:02Z zmedico $
 
-import errno, os, sys
-import fcntl
-import termios
-import portage
-from portage.output import get_term_size, set_term_size
+import sys
+from portage import os
 from portage.tests import TestCase
-from _emerge import PipeReader, SpawnProcess, TaskScheduler
+from _emerge.TaskScheduler import TaskScheduler
+from _emerge.PipeReader import PipeReader
+from _emerge.SpawnProcess import SpawnProcess
 
 class PipeReaderTestCase(TestCase):
 
@@ -28,12 +27,18 @@ class PipeReaderTestCase(TestCase):
 		test_string = 2 * "blah blah blah\n"
 
 		master_fd, slave_fd = self._create_pipe()
-		master_file = os.fdopen(master_fd, 'r')
+		master_file = os.fdopen(master_fd, 'rb')
 
 		task_scheduler = TaskScheduler(max_jobs=2)
 		scheduler = task_scheduler.sched_iface
 
-		producer = SpawnProcess(
+		class Producer(SpawnProcess):
+			def _spawn(self, args, **kwargs):
+				rval = SpawnProcess._spawn(self, args, **kwargs)
+				os.close(kwargs['fd_pipes'][1])
+				return rval
+
+		producer = Producer(
 			args=["bash", "-c", "echo -n '%s'" % test_string],
 			fd_pipes={1:slave_fd}, scheduler=scheduler)
 
@@ -44,10 +49,9 @@ class PipeReaderTestCase(TestCase):
 		task_scheduler.add(producer)
 		task_scheduler.add(consumer)
 
-		def producer_start_cb(task):
-			os.close(slave_fd)
-
-		producer.addStartListener(producer_start_cb)
 		task_scheduler.run()
 
+		if sys.hexversion >= 0x3000000:
+			test_string = test_string.encode()
+
 		self._assertEqual(test_string, consumer.getvalue())
