@@ -1,10 +1,11 @@
 # Copyright 1999-2009 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: update.py 14418 2009-09-25 03:29:54Z zmedico $
+# $Id: update.py 15139 2009-12-24 04:01:35Z zmedico $
 
 import codecs
 import errno
 import re
+import stat
 import sys
 
 from portage import os
@@ -63,15 +64,19 @@ def update_dbentries(update_iter, mydata):
 		k_unicode = _unicode_decode(k,
 			encoding=_encodings['repo.content'], errors='replace')
 		if k_unicode not in ignored_dbentries:
+			orig_content = mycontent
 			mycontent = _unicode_decode(mycontent,
 				encoding=_encodings['repo.content'], errors='replace')
+			is_encoded = mycontent is not orig_content
 			orig_content = mycontent
 			for update_cmd in update_iter:
 				mycontent = update_dbentry(update_cmd, mycontent)
 			if mycontent != orig_content:
-				updated_items[k] = _unicode_encode(mycontent,
-					encoding=_encodings['repo.content'],
-					errors='backslashreplace')
+				if is_encoded:
+					mycontent = _unicode_encode(mycontent,
+						encoding=_encodings['repo.content'],
+						errors='backslashreplace')
+				updated_items[k] = mycontent
 	return updated_items
 
 def fixdbentries(update_iter, dbdir):
@@ -118,7 +123,7 @@ def grab_updates(updpath, prev_mtimes=None):
 		file_path = os.path.join(updpath, myfile)
 		mystat = os.stat(file_path)
 		if file_path not in prev_mtimes or \
-		long(prev_mtimes[file_path]) != long(mystat.st_mtime):
+		long(prev_mtimes[file_path]) != mystat[stat.ST_MTIME]:
 			content = codecs.open(_unicode_encode(file_path,
 				encoding=_encodings['fs'], errors='strict'),
 				mode='r', encoding=_encodings['repo.content'], errors='replace'
@@ -197,15 +202,20 @@ def update_config_files(config_root, protect, protect_mask, update_iter):
 		config_file = os.path.join(abs_user_config, x)
 		if os.path.isdir(config_file):
 			for parent, dirs, files in os.walk(config_file):
-				for y in dirs:
+				try:
+					parent = _unicode_decode(parent,
+						encoding=_encodings['fs'], errors='strict')
+				except UnicodeDecodeError:
+					continue
+				for y_enc in list(dirs):
 					try:
-						y = _unicode_decode(y,
+						y = _unicode_decode(y_enc,
 							encoding=_encodings['fs'], errors='strict')
 					except UnicodeDecodeError:
-						dirs.remove(y)
+						dirs.remove(y_enc)
 						continue
 					if y.startswith("."):
-						dirs.remove(y)
+						dirs.remove(y_enc)
 				for y in files:
 					try:
 						y = _unicode_decode(y,
