@@ -1,9 +1,16 @@
 # versions.py -- core Portage functionality
 # Copyright 1998-2006 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: versions.py 14494 2009-10-05 00:18:46Z zmedico $
+# $Id: versions.py 15631 2010-03-02 20:47:57Z zmedico $
+
+__all__ = [
+	'best', 'catpkgsplit', 'catsplit',
+	'cpv_getkey', 'pkgcmp',  'pkgsplit',
+	'ververify', 'vercmp'
+]
 
 import re
+import warnings
 
 
 # \w is [a-zA-Z0-9_]
@@ -100,9 +107,10 @@ def vercmp(ver1, ver2, silent=1):
 	list2 = [int(match2.group(2))]
 
 	# this part would greatly benefit from a fixed-length version pattern
-	if len(match1.group(3)) or len(match2.group(3)):
+	if match1.group(3) or match2.group(3):
 		vlist1 = match1.group(3)[1:].split(".")
 		vlist2 = match2.group(3)[1:].split(".")
+
 		for i in range(0, max(len(vlist1), len(vlist2))):
 			# Implcit .0 is given a value of -1, so that 1.0.0 > 1.0, since it
 			# would be ambiguous if two versions that aren't literally equal
@@ -132,6 +140,11 @@ def vercmp(ver1, ver2, silent=1):
 				list2.append(int(vlist2[i].ljust(max_len, "0")))
 
 	# and now the final letter
+	# NOTE: Behavior changed in r2309 (between portage-2.0.x and portage-2.1).
+	# The new behavior is 12.2.5 > 12.2b which, depending on how you look at,
+	# may seem counter-intuitive. However, if you really think about it, it
+	# seems like it's probably safe to assume that this is the behavior that
+	# is intended by anyone who would use versions such as these.
 	if len(match1.group(5)):
 		list1.append(ord(match1.group(5)))
 	if len(match2.group(5)):
@@ -303,6 +316,26 @@ def pkgsplit(mypkg, silent=1):
 	else:
 		return (cat + '/' + pn, ver, rev)
 
+def cpv_getkey(mycpv):
+	"""Calls catpkgsplit on a cpv and returns only the cp."""
+	mysplit = catpkgsplit(mycpv)
+	if mysplit is not None:
+		return mysplit[0] + '/' + mysplit[1]
+
+	warnings.warn("portage.versions.cpv_getkey() " + \
+		"called with invalid cpv: '%s'" % (mycpv,),
+		DeprecationWarning, stacklevel=2)
+
+	myslash = mycpv.split("/", 1)
+	mysplit = _pkgsplit(myslash[-1])
+	if mysplit is None:
+		return None
+	mylen = len(myslash)
+	if mylen == 2:
+		return myslash[0] + "/" + mysplit[0]
+	else:
+		return mysplit[0]
+
 def catsplit(mydep):
         return mydep.split("/", 1)
 
