@@ -1,7 +1,7 @@
 # repoman: Error Messages
 # Copyright 2007 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: errors.py 13401 2009-04-26 21:14:23Z zmedico $
+# $Id: errors.py 15261 2010-01-29 18:51:48Z zmedico $
 
 COPYRIGHT_ERROR = 'Invalid Gentoo Copyright on line: %d'
 LICENSE_ERROR = 'Invalid Gentoo/GPL License on line: %d'
@@ -17,3 +17,5 @@ EMAKE_PARALLEL_DISABLED = 'Upstream parallel compilation bug (ebuild calls emake
 EMAKE_PARALLEL_DISABLED_VIA_MAKEOPTS = 'Upstream parallel compilation bug (MAKEOPTS=-j1 on line: %d)'
 DEPRECATED_BINDNOW_FLAGS = 'Deprecated bindnow-flags call on line: %d'
 EAPI_DEFINED_AFTER_INHERIT = 'EAPI defined after inherit on line: %d'
+NO_AS_NEEDED = 'Upstream asneeded linking bug (no-as-needed on line: %d)'
+BUILT_WITH_USE = 'built_with_use on line: %d'
